*-- Author :    Hartwig Albrecht   20/11/95

      SUBROUTINE CHDTOC (DARG, NACC, CSEP, COUT)

************************************************************************
*
* Description: convert double precision number to a character string
*
* Called from: anywhere
*
* External references: none
*
* Argument in/out description
*   DARG   in     double number
*   NACC   in     accuracy of the conversion
*                 N <= 0: N decimal digits after the decimal point;
*                 N > 0: at least N decimal digits in total.
*   CSEP   in     separator character (123 456.789 012 or
*                 123,456.789,,012 OR 123_456.789_012).
*                 Accepted: only blank space or comma or
*                 the underscore character.
*                 All other characters: no separation.
*   COUT   out    character string
*
************************************************************************
C
      IMPLICIT NONE

      INTEGER NACC, NA, LOOP, I, N, KE, KF, KP, KD, KS, KL, LENOCC
      DOUBLE PRECISION DARG, D, DR
      CHARACTER*(*) COUT, CSEP
      CHARACTER CDIG*1, CINT*32
      DIMENSION CDIG(0:9), DR(0:16)

      DATA CDIG /'0','1','2','3','4','5','6','7','8','9'/
      DATA DR /.5000000000000001D0,.5000000000000001D0,
     + .050000000000001D0,.005000000000001D0,.000500000000001D0,
     + .000050000000001D0,.000005000000001D0,.000000500000001D0,
     + .000000050000001D0,.000000005000001D0,.000000000500001D0,
     + .000000000050001D0,.000000000005001D0,.000000000000501D0,
     + .000000000000051D0,.000000000000005D0,.000000000000000D0/
      DATA KD /0/

      IF (LEN(COUT) .LE. 2 .AND. DARG .GT. 0.D0)  THEN
        COUT = '**'
        GO TO 110
      END IF

      IF (LEN(COUT) .LE. 3 .AND. DARG .LT. 0.D0)  THEN
        COUT = '***'
        GO TO 110
      END IF

      IF (DARG .EQ. 0D0)  THEN
        COUT = '0.'
        GO TO 110
      END IF

      LOOP = 0
      NA = NACC

 1    KS = 0
      IF (CSEP .EQ. ' ' .OR. CSEP .EQ. '_' .OR. CSEP .EQ. ',')  KS = 1

      IF (DARG .GT. 0D0)  THEN
        KL = 2
        N = 0
        CINT = ' '
        D = DARG
      ELSE
        KL = 3
        N = 1
        CINT = '-'
        D = - DARG
      END IF
C
      IF (D .GE. 1D12 .OR. D .LT. 1.D-12)  THEN
        KE = DLOG10 (D)
        IF (KE .LT. 0)  KE = KE - 1
        D = D * 1.D1 ** (-KE)
      ELSE IF (D .GE. 1D0)  THEN
        IF (D .LT. 1D2)  THEN
          IF (D .LT. 1D1)  THEN
            KE = 0
          ELSE
            KE = 1
            D = D * 1D-1
          END IF
        ELSE IF (D .LT. 1D4)  THEN
          IF (D .LT. 1D3)  THEN
            KE = 2
            D = D * 1D-2
          ELSE
            KE = 3
            D = D * 1D-3
          END IF
        ELSE IF (D .LT. 1D6)  THEN
          IF (D .LT. 1D5)  THEN
            KE = 4
            D = D * 1D-4
          ELSE
            KE = 5
            D = D * 1D-5
          END IF
        ELSE IF (D .LT. 1D8)  THEN
          IF (D .LT. 1D7)  THEN
            KE = 6
            D = D * 1D-6
          ELSE
            KE = 7
            D = D * 1D-7
          END IF
        ELSE IF (D .LT. 1D10)  THEN
          IF (D .LT. 1D9)  THEN
            KE = 8
            D = D * 1D-8
          ELSE
            KE = 9
            D = D * 1D-9
          END IF
        ELSE
          IF (D .LT. 1D11)  THEN
            KE = 10
            D = D * 1D-10
          ELSE
            KE = 11
            D = D * 1D-11
          END IF
        END IF
      ELSE IF (D .GE. 1D-2)  THEN
        IF (D .GE. 1D-1)  THEN
          KE = -1
          D = D * 1D1
        ELSE
          KE = -2
          D = D * 1D2
        END IF
      ELSE IF (D .GE. 1D-4)  THEN
        IF (D .GE. 1D-3)  THEN
          KE = -3
          D = D * 1D3
        ELSE
          KE = -4
          D = D * 1D4
        END IF
      ELSE IF (D .GE. 1D-6)  THEN
        IF (D .GE. 1D-5)  THEN
          KE = -5
          D = D * 1D5
        ELSE
          KE = -6
          D = D * 1D6
        END IF
      ELSE IF (D .GE. 1D-8)  THEN
        IF (D .GE. 1D-7)  THEN
          KE = -7
          D = D * 1D7
        ELSE
          KE = -8
          D = D * 1D8
        END IF
      ELSE IF (D .GE. 1D-10)  THEN
        IF (D .GE. 1D-9)  THEN
          KE = -9
          D = D * 1D9
        ELSE
          KE = -10
          D = D * 1D10
        END IF
      ELSE
        IF (D .GE. 1D-11)  THEN
          KE = -11
          D = D * 1D11
        ELSE
          KE = -12
          D = D * 1D12
        END IF
      END IF

      IF (NA .GT. 0)  GO TO 20
      KP = KE + 1
      KD = KP - NA
      IF (KD .LT. 0)  GO TO 80
      IF (KD .GT. 16)  THEN
        KD = 16
        GO TO 30
      END IF

 10   KE = 0
      D = D + DR(KD)
      IF (D .GE. 1D1)  THEN
        D = D * 1D-1
        KP = KP + 1
        KD = KD + 1
      END IF
      IF (KP .EQ. 0 .AND. KD .EQ. 0)  KD = 1
      IF (KD .LE. 0)  GO TO 80
      GO TO 40

 20   KD = MIN (NA, 15)
      IF (KE. GE. -2 .AND. KE .LE. MIN (KD, 14))  THEN
        KP = KE + 1
        KD = MAX (KP, KD)
        GO TO 10
      END IF

 30   D = D + DR(KD)
      IF (D .GE. 1D1)  THEN
        D = D * 1D-1
        KE = KE + 1
      END IF
C
      KP = 1
      I = MOD (KE,3)
      IF (I .GT. 0)  THEN
        IF (I .EQ. 1)  THEN
          KE = KE - 1
          KP = KP + 1
        ELSE
          KE = KE - 2
          KP = KP + 2
        END IF
      ELSE IF (I .LT. 0)  THEN
        IF (I .EQ. -1)  THEN
          KE = KE + 1
          KP = KP - 1
        ELSE
          KE = KE - 1
          KP = KP + 1
        END IF
      END IF
C
 40   KF = 0
      IF (KP .GE. 0)  GO TO 60
      N = N + 1
      CINT(N:N) = '.'
      DO 50 I = KP, -1
        N = N + 1
        CINT(N:N) = CDIG(0)
        IF (MOD (KP-I, 3) .EQ. -2 .AND. KP .NE. I .AND.
     +      KS .GT. 0)  THEN
          N = N + 1
          CINT(N:N) = CSEP
        END IF
 50   CONTINUE

 60   IF (KP .EQ. 0)  THEN
        N = N + 1
        CINT(N:N) = '.'
      ELSE IF (MOD (KP, 3) .EQ. 0 .AND. KS .GT. 0 .AND.
     +    KD .GE. 1 .AND. KF. GT. 1)  THEN
        N = N + 1
        CINT(N:N) = CSEP
      END IF
      IF (KD .EQ. 0)  GO TO 70
      N = N + 1
      CINT(N:N) = CDIG(INT(D))
      D = DMOD (D * 1D1, 1D1)
      KP = KP - 1
      KD = KD - 1
      KF = KF + 1
      GO TO 60

 70   IF (CINT(N:N) .EQ. CDIG(0) .OR.
     +    CINT(N:N) .EQ. CSEP)  THEN
        IF (N .GT. KL)  THEN
          CINT(N:N) = ' '
          N = N - 1
          GO TO 70
        END IF
      END IF
      IF (CINT(N-1:N-1) .EQ. CSEP)  THEN
        CINT(N-1:N-1) = CINT(N:N)
        CINT(N:N) = ' '
        N = N - 1
      END IF

      IF (KE .EQ. 0)  GO TO 90
      N = N + 1
      CINT(N:N) = 'e'
      IF (KE .LT. 0)  THEN
        N = N + 1
        CINT(N:N) = '-'
        KE = - KE
      END IF
      IF (KE .GE. 10)  THEN
        N = N + 1
        CINT(N:N) = CDIG(KE/10)
        KE = MOD (KE, 10)
      END IF
      N = N + 1
      CINT(N:N) = CDIG(KE)
      GO TO 90

 80   IF (DARG .GT. 0.D0)  THEN
        COUT = '.00'
      ELSE
        COUT = '-.00'
      END IF
      GO TO 110

 90   IF (LENOCC (CINT) .GT. LEN(COUT))  THEN
        IF (LOOP .NE. 0)  THEN
          COUT = '***'
          GO TO 110
        END IF
        IF (NA .GT. 0)  THEN
          NA = -LEN(COUT) + 1
          IF (DARG .LT. 0.D0)  NA = NA + 1
          LOOP = 1
          GO TO 1
        END IF
      END IF

 100  COUT = CINT
 110  END

************************************************************************
* End of CHDTOC
************************************************************************
