//                              -*- Mode: C++ -*-
//:ECALDetector.C
//
// Author: Mauro Villa   
// ECAL group 
// HERA-B experiment, Desy
// e-mail: villa@bo.infn.it
//
// Created 1/10/97
// Last Update ........
//
//

#include "care/ECALDetector.hh"
#include "care/care.hh"
#include "care/ECBuilder.hh"
#include "care/EMClusterTree.hh"
#undef PERFORMANCES
#ifdef PERFORMANCES
#  include "arte/Chrono.hh"
#  include "arte/ChronoManager.hh"
#endif
#include "arte/ArteMessage.hh"
#include "care/caredata.hh"
#include "care/ELLUTsManager.hh"
#include "care/ECHitCellArray.hh"
#include "care/EDClusHypotModifier.hh"
#include "care/EDClusTraHypotModifier.hh"
#include "care/EDBoundCellModifier.hh"
#include "care/EAHypTraAnalyzer.hh"
#include "care/EAHypAnalyzer.hh"
#include "care/ECMCcalib.hh"
#include "care/CmdCareDump.hh"
#include "care/CmdCareLevel.hh"
#ifndef ONLINE
#include "care/CmdCareECDBMIZ.hh"
#include "table.h"
#endif
#include "care/MMBank.hh"

extern "C" void dgcalinit_();

const char VERSION[] = "3.50/00";

#ifdef PERFORMANCES

Chrono *  filling = 0;
Chrono *  init = 0;
Chrono *  clustering = 0;
Chrono *  merging = 0;
Chrono *  bankwrite = 0;
Chrono *  firststage = 0;
Chrono *  secondstage = 0;
Chrono *  matching = 0;
Chrono *  matching1 = 0;
Chrono *  extrap = 0;
Chrono *  matching2 = 0;
Chrono *  matching3 = 0;
Chrono *  splitting = 0;
Chrono *  bankupdate = 0;

#endif

static char stars[] = 
      "***********************************************************";

#undef WRITE


/**
class ECALDetector.
  Provide a collection of basic function for the ECAL detector 
  reconstruction and analysis.

*/

ECALDetector* ECALDetector::ecalDetector = 0;

ECALDetector::ECALDetector(){
  topClusterTree = new EMClusterTree();
  clusBuilder = new ECBuilder();
  pSignalCells = new ECHitCellArray();
  int* storage = new int[50000]; // reserve 4 mega
  evhd.evt = 0;
  evhd.run = 0;
  evhd.exp = 0;
  evhd.date = 0;
  evhd.time = 0;
  evhd.vfld = 0;
  delete [] storage;
}

int ECALDetector::initialize() {
  if(ecalDetector==0){

    if( !ECalInfo.init){
      //
      // Searching for Calorimeter informations in geometry tables
      //
      // Ecal Pointer to GESL table
      ArteTable<GESL>::iterator ecalp = ArteTable<GESL>::begin();
      for(; ecalp!=ArteTable<GESL>::end(); ++ecalp){
	if(!strncmp(ecalp->name,"CAL",3)){
	  // ECAL found. ( it could be CALL CALS CALW)
	  // Exit this loop
	  break;
	}
      }
      if( ecalp!=ArteTable<GESL>::end() ){

	ecalDetector = new ECALDetector();
	//
	// Notify calorimeter initialization
	//
	arteMsg<<stars;
	arteMsg.allp();
	arteMsg<<" Care  "<<VERSION;
	arteMsg.allp();
	arteMsg<<" Monte Carlo & Real Data Version ";
	arteMsg.allp();
	arteMsg<<stars;
	arteMsg.allp();
	
	//
	// Initialize the LUTs  
	ELLUTsManager::instance();  
	//
#       ifdef PERFORMANCES
          // Initialize the ChronoManager
          ChronoManager::instance();
#       endif
	//
	//dgcalinit_();
	//
	arteMsg<<stars;
	arteMsg.allp();  
	// 
	
	ECMCcalib::dumpCalib();    // printout calibration data
	ECHitCellArray::dumpCutOff();   // printout cutoff data
	//
	arteMsg<<" Merging Cut Off constants:";
	arteMsg.allp();
	arteMsg<<" Minimum energy: "<<ECalInfo.clusThr<<" GeV "
	       <<" Valley energy: "<<ECalInfo.valley<<" GeV";
	arteMsg.allp();
	arteMsg<<stars;
	arteMsg.allp();
	// 
	// set to true the initialization flag
	ECalInfo.init = true;
      } else {
	//
	//  ECAL not found: print a warning
	//
	arteMsg<<"GESL ECAL entry (CALx) not found: "
	       <<"Calorimeter reconstruction unavailable";
	arteMsg.warn();
      }
    } else {
      //
      //  careinit called more than once
      //
      arteMsg<<"More than one call to careinit_() :  check your code !";
      arteMsg.error();
    }
  }
  return ECalInfo.init? 0 : 1;
}

//--
/// Find and return shower hypotheses
const std::vector<const EMCluster*> & ECALDetector::findShowers(){
  EcalEvent_t * event=0;
  return findShowers(event);
}

//--
/// Find and return shower hypotheses
const std::vector<const EMCluster*> & ECALDetector::findShowers(EcalEvent_t * event){
  int rc =0;
# ifdef PERFORMANCES
  if(firststage == 0){
    ChronoManager CM;
    filling    = CM.createChrono("Cls: Filling");
    init       = CM.createChrono("Cls: Start");
    clustering = CM.createChrono("Cls: Clustering");
    merging    = CM.createChrono("Cls: Merging   ");
    bankwrite  = CM.createChrono("Cls:Bank Wrt");
    firststage = CM.createChrono("All First Stage");
  }
  firststage->start();
# endif
  /////////////
  // start testing if the GCAX table is OK
  if( ArtePointer<GCAL>(ECalInfo.ecalp->fpl)->fcl==0){
    // rebuild the table
    static int wgcal = 0xECA0002;
    arteMsg<<"CAREEVNT1: GCAL table wrong! ";
    arteMsg.warn(wgcal);
    ELLUTsManager::RebuildGCAL();
  }
#ifndef ONLINE
  ////////////////////// look for the first event
  if( evhd.evt==0){
     ArtePointer<EVHD> header(1);
     evhd.evt = header->evt;
     evhd.run = header->run;
     evhd.exp = header->exp;
     evhd.date = header->date;
     evhd.time = header->time;
     evhd.vfld = header->vfld;
     if(ECalInfo.dbflag==0 && evhd.exp>0) { // not MC data and no external IO
 
      CmdCareECDBMIZ dbmiz;
       char dbname[256];
       dbname[0] = 0;
       ECalInfo.dbflag=3;
       char *slbuf = getenv("HBECALDB");
       if( slbuf==0 ){
	 strcpy(dbname,"/ECAL_CALIB");
       } else {
	 strcpy(dbname,slbuf);
       }
       dbmiz.setDB(dbname, evhd.run, TABLEmaxrev);
       rc += dbmiz.readDB(CmdCareECDBMIZ::TALL, TABLEle);  
       if( rc!=0 ){
	 arteMsg<<"ECAL: Error reading DB. Fatal.";
         arteMsg.error();
       }
     }
   }
#endif
  //////////////////////////////////////////////////////////////////////////

# ifdef PERFORMANCES
  filling->start();
# endif
  pSignalCells->fill(event);  // Fills the array of pointers

# ifdef PERFORMANCES
  filling->stop();
# endif
  if( CmdCareDump::get(HFLAG) )
    pSignalCells->dump(); // for tests: Dump array 
# ifdef PERFORMANCES
  init->start();
# endif

  //
  // Loop on pSignalCells with the help of the builder
  //
  // initialize loop processing: sort in energy
  //
 
  clusBuilder->startClustering(*pSignalCells, *topClusterTree);

# ifdef PERFORMANCES
  init->stop();
  if( CmdCareDump::get(HFLAG) )
    pSignalCells->dump(); // for tests: Dump array 
  clustering->start();
# endif
  
  //
  // Clustering LOOP: loop on HITC cells
  //
  unsigned int cells = CmdCareLevel::ncells();
  // test if there are more cells to consider
  while( clusBuilder->more() && cells>0 ){    
    clusBuilder->doClusterize();  // Clusterize with current cell
    clusBuilder->next();       // consider next cell to clusterize
    cells--;
  };
  
# ifdef PERFORMANCES
  clustering->stop();
  merging->start();
# endif

  //
  // Analyze the Top Cluster List to find the number of hypothetical 
  // particle giving rise to a cluster
  //
  EDClusHypotModifier  clusHypMd;
  clusHypMd.doModify(topClusterTree);

# ifdef PERFORMANCES
  merging->stop();
  bankwrite->start();
# endif
  //
  // Now store the informations on shower hypotheses on the RCCL bank
  //
  EAHypAnalyzer hypAn(showerArray);
  hypAn.doAnalysis(topClusterTree);

  if( CmdCareDump::get(CFLAG) )
    topClusterTree->dump();

# ifdef PERFORMANCES
  bankwrite->stop();
  firststage->stop();
# endif

  return showerArray;
}

//--
/// Find and return all signals detected on ECAL
const std::vector<const EMCluster*> & ECALDetector::findAllSignals(){

# ifdef PERFORMANCES
  if(secondstage == 0){
    ChronoManager CM;
    secondstage = CM.createChrono("Second Stage");
    matching    = CM.createChrono("Care Match  ");    
    matching1   = CM.createChrono("Care Match1  ");    
    extrap      = CM.createChrono("Marple extrap");
    matching2   = CM.createChrono("Care Match2  ");    
    matching3   = CM.createChrono("Care Match3  ");
    splitting   = CM.createChrono("Splitting");
    bankupdate  = CM.createChrono("Bank update ");
  }
  secondstage->start();
  matching->start();
  matching1->start();
# endif

  //
  // Extrapolate charged tracks to the surface of the calorimeter
  // and find in which cluster the extrapolated point is.
  // Add to the cluster tree infos from tracks
  //

  clusBuilder->trackClusterMatch();
# ifdef WRITE
  arteMsg<<"After Track Cluster Match";
  arteMsg.allp();
# endif
#   ifdef PERFORMANCES
  matching1->stop();
  matching2->start();
# endif
  //
  // Analyze the Top Cluster List to find the number of hypothetical 
  // charged and neutral particles
  // In doing this the number of hypothetical particle can only increase and
  // can never decrease. At the end of the analysis the number of hypotetical
  // particles is given by the sum of charged and neutral clusters
  //

  EDClusTraHypotModifier  clusTraHypMd(clusBuilder);
  clusTraHypMd.doModify(topClusterTree);
  //
  //
# ifdef WRITE
  arteMsg<<"Cluster List Hypotheses: ";
  topClusterTree->getAllHypotheses().dump();
# endif
# ifdef PERFORMANCES
  matching2->stop();
  matching3->start();
# endif
  //
  // An attempt to recover those tracks whose extrapolation point does not
  // belong to any hit cell is done. 
  //
  clusBuilder->recoverUnmatchedTracks();
# ifdef PERFORMANCES
  matching3->stop();
  matching->stop();
# endif
# ifdef WRITE
  arteMsg<<"After Recover Unmatched Tracks";
  arteMsg.allp();
# endif

  //
  // At this point boundary region cells should be recovered
  // 
  if( CmdCareLevel::level()>3){
#   ifdef PERFORMANCES
    splitting->start();
#   endif
    /// split cells only at level 4 and 5
    EDBoundCellModifier  clusBoundCellMd(clusBuilder);
    clusBoundCellMd.doModify(topClusterTree);
#   ifdef PERFORMANCES
    splitting->stop();
#   endif
  }
  //
  //if( CmdCareDump::get(CTFLAG) )
  //    topClusterTree->dump();
  //
  // At this point all clusters are labelled as neutral, single charge
  // or many charges
  //
# ifdef PERFORMANCES
  bankupdate->start();
# endif
  EAHypTraAnalyzer hypTraAn(showerArray, signalArray);
  hypTraAn.analyzeTopList(*topClusterTree);

# ifdef PERFORMANCES
  bankupdate->stop();
  secondstage->stop();
# endif
 
# ifdef WRITE 
  arteMsg<<"Cluster List Hypotheses: ";
  //  topClusterTree->getAllHypotheses().dump();

  if(topClusterTree->getTracks() >= topClusterTree->getChargHyp() &&
     topClusterTree->getHypotheses()==
     topClusterTree->getNeuHyp()+topClusterTree->getChargHyp() ){
    arteMsg<<" GOOD TOP Cluster List";
    arteMsg.allp();
  } else{
    arteMsg<<" BAD TOP Cluster List";
    arteMsg.allp();
    if( !CmdCareDump::get(CTFLAG) )
      topClusterTree->dump();
  }
# endif

  if( CmdCareDump::get(CTFLAG) ){
    topClusterTree->dump();
  }
  return signalArray;
} // find All Signals


//
// Store neutrals into the RTRA bank
int ECALDetector::storeNeutrals(){
  int rc = 0;
# ifndef ONLINE
  if( ArtePointer<RCCL>::getLink()()!=0){
    for(ArteTable<RCCL>::iterator rccl = ArteTable<RCCL>::begin(); 
	rccl!=ArteTable<RCCL>::end(); ++rccl){
//      ArtePointer<RCLX> rclx(rccl);
      // Neutral clusters are defined as those without segments connected
      if(  rccl->rseg.begin() == rccl->rseg.end() ){ // test  for connected segments
	// get a table entry
	MMBank<RTRA>::Book(1);
	ArtePointer<RTRA> rtra(ArtePointer<RTRA>::nrow());
	// Now copy all the relevant informations
	rtra->zf = rccl->z;
	rtra->xf = rccl->x;
	rtra->yf = rccl->y;
	rtra->txf = rccl->tx;
	rtra->tyf = rccl->ty;
	rtra->pf = 1./rccl->esum;
	for(int i=0; i<14; i++){
	  rtra->cvf[i] = rccl->cve[i];
	}
	float tmp = rccl->esum*rccl->esum;
	rtra->cvf[14] = rccl->cve[14]/(tmp*tmp);
	rtra->hca = (int) (rccl->wid);
	rtra->fit = 6;      // set fit flag

	// Now I've finished filling RTRA fields. 
	// Let's begin with Relations: RCCL, HITC, MTRA, (RSEG)
	
	// main relation to RCCL
	ArtePointer<RCCL> rcl(rccl);
	createRelation(rtra, rcl, &RCCL::rtra);

	// Relation to HITC
	std::vector<ArtePointer<HITC> > vhitc;
	for(ArteRelationIterator<HITC> hitc(rccl->hitc.begin()); 
	    hitc != rccl->hitc.end(); hitc++ ){
	  ArtePointer<HITC> hit(hitc);
	  vhitc.push_back(hit);
	}
	for(std::vector<ArtePointer<HITC> >::reverse_iterator iter =
                                             vhitc.rbegin();
	    iter!=vhitc.rend(); iter++){
	  createRelation(rtra, *iter, &HITC::rtra);
	}

	// Relation to MTRA
	std::vector<ArtePointer<MTRA> > vmtra;
	for(ArteRelationIterator<MTRA> mtra(rccl->mtra.begin()); 
	    mtra != rccl->mtra.end(); mtra++ ){
	  ArtePointer<MTRA> mct(mtra);
	  vmtra.push_back(mct);
	}
	for(std::vector<ArtePointer<MTRA> >::reverse_iterator iter =
                                                    vmtra.rbegin();
	    iter!=vmtra.rend(); iter++){
	  createRelation(rtra, *iter, &MTRA::rtra);
	}
	
	// Relation to RSEG not set since
	// the DEFINITION of a neutral track is RSEG relations=0!
	
      }
    }
  }
# endif
  return rc;
}



int ECALDetector::careevnt1() {
  int rc = 0;
  if( !ECalInfo.init){
    //
    //  careevnt1 called with uninitialized reconstruction
    //
#   if 0
    arteMsg<<"Careevnt1() called without prior initialization:"
	   <<" check your code !";
    arteMsg.error();
#   endif
    // 14/1/99 Mods to remove the initialization in usinit
    rc = 1;
  }
  instance()->findShowers(0);
  return rc;
}

int ECALDetector::careevnt2() {
  int rc = 0;
  if( !ECalInfo.init){
    //
    //  careevnt2 called with uninitialized reconstruction
    //
    arteMsg<<"Careevnt2_() called without prior initialization:"
	   <<" check your code !";
    arteMsg.error();
    rc = 1;
  }
  instance()->findAllSignals();
  return rc;
}

//--
/// Third step of ECAL reconstruction. 
int ECALDetector::careevnt3(){
  return 0;
}; // No action for the moment !!!  


int ECALDetector::stop() {
  int rc = 0;
  if( ECalInfo.init ){
    //
    //
#   ifdef PERFORMANCES
      ChronoManager CM;
      CM.printInformations();
#   endif
    arteMsg<<stars;
    arteMsg.allp();
    arteMsg<<" Stop of Care  "<<VERSION;
    arteMsg.allp();
    arteMsg<<stars;
    arteMsg.allp();
    // set to false the initialization flag
    ECalInfo.init = false;
  } else {
    //
    //  carestop called when the calorimeter was not initialized
    //
    arteMsg<<"STOP for CARE called without a previous initialization call";
    arteMsg.allp();
    rc = 1;
  }
  return rc;
}

//--
/// Store parameters of a track
void  ECALDetector::storeTrack(TrackInfo & trackInfo){
  instance()->topClusterTree->newTrack(trackInfo);
}

//--
/// get the information for the new DB data
int ECALDetector::newDBData(int run) {
  int rc = 0;
#ifndef ONLINE
  CmdCareECDBMIZ dbmiz;
  char dbname[256];
  dbname[0] = 0;
  char *slbuf = getenv("HBECALDB");
  if( slbuf==0 ){
    strcpy(dbname,"/ECAL_CALIB");
  }
  strcpy(dbname,slbuf);
  if( run==0 ){
    if( ArteTable<EVHD>::size()>0){
      run = (*ArteTable<EVHD>::begin())->run;
    } else {
      run = TABLEmaxrev;
    }
  }
  
  dbmiz.setDB(dbname, run, TABLEmaxrev);
  if( ArteTable<ECBO>::size()==0 ){
    rc += dbmiz.readDB(CmdCareECDBMIZ::TALL, TABLEle);
  } else {
    rc += dbmiz.readDB(CmdCareECDBMIZ::TFAST, TABLEle);
  }
  if( rc==0 ){
    ECalInfo.dbflag=3;
    if( ecalDetector!=0 && ecalDetector->pSignalCells!=0){
      ecalDetector->pSignalCells->initThresholds();
    }
  } else {
    arteMsg<<"ECAL: error reading DB";
    arteMsg.warn();
  }
#endif

  return rc;
}

//--
/// get the information for the new DB data (all info given)
int ECALDetector::newDBData(const char * dbname, int *major, int *minor) {
  int rc = 0;

#ifndef ONLINE
  CmdCareECDBMIZ dbmiz;
  for(int i=0; i<5; i++){
    if( major[i]!=-1 ){
      dbmiz.setDB(dbname, major[i], minor[i]);
      rc += 
	dbmiz.readDB((CmdCareECDBMIZ::Tbset)(i+CmdCareECDBMIZ::TECBO),
		     TABLEeq);
    }
  }

  rc += dbmiz.makeECHF();
  
  if( rc==0){
    ECalInfo.dbflag=3;
    if( ecalDetector!=0 && ecalDetector->pSignalCells!=0){
      ecalDetector->pSignalCells->initThresholds();
    }
  } else {
    arteMsg<<"ECAL: error reading DB";
    arteMsg.warn();
    rc = 1;
  }
#endif

  return rc;
}
