//                              -*- Mode: C++ -*-
//:ELCellGrid.C
//
//
// Author: Mauro Villa   
// ECAL group 
// HERA-B experiment, Desy
// e-mail: villa@bo.infn.it
//
// Created 9/4/97
// Last Update ........
//

#include "arte/ArteMessage.hh"
#include "care/care.hh"
#include "care/ELCellGrid.hh" 
#include "care/ECALtypes.hh"
#include <stdio.h>
#include "care/caredata.hh"
#include "care/ECCellID.hh"
#include "care/MMBank.hh"
#undef DEBUG
/*
CLASS
  ELCellGrid
  
*/


// ---------------- ELCellGrid class definitions

//--
//  Detail of constructions left to LUTsManager 

ELCellGrid::ELCellGrid(){}

//--
//  Get the index of the first cell in a module
CellIdx_t  ELCellGrid::lookUp(int ix, int iy){
  ArtePointer<GCGD> cgd(ix+iy*ECalInfo.nModX+1);
# ifdef DEBUG
  std::cout<<"CG lookup: ix="<<ix<<" iy="<<iy<<" cell"<< cgd->cidx<<std::endl;
# endif
  return (CellIdx_t) (cgd->cidx);
};

//--
//  Set the index of the first cell in a module
void ELCellGrid::set(int ix, int iy, CellIdx_t ncell){
  ArtePointer<GCGD> cgd(ix+iy*ECalInfo.nModX+1);
#ifdef DEBUG
  if( cgd.index()<1 || cgd.index()>2352){
    std::cout<<"error in set: ix="<<ix<<", iy="<<iy<<" ncell="<<ncell<<std::endl;
  }
#endif
  cgd->cidx = ncell.index();
};

//--
//  Get the index of the first cell in a module
CellIdx_t  ELCellGrid::lookUp(int ix, int iy) const{
  ArtePointer<GCGD> cgd(ix+iy*ECalInfo.nModX+1);
#ifdef DEBUG
  std::cout<<"CG lookup: ix="<<ix<<" iy="<<iy<<" cell"<< cgd->cidx<<std::endl;
#endif
  return (CellIdx_t) (cgd->cidx);
};


//--
//  reserve space for entries
void ELCellGrid::reserve(int size){
# ifndef ONLINE
  if( ArtePointer<GCGD>::getLink().getRef()!=0)
    MMBank<GCGD>::Drop();
# endif
  GCGD cgd = {0};
  for(int i=0; i<size; i++){
    ArtePointer<GCGD>::append(cgd); 
  }
# ifdef DEBUG
  arteMsg<<" * CellGrid           LUT filled with "
	 <<ArtePointer<GCGD>::nrow()<<" elements of "<< size;
  arteMsg.allp();
# endif
}

//--
// Find IX and IY of the module

CellIdx_t ELCellGrid::findIXIY(float x, float y, int & ixm, int & iym, 
			     ErrorTypes & ierr) const {
  ierr = OK; // assume a valid module will be found
  CellIdx_t firstcell = 0; 
  if( x>=ECalInfo.x0 && y>=ECalInfo.y0 ){
    ixm = (int)((x-ECalInfo.x0)/ECalInfo.xMsize);
    iym = (int)((y-ECalInfo.y0)/ECalInfo.yMsize);
    
    if( ixm>=0 && ixm<ECalInfo.nModX && iym>=0 && iym<ECalInfo.nModY ){
      firstcell = lookUp(ixm,iym);
      if( firstcell.index()==0 )
	ierr = HOLE; // point in holes of ecal
    } else 
      ierr = OUTECAL; // point outside ECal
  } else
    ierr = OUTECAL; // point outside ECal
  return firstcell;
}



//--
//  Get the module number in which a point on ECAL is
ArtePointer<GCAL> ELCellGrid::module(float x, float y, 
				     ErrorTypes & ierr) const {
  ArtePointer<GCAL> modul = 0; 
  int ix,iy;
  CellIdx_t firstcell = findIXIY(x,y,ix,iy,ierr);
  if( ierr==OK ){ // a valid module has been found
    ELLUTsManager LUTs;
    const GCCE & info = LUTs.cellsDesc(firstcell);
    modul = ECCellID(info.ceid).modulePointer();
  }
  return modul;
}


//--
//  Get the cell index in which a point on ECAL is
CellIdx_t ELCellGrid::cellIdx(float x, float y, ErrorTypes & ierr) const{
  int ix,iy;
  CellIdx_t cell = cellIdxBase(x,y,ix,iy,ierr);
  if( ierr==OK ){
    ELLUTsManager LUTs;
    const GCCE & info = LUTs.cellsDesc(cell);
    ArtePointer<GCAL> modul = ECCellID(info.ceid).modulePointer();
    cell = cell.index()+(ix-1)*modul->ncl+(iy-1);
  }
  return cell;
}


//--
//  Get the cell index in which a point on ECAL is
ECCellID  ELCellGrid::cellIDen(float x, float y, ErrorTypes & ierr) const{
  ECCellID cell; 
  int ix,iy;
  CellIdx_t cellix = cellIdxBase(x,y,ix,iy,ierr);
  if( ierr==OK){
    ELLUTsManager LUTs;
    const GCCE & info = LUTs.cellsDesc(cellix);
    ArtePointer<GCAL> modul = ECCellID(info.ceid).modulePointer();
    cell(modul,ix,iy);
  }
  return cell;
}

//--
//  Get the cell index in which a point on ECAL is
CellIdx_t ELCellGrid::cellIdxBase(float x, float y, int & ix, int & iy,
				ErrorTypes & ierr) const{
  //  CellIdx_t cell = -1;
  int count = 0;

  CellIdx_t firstcell = 0;
  do{
    firstcell = findIXIY(x,y,ix,iy,ierr);
    if( ierr==OK ){ 
#     ifdef DEBUG
      std::cout<<"Firstcell from findIXIY "<<firstcell
	  <<" id="<<firstcell->ceid<<std::endl;
#     endif
      // a valid module has been found. Get its index
      ELLUTsManager LUTs;
      const GCCE & info = LUTs.cellsDesc(firstcell);
      ArtePointer<GCAL> modul = ECCellID(info.ceid).modulePointer();
      Coord_t widthx = ECalInfo.xMsize/modul->ncl;
      Coord_t widthy = ECalInfo.yMsize/modul->ncl;
      ix = (int)((x-modul->xmin)/widthx + 1.);
      iy = (int)((y-modul->ymin)/widthy + 1.);
      if(ix<1){
	ierr = AGAIN; // error on IX
	x -= widthx*0.1;
      }
      if(ix>modul->ncl){
	ierr = AGAIN; // error on IX
	x += widthx*0.1;
      }
      if(iy<1){
	ierr = AGAIN; // error on IY
	y -= widthy*0.1;
      }
      if(iy>modul->ncl){
	ierr = AGAIN;  // error on IY
	y += widthy*0.1;
      }
     
      count++;
      if(count>10){
	//	ArteMessage mess;
	static int ELCellGridWarn = 0x0ECA0002;
	//char buffer[80];
	arteMsg<<"ELCellGrid::CellIdxBase: 10 errors accessing ELCellGrid table";
	arteMsg.warn(ELCellGridWarn);
	arteMsg<<" ix="<<ix<<"  iy="<<iy<<"  ncl= "<<modul->ncl;
	arteMsg.allp();
	arteMsg<<" x ="<<x<<" y="<<y
	       <<"  xmin="<<modul->xmin<<" ymin= "<<modul->ymin
	       <<" widthx="<<widthx<<" widthy="<<widthy;
	arteMsg.allp();

	ierr = OUTECAL;
       }
    }
  } while( ierr==AGAIN);
  return firstcell;
}
