//                              -*- Mode: C++ -*-
//:ELNearHitCells.C
//
// Author: Mauro Villa   
// ECAL group 
// HERA-B experiment, Desy
// e-mail: villa@bo.infn.it
//
// Created 27/5/97
/* $Log: ELNearHitCells.C,v $
/* Revision 1.9  2000/03/14 06:04:54  natasha
/* Cell Neighbours on crux and square regions
/*
 * Revision 1.8  2000/01/11 11:24:44  natasha
 * care (3.50):
 * The major changes with respect to the previous release are:
 * 1) Digi MC inizialization is skipped for real data.
 * 2) the care/dglev command has been removed in favour of the hbdigi/dglev
 * 3) MC calibration constants retuned on MC data with magnet ON
 * 4) Cluster z correction retuned
 * 5) More flexibility for cluster threshold settings (ECCutFunction.C)
 * 6) The most important warning messages are counted
 * 7) No more chashes on missing DB
 * 8) Full C++ 9X standard conformance of the code
 *
 * Revision 1.7  1999/08/04 15:21:39  natasha
 * new care version from Mauro
 *
 * Revision 1.3  1999/06/01  16:07:41  villa
 * before HH
 *
 */
//

#include "care/ELNearHitCells.hh"
#include "care/EMClusterTree.hh"
#include "arte/arte.hh"
#undef WRITE

/*
CLASS 
  ELNearHitCells
  Get the list of nearby clusterized cells
  and provide access to the list

*/

///////////////// ELNearHitCells.C member function definitions ////////
//--
//  constructor
ELNearHitCells::ELNearHitCells() : 
   numNeib(0), curr(0) {}
//--
//  destructor
ELNearHitCells::~ELNearHitCells(){}


//--
//--
//  Get the neibours of a given cell
void ELNearHitCells::of(const EMHitCell* hcell, const EMClusterTree & tree,
	ELNearCells::NearRel cellrel){
  //--
  // find the index of the cell pointed by the current cell
  of(hcell->cellIdx(), tree, cellrel);
}

//--
//  Get neibouring cells of a given cell
void ELNearHitCells::of(CellIdx_t cell, const EMClusterTree & tree,
	ELNearCells::NearRel cellrel){

  //
  ELNearCells nearCells;
  nearCells.of(cell, cellrel);
  // for each neibouring cell find if it is linked with a cluster
  numNeib = 0;
  CellIdx_t nearCell; 
  EMHitCell * nearCluster;

# ifdef WRITE
  std::cout<<" Looking for neiboouring of cell "<< cell.index()<<std::endl;
# endif
  for(nearCells.begin(); nearCells.more(); nearCells.next()){
    nearCell = *nearCells;// get the index of the neibouring cell i
    
# ifdef WRITE
    std::cout<<" "<< nearCell.index();
# endif
    nearCluster = tree.whichHitCell(nearCell);
    if(nearCluster!=0){
       
# ifdef WRITE
     std::cout<<"Stored   ";
# endif
      // found a nearby cell already clusterized. 
      neibCells[numNeib] = nearCluster;
      numNeib++;
    }
  }
# ifdef WRITE
     std::cout<<std::endl;
# endif

  curr = 0;
}

