//**********************************************************************/
//
// Function for matching Care and Reconstructed SEGments
//
//**********************************************************************/

#include "marple/marple.hh"
#include "care/ECALDetector.hh"
#include "arte/ArteMessage.hh"
#include "arte/RSEG_cmp.hh"

std::vector<RcSegPoint* > RangSegList;
bool selectTrack( ArtePointer<RSEG> segm);

void matchRsegCare()
{
  if(RangSegList.size()>0){
    for(std::vector<RcSegPoint* >::iterator iter = RangSegList.begin();
      iter!=RangSegList.end(); iter++){
      delete *iter;
    }
    RangSegList.erase(RangSegList.begin(), RangSegList.end());		
  }

  // RANGER - Care matching
  // create list of tracks from Ranger
  if(ArtePointer<RSEG>::getLink()()==0) return;
 
  for( ArteTable<RSEG>::iterator irseg=ArteTable<RSEG>::begin(); 
       irseg!=ArteTable<RSEG>::end(); ++irseg ){
    
    if( selectTrack(*irseg)){

      if( (*irseg)->ze < 1350.0 ){ // main tracker track
	RangSegList.push_back( new RcSegPoint( *irseg, 1 ));
	RangSegList.back()->setty( (*irseg)->yf/(*irseg)->zf ); // since first
                                            //point is much more precise (O.I.)
      } else { // muon track
	RangSegList.push_back( new RcSegPoint(*irseg, 0 ));
      }
      RangSegList.back()->lineswim(1330.); // to speed up procedure 
    }
  }

  for(std::vector<RcSegPoint* >::iterator iter = RangSegList.begin();
      iter!=RangSegList.end(); iter++){
    // store the pointer to the track segment
    ECALDetector::storeTrack(**iter);
  }
  //arteMsg<<"Matching "<<RangSegList.size()<<" segments ";
  // arteMsg.allp();
}
  
// Selection function for segments to be matched
bool selectTrack( ArtePointer<RSEG> segm){
  bool selq = ( segm->fit == 0                   // good track 
	       && !(segm->cmp & Rsegc::bitsign)   // not killed
	       && (segm->cmp & (Rsegc::patt | Rsegc::trit |Rsegc::muon) )
	       // PC+TC+Muon track selection
	       // no downstream segment
	       && segm->ze > 730.0);            // behind magnet
  if( selq && (segm->cmp & Rsegc::muon) 
      && (segm->rbeg.begin()!=segm->rbeg.end()) && segm->zf > 1380.0) {
    // muon track with an initial segment: discard it, if previous segment
    // is not too far from calorimeter (I would like only TC tracks!)
    selq = selq && (*(segm->rbeg.begin()))->ze<950.0; 
  } else if( selq && (segm->cmp & Rsegc::patt) 
	     && (segm->rend.begin()!=segm->rend.end())){
    selq = selq && (*(segm->rend.begin()))->zf>1380.0;
  }
  
  return selq;
}
