      SUBROUTINE DGPTL( IPLANE )
***********************************************************************
*                                                                     *
* DDDDDDDDD      GGGGGGGGGG   PPPPPPPPPPP   TTTTTTTTTTTT  LL          *
* DDDDDDDDDD    GGGGGGGGGGGG  PPPPPPPPPPPP  TTTTTTTTTTTT  LL          *
* DD       DD   GG        GG  PP        PP       TT       LL          *
* DD        DD  GG            PP        PP       TT       LL          *
* DD        DD  GG            PP        PP       TT       LL          *
* DD        DD  GG            PPPPPPPPPPPP       TT       LL          *
* DD        DD  GG     GGGGG  PPPPPPPPPPP        TT       LL          *
* DD        DD  GG     GGGGG  PP                 TT       LL          *
* DD        DD  GG        GG  PP                 TT       LL          *
* DD       DD   GG        GG  PP                 TT       LL          *
* DDDDDDDDDD    GGGGGGGGGGGG  PP                 TT       LLLLLLLLLLL *
* DDDDDDDDD      GGGGGGGGGG   PP                 TT       LLLLLLLLLLL *
*                                                                     *
***********************************************************************
*                                                                     *
*   Original version by I. Belyaev        May, 1997                   *
*   Rewritten by A. Schwartz/A. Tumanov   Aug, 1997                   *
*   Modified  again by I.Belyaev          Sep, 1997                   *
*                                                                     *
*   Description: this routine does two things:                        *
*              (a) it calls routine DGPTI for inner gas pixel         *
*                  chambers and routine DGPTO for outer straw tube    *
*		   chambers. The routines translate mimps from geant  *
*		   output into Arte table TEMP .                      *
*	       (b) it compares hits in DIGB to a discriminator        *
*	           threshold value. If amount of charge is above      *
*		   threshold, hit in TEMP is put into Arte tables     *
*		   HITB&DIGB.The thresholds are based on calibration  *
*		   curve measured by A. Schwartz/C. Lu at Princeton.  *
*                                                                     *
*   Called from: DGEVNT                                               *
*                                                                     *
*   Arguments:   IPLANE     detector plane                            *
***********************************************************************

      IMPLICIT NONE
      SAVE

#include "arte/Arte.fi"
#include "arte/ArteFlag.fi"
#include "auto/GEO.fi"
#include "auto/EV.fi"
#include "auto/TEMP.fi"
#include "gean/ZGESL.fi"
#include "gean/ZGMOTH.fi"


#include "digi/TMPDIG.fi" 


      INTEGER    IPLANE 

      REAL       ASD_FRAC            ! Fraction of charge seen by ASD-8
      PARAMETER( ASD_FRAC=0.16 )     ! due to 6 ns shaping time
      
      REAL       THRESH1             ! ADC threshold in volts for inner
      PARAMETER( THRESH1=0.80 )      

      REAL       THRESH2             ! ADC threshold in volts for outer
      PARAMETER( THRESH2=1.32 )   
      
      LOGICAL    LPIXEL, LTHRESH1, LTHRESH2

      INTEGER    iPL, iMIM, iMIMF, iMIML, iDIGB1, iDIGB2, jAUX
      INTEGER    NX, NY, I, IX, IY, IP, ID1, ID2
      INTEGER    ICELL1 , ICELL2 , ITEMP1 , ITEMP2 
       
      REAL       XPMIN, XPMAX, YPMIN, YPMAX, ZPMIN, ZPMAX
      REAL       XPAD, YPAD, ZPAD, X_PAD, Y_PAD
      REAL       QUAN, ASDQUAN , QCELL 

      INTEGER  LENOCC
      EXTERNAL LENOCC

      REAL THRESHOLD 
      

#include "arte/Arte.ff"
#include "auto/GEO.ff"
#include "auto/EV.ff"
#include "auto/TEMP.ff"
 

#include "digi/TMPDIG.ff"



C     *   statement function for threshold curve of ASD-8 
      THRESHOLD( QUAN ) = 1.937 * ( 1. - EXP(-0.2422*QUAN) )

****************************************************************************
*                                                                          *
*    ssssssssss   tttttttttttt   aaaaaaaaaa   rrrrrrrrrrr   tttttttttttt   *
*   ssssssssssss  tttttttttttt  aaaaaaaaaaaa  rrrrrrrrrrrr  tttttttttttt   *
*   ss        ss       tt       aa        aa  rr        rr       tt        *
*   ss                 tt       aa        aa  rr        rr       tt        *
*   sss                tt       aa        aa  rr        rr       tt        *
*    sssssssss         tt       aaaaaaaaaaaa  rrrrrrrrrrrr       tt        *
*     sssssssss        tt       aaaaaaaaaaaa  rrrrrrrrrrr        tt        *
*            sss       tt       aa        aa  rr    rr           tt        *
*             ss       tt       aa        aa  rr     rr          tt        *
*   ss        ss       tt       aa        aa  rr      rr         tt        *
*   ssssssssssss       tt       aa        aa  rr       rr        tt        *
*    ssssssssss        tt       aa        aa  rr        rr       tt        *
*                                                                          *
****************************************************************************
    
C     *  start of codes
      IPL = IPLANE 

C     --calculate number of mimps
      IF(     iPL .LE. 0 .OR. iPL. GT. NGEDE() )THEN 
         C78='DGPTL error! iPL='
         CALL CH78AI( iPL ) 
         CALL CH78BC( '( must be in [' )
         CALL CH78AI( 1 ) 
         CALL CH78CI( NGEDE() ) 
         CALL CH78AC( ']' ) 
         CALL MEMESC(376472,1,ILARGEST,C78(1:LENOCC(C78)))
         RETURN 
      ELSEIF( IGEDE_CMP( iPL ).NE. LGESL_PTC )THEN 
         C78='DGPTL error! Plane number'
         CALL CH78BI( iPL ) 
         CALL CH78BC( 'is not a PTL chamber!' ) 
         CALL MEMESC(12323212,1,ILARGEST,C78(1:LENOCC(C78)))
         RETURN 
      ENDIF 
      
      IF( IGEDE_HIT1(iPL).GE.1 .AND. 
     &        IGEDE_HITL(iPL).GE.
     &        IGEDE_HIT1(iPL)      )THEN 
         C78='DGPTL warning! HITBs are existing for iPL='
         CALL CH78AI( iPL ) 
         CALL MEMESC(8237238,1,ILARGEST,C78(1:LENOCC(C78)))
         GO TO 300 
      ELSEIF( IGEDE_DIG1(iPL).GE.1 .AND. 
     &        IGEDE_DIGL(iPL).GE. 
     &        IGEDE_DIG1(iPL)      )THEN 
         C78='DGPTL warning!DIGBs are existing for iPL='
         CALL CH78AI( iPL ) 
         CALL MEMESC(237263,1,ILARGEST,C78(1:LENOCC(C78)))
         GO TO 300  
      ENDIF 

C     --initialize HITB and DIGB pointers for IPL  
      IF( JHITB .NE. 0)THEN
         IQ( IQGEDE_HIT1( iPL ) ) =  NHITB()+1
         IQ( IQGEDE_HITL( iPL ) ) = -1
      ELSE 
         IQ( IQGEDE_HIT1( iPL ) ) =  0
         IQ( IQGEDE_HITL( iPL ) ) = -1
      ENDIF 
      
      IF( JDIGB .NE. 0 )THEN 
         IQ( IQGEDE_DIG1( iPL ) ) =  NDIGB()+1
         IQ( IQGEDE_DIGL( iPL ) ) = -1
      ELSE 
         IQ( IQGEDE_DIG1( iPL ) ) =  0
         IQ( IQGEDE_DIGL( iPL ) ) = -1
      ENDIF 
      
C     --first and last mimps in plane 
      IMIMF  = IGEDE_MIM1( iPL )
      IMIML  = IGEDE_MIML( iPL )
      IF (IMIMF.LE.0 .OR. IMIML.LT.IMIMF)THEN
         GO TO 300 
      ENDIF 
      
      IF( IMIMF.GT.NMIMP() .OR. IMIML.GT.NMIMP() )THEN 
         C78='DGPTL warning! first MIMP is wrong!'
         CALL MEMESC(888487,1,ILARGEST,C78(1:LENOCC(C78)))       
         GO TO 300 
      ENDIF 
      
      
C     --get some geometry information
      XPMIN = FGEDE_SXMN(     iPL )       
      XPMAX = FGEDE_SXMX(     iPL )       
      YPMIN = FGEDE_SYMN(     iPL )       
      YPMAX = FGEDE_SYMX(     iPL )       
      ZPMIN = FGEDE_ZMIN(     iPL )       
      ZPMAX = FGEDE_ZMAX(     iPL )      
      
      NX    = IGEDE_NWI ( 1 , IPL )        
      NY    = IGEDE_NWI ( 2 , IPL )        
      
      XPAD  = ( XPMAX - XPMIN ) / FLOAT( NX )
      YPAD  = ( YPMAX - YPMIN ) / FLOAT( NY )
      ZPAD  = ( ZPMAX - ZPMIN ) 
      

C     -- Book temporary space for Temporary "Digits" 
      CALL MMBWRK( JTEMP, 0 , JTMPD_LEN , 50 ) 
 
C     --call DGPTI or DGPTO routines
      LPIXEL = .TRUE.
      IF (ZPAD .LT. 2.0)  LPIXEL = .FALSE.
      
      IF (LPIXEL) THEN
         CALL DGPTI( iPL , iMIMF , iMIML )
      ELSE
         CALL DGPTO( iPL , iMIMF , iMIML )
      ENDIF
      
      
C     -- check the temporary table
      IF( JTEMP .EQ. 0 )THEN
         C78='DGPTL error! temporary table does not exist!' 
         CALL ME78WC( 3209847 ) 
         GO TO 300 
      ENDIF 

      IF( N_C( JTEMP ) .NE. JTMPD_LEN )THEN
         C78='DGPTL error! Temporary table is not OK!'
         CALL ME78WC( 923874  ) 
         CALL MMDROP( JTEMP   ) 
         GO TO 300 
      ENDIF 

      IF( N_U( JTEMP ) .LE. 0 )THEN
C     * temporary table is empty 
         GO TO 300 
      ENDIF        
      
C     --loop over Temporary DIGITS 
      
      DO iTEMP1 = 1 , N_U( JTEMP ) 
         IF( ITMPD_USED( iTEMP1 ) .EQ. 0 ) THEN
            IQ( IQTMPD_USED( iTEMP1 ) ) = 1  
            ICELL1 = ITMPD_ID( iTEMP1 ) 
            QCELL  = FTMPD_Q   ( iTEMP1 ) 
            DO iTEMP2 = ITEMP1 + 1 , N_C(JTEMP )
               IF( ITMPD_USED( iTEMP2 ) .EQ. 0 )THEN 
                  ICELL2 = ITMPD_ID( iTEMP2 ) 
                  IF( ICELL2 .EQ. ICELL1 )THEN 
                     QCELL = QCELL + FTMPD_Q( iTEMP2 )
                     IQ( IQTMPD_USED( iTEMP2 ) ) = 1  
                  ENDIF       
               ENDIF 
            ENDDO
C     * here we have summed all charge on the given cell  
            QUAN = QCELL * ASD_FRAC 
C     --correction factor due to ASD-8 integration time
            QUAN = QUAN * ASD_FRAC
            QUAN = MIN( QUAN, 120.0 )
            
C     --compare to ASD-8 disriminator threshold. Threshold response 
C     as a function of input charge was measured by A. Schwartz/C. Lu
C     at Princeton. 

            ASDQUAN  = THRESHOLD( QUAN ) 
            
            LTHRESH1 = ASDQUAN .GT. THRESH1 
            LTHRESH2 = ASDQUAN .GT. THRESH2
            
            IF (     LPIXEL .AND. .NOT.LTHRESH1)   GOTO 200
            IF (.NOT.LPIXEL .AND. .NOT.LTHRESH2)   GOTO 200
            
            ID1 = ICELL1 
            
C     --book the hit
            IX = (ID1/   1000000) 
            IY = (ID1-IX*1000000)/  10000 
            IP =  ID1-IX*1000000-IY*10000
            IF( IP .NE. IPL )THEN 
               C78='DGPTL Error! Wrong Digit Plane='
               CALL CH78AI( iP ) 
               CALL CH78BC( '(must be' )
               CALL CH78BI( iPL ) 
               CALL CH78AC( ')' )
               CALL MEMESC(439848,1,ILARGEST,C78(1:LENOCC(C78)))
               GO TO 200 
            ENDIF 
            
            IF( IX .LE. 0 .OR. IX .GT. NX .OR.
     &           IY .LE. 0 .OR. IY. GT. NY ) THEN 
               C78='DGPTL Error! Wrong IX='
               CALL CH78AI( iX ) 
               CALL CH78BC( 'or IY=' )
               CALL CH78AI( iY ) 
               CALL CH78BC( 'for iPL' )
               CALL CH78AI( iPL ) 
               CALL MEMESC(97483,1,ILARGEST,C78(1:LENOCC(C78)))
               GO TO 200 
            ENDIF 
            
            X_PAD = (FLOAT(IX)-0.5)*XPAD + XPMIN
            Y_PAD = (FLOAT(IY)-0.5)*YPAD + YPMIN 
            CALL MMBOOK(JHITB,1)
            IQ(IQGEDE_HITL(iPL)) = NHITB()
            Q( QHITB_X_L (   ))  = X_PAD
            Q( QHITB_Y_L (   ))  = Y_PAD
            Q( QHITB_Z_L (   ))  = (ZPMAX + ZPMIN)/2.0
            Q( QHITB_Q_L (   ))  = ASDQUAN
            Q( QHITB_R_L (   ))  = 0.0
           IQ(IQHITB_GEDE_L( ))  = iPL 
            
C     *   Book the DIGB table 
            CALL MMBOOK( JDIGB , 1 ) 
            IQ( IQGEDE_DIGL(iPL)) = NDIGB() 
            IQ( IQDIGB_ID_L( ) )  = ID1 
             Q(  QDIGB_Q_L() )    = QUAN 
C     *   Book HITB<-->DIGB relation 
            CALL MMBREL( JHIDI , NHITB() , NDIGB() )
            
C     *  Book MIMP <--> HITB relations
C     *  Book MIMP <--> DIGB relations
            iCELL1 = ITMPD_ID( iTEMP1 ) 
            DO iTEMP2 = ITEMP1 , N_U( JTEMP ) 
               iCELL2 = ITMPD_ID( iTEMP2 )
               IF( iCELL2 .EQ. iCELL1 )THEN 
                  iMIM   = ITMPD_MIMP( iTEMP2 ) 
                  IF( iMIM .LE. 0 .OR. iMIM .GT. NMIMP() )THEN
                     C78='DGPTL Error! Wrong MIMP number='
                     CALL CH78AI( iMIM )  
                     CALL ME78WC( 9218347 ) 
                     GO TO 200 
                  ENDIF 
                  CALL MMBREL( JMIHI ,iMIM , NHITB() )
                  CALL MMBREL( JMIDI ,IMIM , NDIGB() )
               ENDIF 
            ENDDO  
         ENDIF 
 200     CONTINUE 
      ENDDO
      
      
 300  CONTINUE 
      
C     *    reset the temporary table JTEMP 
C     *    do not use MMDROP, 
C     *    instead reset the number of used rows
      IF( JTEMP .NE. 0 ) THEN 
C     *    table TEMP is valid, reset it!
C     *    First  - clear the table, 
C     *    second - reset the table itself. 
C     *
         DO iTEMP1 = 1 , N_C( JTEMP ) * N_U( JTEMP )
            IQ( JTEMP + 2 + iTEMP1 ) = 0 
         ENDDO  
         IQ( JTEMP + 1 ) = 1 
         IQ( JTEMP + 2 ) = 0 
      ENDIF 
      
      IF( IGEDE_HITL( iPL ) .LT. IGEDE_HIT1( iPL ) ) THEN
         IQ( IQGEDE_HIT1( iPL ) ) =  0 
         IQ( IQGEDE_HITL( iPL ) ) = -1
      ENDIF 
      
      IF( IGEDE_DIGL( iPL ) .LT. IGEDE_DIG1( iPL ) ) THEN
         IQ( IQGEDE_DIG1( iPL ) ) =  0 
         IQ( IQGEDE_DIGL( iPL ) ) = -1
      ENDIF 
      
      RETURN 
      END
****************************************************************************
*                    end of routine(function) dgptl                        * 
****************************************************************************
*                                                                          *
*            EEEEEEEEEEEE  NN        NN  DDDDDDDDD                         *
*            EEEEEEEEEEEE  NNN       NN  DDDDDDDDDD                        *
*            EE            NNNN      NN  DD       DD                       *
*            EE            NN NN     NN  DD        DD                      *
*            EE            NN  NN    NN  DD        DD                      *
*            EEEEEEEE      NN   NN   NN  DD        DD                      *
*            EEEEEEEE      NN    NN  NN  DD        DD                      *
*            EE            NN     NN NN  DD        DD                      *
*            EE            NN      NNNN  DD        DD                      *
*            EE            NN       NNN  DD       DD                       *  
*            EEEEEEEEEEEE  NN        NN  DDDDDDDDDD                        *   
*            EEEEEEEEEEEE  NN         N  DDDDDDDDD                         *
*                                                                          *
****************************************************************************
*                    end of routine(function) dgptl                        * 
****************************************************************************
