 
// *****************************************************************************
//
// source:    DQ_FARM_rec_init.C
//
// type:      source code
//
// created:   17 Nov 1999
//
// author:    Dr. Andreas Gellrich
//            HERA-B Collaboration
//            Humboldt-University Berlin
//            Physics Department
//            Invalidenstr. 110
//            10115 Berlin
//            Germany
//            email: Andreas.Gellrich@desy.de
//
// called by: DQ_init()
//
// changes:
// 17 Nov 1999 (AG) creation
// 13 Mar 2000 (AG) monitor RTRA, RSEG, RCCL, RVER
// 08 May 2000 (AG) event classification flag
// 12 May 2000 (AG) increase scale for ECLASS by 1
// 17 May 2000 (AG) add timing histograms for reco modules
// 22 May 2000 (AG) comment 2-d histograms
// 24 May 2000 (AG) histogram bytes
// 26 May 2000 (AG) histogram time_rcevnt here
// 31 May 2000 (AG) change scale for RECO
//  5 Jun 2000 (AG) change RICH scale to 20 sec
//  7 Jun 2000 (AG) remove some histograms
// 23 Jun 2000 (AG) add time vs hits histos
// 26 Jun 2000 (AG) adjusted histogram binning
// 27 Jun 2000 (AG) change scales once again
//
// ******************************************************************

// standard includes
#include <iostream.h>
#include <stdlib.h>
#include <stdio.h>

// ONLINE includes
#include "rhp.h"

// Arte includes
#include "arte/arte.hh"
#include "farm/farm.hh"
#include "dq/DQ.h"
#include "dq/DQ_FARM.h"

//
// function code
//
int
DQ_FARM_rec_init()
{
  // show up
  ARTE_mark( "DQ_FARM_rec_init 0", 0 );
  cout << "DQ_FARM_rec_init()" << endl;

  // 4lt trigger
  rhp_hbook1( DQ_ID( COMP_FARM, LVL_REC, 1 ), "FARM ECLASS bits", 33, -0.5, 32.5 );

  // reco tables
  rhp_hbook1( DQ_ID( COMP_FARM, LVL_REC, 10 ), "FARM RTRA / rows", 50, 0, 1000 );
  rhp_hbook1( DQ_ID( COMP_FARM, LVL_REC, 11 ), "FARM RSEG / rows", 50, 0, 1000 );
  rhp_hbook1( DQ_ID( COMP_FARM, LVL_REC, 12 ), "FARM RCCL / rows", 50, 0,  500 );
  rhp_hbook1( DQ_ID( COMP_FARM, LVL_REC, 13 ), "FARM RVER / rows", 50, 0,   50 );

  // will be removed soon
  rhp_hbook1( DQ_ID( COMP_FARM, LVL_HIT, 5 ), "FARM RECO / sec", 100, 0, 100.0 );

  // reco standalone timings
  rhp_hbook1( DQ_ID( COMP_FARM, LVL_REC, 100 ), "FARM RECO / sec",   80, 0, 80 );
  rhp_hbook1( DQ_ID( COMP_FARM, LVL_REC, 101 ), "FARM VDS / sec",    50, 0,  1 );
  rhp_hbook1( DQ_ID( COMP_FARM, LVL_REC, 104 ), "FARM Ranger / sec", 50, 0, 50 );
  rhp_hbook1( DQ_ID( COMP_FARM, LVL_REC, 105 ), "FARM RICH / sec",   50, 0, 25 );
  rhp_hbook1( DQ_ID( COMP_FARM, LVL_REC, 106 ), "FARM TRD / sec",    50, 0,  1 );
  rhp_hbook1( DQ_ID( COMP_FARM, LVL_REC, 107 ), "FARM ECAL / sec",   40, 0,  2 );
  rhp_hbook1( DQ_ID( COMP_FARM, LVL_REC, 108 ), "FARM MUON / sec",   40, 0,  2 );

  // general reco module timings
  rhp_hbook1( DQ_ID( COMP_FARM, LVL_REC, 112 ), "FARM Marple / sec",     60, 0,  6 );
  rhp_hbook1( DQ_ID( COMP_FARM, LVL_REC, 114 ), "FARM Refit / sec",      50, 0, 10 );
  rhp_hbook1( DQ_ID( COMP_FARM, LVL_REC, 116 ), "FARM ParticleID / sec", 80, 0,  8 );
  rhp_hbook1( DQ_ID( COMP_FARM, LVL_REC, 117 ), "FARM Vertex / sec",     50, 0,  2 );

  // correlations
  rhp_hbook2( DQ_ID( COMP_FARM, LVL_REC, 201 ), "FARM VDS time vs hits",    50, 0,  5000, 50, 0,  1 );
  rhp_hbook2( DQ_ID( COMP_FARM, LVL_REC, 204 ), "FARM Ranger time vs hits", 50, 0, 50000, 50, 0, 50 );
  rhp_hbook2( DQ_ID( COMP_FARM, LVL_REC, 205 ), "FARM RICH time vs hits",   60, 0, 12000, 50, 0, 25 );

  ARTE_mark( "DQ_FARM_rec_init 1", 0 );

  return 0;
}
