
// *********************************************************************
//
// Routine:      DQ_init.C
// Author:       D.Samtleben
//
// Description:  calls the different DQ-routines
//
// Changes:
// 09 Sep 1999 (AG) replace cerr by cout
// 16 Sep 1999 (DS) introduced evaluation of component set-up
// 16 Sep 1999 (AG) more print-outs
// 22 Sep 1999 (AG) move faComp() to farm/faUtil.C
// 07 Oct 1999 (AG) code polishing
// 19 Oct 1999 (AG) introduce DQ.hh
// 20 Oct 1999 (AG) use DQ.h
// 22 Oct 1999 (AG) print RCOMPMASK
// 26 Oct 1999 (AG) one DQ-flag per COMP_N_XYZ (component.h)
// 27 Oct 1999 (AG) include TLT
// 09 Nov 1999 (AG) polishing
// 16 Nov 1999 (AG) DQ-flags always hold unsigned short int mask
// 16 Nov 1999 (AG) introdcue DB-checking
// 17 Nov 1999 (AG) invoke DQ_FARM_rec_init()
// 23 Nov 1999 (AG) remove print-outs
// 24 Nov 1999 (AG) add MUON rec
// 24 Nov 1999 (AG) improve print-out
// 26 Nov 1999 (AG) intorduce rcDQ99()
// 03 Apr 2000 (AG) introduce ECAL_raw
// 05 Apr 2000 (AG) introduce DQ_MUON_raw
// 25 Apr 2000 (AG) use ARTE_mark()
//  2 Jun 2000 (AG) use ARTE_mark()
// 26 Jun 2000 (AG) introduced DQ_ITR()
//
// *********************************************************************

// standard includes
#include <iostream.h>
#include <strstream.h>
#include <stdlib.h>
#include <string.h>

// ONLINE includes
#include "component.h"
#include "runcfg.h"
#include "grr.h"

// Arte includes
#include "farm/farm.hh"
#include "dq/DQ.h"
#include "dq/DQ_SVD.h"
#include "dq/DQ_ITR.h"
#include "dq/DQ_HIPT.h"
#include "dq/DQ_OTR.h"
#include "dq/DQ_RICH.h"
#include "dq/DQ_TRD.h"
#include "dq/DQ_ECAL.h"
#include "dq/DQ_MUON.h"
#include "dq/DQ_FLT.h"
#include "dq/DQ_SLT.h"
#include "dq/DQ_TLT.h"
#include "dq/DQ_FARM.h"
#include "dq/DQ_CORR.h"
#include "dq/DQ_RECO.h"

// globals
int DQ_SVD  = 0;
int DQ_ITR  = 0;
int DQ_HIPT = 0;
int DQ_OTR  = 0;
int DQ_RICH = 0;
int DQ_TRD  = 0;
int DQ_ECAL = 0;
int DQ_MUON = 0;
int DQ_FLT  = 0;
int DQ_SLT  = 0;
int DQ_TLT  = 0;
int DQ_FARM = 0;

int DQ_CORR = 0;
int DQ_RECO = 0;

// special
int DQ_DB   = 0;
int DQ_EVAL = 0;

// old
int DQ_TARG = 0;

// externals
extern int FADEB;

//
// program code
//
int
DQ_init()
{
  // show up
  //cout << "DQ_init()" << endl;
  ARTE_mark( "DQ_init 0", 0 );
  
  // component flags
  char *rcompmask = getenv( RC_Comp_Mask_ENV );
  int mask = 0;
  if( rcompmask ) mask = atoi( rcompmask );
  char txt[80];
  memset( txt, 0, 80 );
  sprintf( txt, "DQ_init(): Component env mask=%d", mask );
  ARTE_error( GRR_SINFO, txt, 0 );

  // print
  cout << "DQ_init(): Components in run (RC_Comp_Mask_ENV): Mask=" << mask << endl;
  if( FADEB ){
    cout << "           DAQ= " << faComp( COMP_M_DAQ  ) << endl;
    cout << "           SVD= " << faComp( COMP_M_SVD  ) << endl;
    cout << "           ITR= " << faComp( COMP_M_ITR  ) << endl;
    cout << "           HIPT=" << faComp( COMP_M_HIPT ) << endl;
    cout << "           OTR= " << faComp( COMP_M_OTR  ) << endl;
    cout << "           RICH=" << faComp( COMP_M_RICH ) << endl;
    cout << "           TRD= " << faComp( COMP_M_TRD  ) << endl;
    cout << "           ECAL=" << faComp( COMP_M_ECAL ) << endl;
    cout << "           MUON=" << faComp( COMP_M_MUON ) << endl;
    cout << "           FLT= " << faComp( COMP_M_FLT  ) << endl;
    cout << "           SLT= " << faComp( COMP_M_SLT  ) << endl;
    cout << "           TLT= " << faComp( COMP_M_TLT  ) << endl;
    cout << "           FARM=" << faComp( COMP_M_FARM ) << endl;
  }

  //
  // take only dq which is asked for in the kumac & RCOMPMASK
  //
  cout << "DQ_init(): DQ settings (DQ kumac && comp env):" << endl;

  cout << "           SVD:  RAW/HIT/REC "
       << DQ_flag( COMP_SVD, LVL_M_RAW ) << " "
       << DQ_flag( COMP_SVD, LVL_M_HIT ) << " "
       << DQ_flag( COMP_SVD, LVL_M_REC ) << endl;
  
  cout << "           ITR:  RAW/HIT/REC "
       << DQ_flag( COMP_ITR, LVL_M_RAW ) << " "
       << DQ_flag( COMP_ITR, LVL_M_HIT ) << " "
       << DQ_flag( COMP_ITR, LVL_M_REC ) << endl;
  
  cout << "           HIPT: RAW/HIT/REC "
	 << DQ_flag( COMP_HIPT, LVL_M_RAW ) << " "
	 << DQ_flag( COMP_HIPT, LVL_M_HIT ) << " "
	 << DQ_flag( COMP_HIPT, LVL_M_REC ) << endl;
    
  cout << "           OTR:  RAW/HIT/REC "
       << DQ_flag( COMP_OTR, LVL_M_RAW ) << " "
       << DQ_flag( COMP_OTR, LVL_M_HIT ) << " "
       << DQ_flag( COMP_OTR, LVL_M_REC ) << endl;
  
  cout << "           RICH: RAW/HIT/REC "
       << DQ_flag( COMP_RICH, LVL_M_RAW ) << " "
       << DQ_flag( COMP_RICH, LVL_M_HIT ) << " "
       << DQ_flag( COMP_RICH, LVL_M_REC ) << endl;
  
  cout << "           TRD:  RAW/HIT/REC "
       << DQ_flag( COMP_TRD, LVL_M_RAW ) << " "
       << DQ_flag( COMP_TRD, LVL_M_HIT ) << " "
       << DQ_flag( COMP_TRD, LVL_M_REC ) << endl;
  
  cout << "           ECAL: RAW/HIT/REC "
       << DQ_flag( COMP_ECAL, LVL_M_RAW ) << " "
       << DQ_flag( COMP_ECAL, LVL_M_HIT ) << " "
       << DQ_flag( COMP_ECAL, LVL_M_REC ) << endl;
  
  cout << "           MUON: RAW/HIT/REC "
       << DQ_flag( COMP_MUON, LVL_M_RAW ) << " "
       << DQ_flag( COMP_MUON, LVL_M_HIT ) << " "
       << DQ_flag( COMP_MUON, LVL_M_REC ) << endl;
  
  cout << "           FLT:  RAW/HIT/REC "
       << DQ_flag( COMP_FLT, LVL_M_RAW ) << " "
       << DQ_flag( COMP_FLT, LVL_M_HIT ) << " "
       << DQ_flag( COMP_FLT, LVL_M_REC ) << endl;
  
  cout << "           SLT:  RAW/HIT/REC "
       << DQ_flag( COMP_SLT, LVL_M_RAW ) << " "
       << DQ_flag( COMP_SLT, LVL_M_HIT ) << " "
       << DQ_flag( COMP_SLT, LVL_M_REC ) << endl;
  
  cout << "           TLT:  RAW/HIT/REC "
       << DQ_flag( COMP_TLT, LVL_M_RAW ) << " "
       << DQ_flag( COMP_TLT, LVL_M_HIT ) << " "
       << DQ_flag( COMP_TLT, LVL_M_REC ) << endl;
  
  cout << "           FARM: RAW/HIT/REC "
       << DQ_flag( COMP_FARM, LVL_M_RAW ) << " "
       << DQ_flag( COMP_FARM, LVL_M_HIT ) << " "
       << DQ_flag( COMP_FARM, LVL_M_REC ) << endl;
  
  cout << "           CORR: RAW/HIT/REC "
       << DQ_flag( COMP_CORR, LVL_M_RAW ) << " "
       << DQ_flag( COMP_CORR, LVL_M_HIT ) << " "
       << DQ_flag( COMP_CORR, LVL_M_REC ) << endl;
  
  cout << "           RECO: RAW/HIT/REC "
       << DQ_flag( COMP_RECO, LVL_M_RAW ) << " "
       << DQ_flag( COMP_RECO, LVL_M_HIT ) << " "
       << DQ_flag( COMP_RECO, LVL_M_REC ) << endl;
  
  //
  // call  init functions
  //
  int dq_flag = 0;

  // SVD
  if( DQ_flag( COMP_SVD,  LVL_M_HIT ) ){ 
    ARTE_mark( "DQ_SVD_hit_init", 0 );
    dq_flag = DQ_SVD_hit_init();
  }
  if( DQ_flag( COMP_SVD,  LVL_M_REC ) ){
    ARTE_mark( "DQ_SVD_rec_init", 0 );
    dq_flag = DQ_SVD_rec_init();
  }

  // ITR
  if( DQ_flag( COMP_ITR,  LVL_M_HIT ) ){ 
    ARTE_mark( "DQ_ITR_hit_init", 0 );
    dq_flag = DQ_ITR_hit_init();
  }
  if( DQ_flag( COMP_ITR,  LVL_M_REC ) ){
    ARTE_mark( "DQ_ITR_rec_init", 0 );
    dq_flag = DQ_ITR_rec_init();
  }
  
  // HIPT
  if( DQ_flag( COMP_HIPT, LVL_M_RAW ) ){
    ARTE_mark( "DQ_HIPT_raw_init", 0 );
    dq_flag = DQ_HIPT_raw_init();
  }
  
  // OTR
  if( DQ_flag( COMP_OTR,  LVL_M_HIT ) ){
    ARTE_mark( "DQ_OTR_hit_init", 0 );
    dq_flag = DQ_OTR_hit_init();
  }
  if( DQ_flag( COMP_OTR,  LVL_M_REC ) ){
    ARTE_mark( "DQ_OTR_rec_init", 0 );
    dq_flag = DQ_OTR_rec_init();
  }
  
  // RICH
  if( DQ_flag( COMP_RICH, LVL_M_RAW ) ){
    ARTE_mark( "DQ_RICH_raw_init", 0 );
    dq_flag = DQ_RICH_raw_init();
  }
  if( DQ_flag( COMP_RICH, LVL_M_HIT ) ){
    ARTE_mark( "DQ_RICH_hit_init", 0 );
    dq_flag = DQ_RICH_hit_init();
  }
  if( DQ_flag( COMP_RICH, LVL_M_REC ) ){
    ARTE_mark( "DQ_RICH_rec_init", 0 );
    dq_flag = DQ_RICH_rec_init();
  }
  
  // ECAL
  if( DQ_flag( COMP_ECAL, LVL_M_RAW ) ){
    ARTE_mark( "DQ_ECAL_raw_init", 0 );
    dq_flag = DQ_ECAL_raw_init();
  }
  if( DQ_flag( COMP_ECAL, LVL_M_HIT ) ){
    ARTE_mark( "DQ_ECAL_hit_init", 0 );
    dq_flag = DQ_ECAL_hit_init();
  }
  if( DQ_flag( COMP_ECAL, LVL_M_REC ) ){
    ARTE_mark( "DQ_ECAL_rec_init", 0 );
    dq_flag = DQ_ECAL_rec_init();
  }
  
  // MUON
  if( DQ_flag( COMP_MUON, LVL_M_RAW ) ){
    ARTE_mark( "DQ_MUON_raw_init", 0 );
    dq_flag = DQ_MUON_raw_init();
  }
  if( DQ_flag( COMP_MUON, LVL_M_HIT ) ){
    ARTE_mark( "DQ_MUON_hit_init", 0 );
    dq_flag = DQ_MUON_hit_init();
  }
  if( DQ_flag( COMP_MUON, LVL_M_REC ) ){
    ARTE_mark( "DQ_MUON_rec_init", 0 );
    dq_flag = DQ_MUON_rec_init();
  }
  
  // TLT
  if( DQ_flag( COMP_TLT,  LVL_M_REC ) ){
    ARTE_mark( "DQ_TLT_rec_init", 0 );
    dq_flag = DQ_TLT_rec_init();
  }

  // FARM
  if( DQ_flag( COMP_FARM, LVL_M_HIT ) ){
    ARTE_mark( "DQ_FARM_hit_init", 0 );
    dq_flag = DQ_FARM_hit_init();
  }
  if( DQ_flag( COMP_FARM, LVL_M_REC ) ){
    ARTE_mark( "DQ_FARM_rec_init", 0 );
    dq_flag = DQ_FARM_rec_init();
  }
  
  // CORR
  if( DQ_flag( COMP_CORR, LVL_M_HIT ) ){
    ARTE_mark( "DQ_CORR_hit_init", 0 );
    dq_flag = DQ_CORR_hit_init();
  }
  
  // RECO
  if( DQ_flag( COMP_RECO, LVL_M_REC ) ){
    ARTE_mark( "rcDQinit", 0 );
    rcDQinit();
  }

  // after initialization has been done read database
  // dq_flag = DQ_db();

  ARTE_mark( "DQ_init 1", 0 );
  
  return 0;
}
