//                              -*- Mode: C++ -*-
//
// LkParticle.h -- A Lockable Particle class (for doing Particle combinatorics)
//
// $Id: DQ_ClusParticle.hh,v 1.1 2000/04/19 20:42:15 natasha Exp $
//
// Copyright (C)  1993  The Board of Trustees of The Leland Stanford
//                                  Junior University.  All Rights Reserved.
//
// Creator         : Tom Pavel  <pavel>     Thu Oct 21 10:05:52 1993
// Modifier        : Tom Pavel  <pavel>     Thu Oct 21 11:27:58 1993
// Update Count    : 2
// Status          : Unknown, Use with caution!
//

#ifndef _ClusParticle_h_
#define _ClusParticle_h_

#ifdef __GNUG__
#pragma interface
#endif

//#define HEP_SHORT_NAMES
//#include <CLHEP/Lockable.h>
//#undef HEP_SHORT_NAMES
#include "dq/DQ_LkParticle.hh"
#include "arte/arte.hh"


class ClusParticle : public LkParticle {
public:
  ClusParticle() {}
  ClusParticle(const LkParticle& p, ArtePointer<RCCL> prccl =0 ) : 
    LkParticle(p), rccl(prccl) {fill();}

  ClusParticle(const Particle& p, ArtePointer<RCCL> prccl =0) :
    LkParticle(p), rccl(prccl) { fill();}
  ClusParticle(const ClusParticle & p) :
    LkParticle(p) { copy(p);}
  virtual ~ClusParticle();

  virtual int isA(ClassId id);
  
  static ClassId ClassID() { static int x; return &x; }
  
  virtual ClusParticle& operator += (const ClusParticle& p){
    LkParticle::operator+=(p); rccl =p.rccl;  copy(p); return *this; }
    
  ClusParticle operator+ (const ClusParticle& p){
    ClusParticle sum = *this; sum += p; rccl = 0; copy(p); return sum; }
  virtual ClusParticle & operator = (const ClusParticle& p){
    LkParticle::operator=(p); rccl = p.rccl; xec = p.xec;
    yec = p.yec; eec = p.eec; rpidec = p.rpidec; ptec = p.ptec;
    nec = p.nec; gcce = p.gcce; nfed = p.nfed; flag = p.flag;
    return *this;}
    
  virtual ClusParticle * child(int i) const { 
    return (ClusParticle*) LkParticle::child(i);
  };

  ArtePointer<RCCL> whichBank() const { return rccl;}
  float xc() const { return xec;};
  float yc() const { return yec;};
  float ec() const { return eec;};
  float rpidc() const { return rpidec;};
  float ptc() const { return ptec;};
  int   width() const { return nec;};
  int   icell() const { return gcce;};
  int   ifed() const { return nfed;};
  int   iflag() const { return flag;};
  void dump() const;

private:
  void fill(); // fill data from the rccl mark.
  void copy(const ClusParticle & cp);

  ArtePointer<RCCL> rccl;
  float xec,yec,eec,rpidec,ptec;
  int   nec,gcce,nfed,flag;
};

#endif  /* _LkParticle_h_ */
