
// *****************************************************************************
//
// source:    faDaqLog.C
//
// type:      source code
//
// created:    1 Dec 1998
//
// author:    Andreas Gellrich
//            HERA-B Collaboration
//            DESY Zeuthen
//            Platanenallee 6
//            D-15738 Zeuthen
//            Germany
//            email: Andreas.Gellrich@desy.de
//
// changed:   
//  1 Dev 1998 (AG) send daq-formated event to logger
// 11 Jan 1999 (AG) includes
// 15 Jan 1999 (AG) return error code
// 26 Jan 1999 (AG) free memory
// 23 Feb 1999 (AG) includes
// 11 Mar 1999 (AG) polishing
//  1 Jun 1999 (AG) put includes
// 17 Aug 1999 (AG) handle GRR_FISHY
// 18 Aug 1999 (AG) always wait 5 sec after rpm_flood() error
// 24 Jan 2000 (AG) correct usleep()
//
// *****************************************************************************

// standard includes
#include <iostream.h>
#include <stdio.h>
#include <unistd.h>

// ONLINE includes
#include "event.h"
#include "grr.h"
#include "esp.h"
#include "log.h"

// Arte includes
#include "arte/ArteFlag.h"
#include "farm/fa.hh"
#include "farm/farm.hh"

#if sun
extern "C" int usleep( unsigned int musec );
#endif

// externals
extern int *evtBuf;
extern int FADEB;
extern int DAQLOG;

//
// program code
//
int
faDaqLog()
{
  // return code
  int rc = GRR_OK;

  // check flag
  if( DAQLOG ){
   
    // get event
    EVENT_HDR *event = ARTE_get_event();

    // get event header
    EVENT_HDR evtHdr;
    ESP_get_event_hdr( event, &evtHdr );
    
    // build log header
    LogHdr *hdr = new LogHdr;
    hdr->exp    = arteflag_.numexp;
    hdr->run    = arteflag_.numrun;
    hdr->evt    = arteflag_.numevt; // re-filled by logger
    hdr->datime = time( 0 );
    
    // print log header
#ifdef _DEB_
    if( FADEB ){
      cout << "LOG: "
	   << "exp="    << hdr->exp      << " "
	   << "run="    << hdr->run      << " "
	   << "evt="    << hdr->evt      << " "
	   << "datime=" << hdr->datime   << " "
	   << "bc="     << evtHdr.length << endl;
    }
#endif

    //
    // send event to logger
    //

    // wait time in musec after rpm_flood() error
    int wait = 5;

    while( 1 ){
      if( ( rc = l4aLogEvt( (char *) evtBuf, evtHdr.length, hdr ) ) == GRR_OK ){
#ifdef _DEB_
	if( FADEB ){
	  cout << "LOG: l4aLogEvt()" << endl;
	}
#endif
	break;
      }
      // fishy (again)
      else if( rc == GRR_FISHY ){
#ifdef _ERR_
	cout << "LOG: l4aLogEvt() GRR_FISHY" << endl;
#endif
	sleep( wait );
      }
      // time out (next)
      else if( rc == GRR_TIME ){
#ifdef _ERR_
	cout << "LOG: l4aLogEvt() GRR_TIME" << endl;
#endif
	sleep( wait );
	break;
      }
      // reject (again)
      else if( rc == GRR_REJECT ){
#ifdef _ERR_
	cout << "LOG: l4aLogEvt() GRR_REJECT" << endl;
#endif
	sleep( wait );
      }
      // bad address (next)
      else if( rc == GRR_BAD_ADDRESS ){
#ifdef _ERR_
	cout << "LOG: l4aLogEvt() GRR_BAD_ADDRESS" << endl;
#endif
	sleep( wait );
	break;
      }
      // error (next)
      else {
#ifdef _ERR_
	cout << "LOG: l4aLogEvt() rc=" << rc << endl;
#endif
	sleep( wait );
	break;
      }
    }

    // free memory
    delete hdr;
  }

  return rc;
}
