
// *****************************************************************************
//
// source:    faUtil.C
//
// type:      source code
//
// created:   29 Oct 1996
//
// author:    Andreas Gellrich
//            HERA-B Collaboration
//            DESY Zeuthen
//            Platanenallee 6
//            D-15738 Zeuthen
//            Germany
//            email: Andreas.Gellrich@desy.de
//
// purpose:   various utilities
//
// changes:         
//                  polishing (AG: 9 Feb 99)
//                  DTAR handling (AG: 10 Feb 99)
//                  DTAR has SLB_SUBHDR inside (AG: 22 Feb 99)
//                  removed faUtil.hh (AG: 23 Feb 99)
//                  provide target and all daq info (2 Mar 99)
//                  Fortran/C callable functions
//                  polishing (AG: 11 Mar 99)
//                  change default return values of utility functions (AG: 26 Mar 99)
//                  prepare for DKEY (AG: 27 Apr 99)
//                  introduce function ARTE_get_event() (AG: 6 May 99)
//                  improve ARTE_get_event() (AG: 11 May 99)
//                  avoid using inline functiuon (AG: 14 May 99)
//                  fixing of typos (AG: 17 May 99)
//                  EVHD has more elements now (AG: 26 May 99)
//                  remove DKEY again (AG: 27 May 99)
//                  make it more secure (AG: 28 May 99)
//                  massive clean-up (AG: 31 May 99)
//                  put includes (AG: 1 Jun 99)
//                  repair DST reading (AG: 3 Jun 99)
// 09 Sep 1999 (AG) always use cout
// 09 Sep 1999 (AG) complete utilities to access tables
// 22 Sep 1999 (AG) include faComp()
// 07 Oct 1999 (AG) include faComp_evt()
// 22 Oct 1999 (AG) modify faComp() to handle DAQ correctly
// 16 Nov 1999 (AG) introduce error logging for online
// 22 Nov 1999 (AG) introduce faReco()
// 23 Nov 1999 (AG) introduce environment variables for run setetings (Run Type)
// 24 Nov 1999 (AG) add node id to ARTE_error()
// 03 Dec 1999 (AG) adapt to new EVHD table (ipaddr and cputime in EVRC now)
// 16 Dec 1999 (AG) introduce EVRC; remove ipaddr and cputime from EVHD
// 17 Dec 1999 (AG) introduce faFillEVRC
// 23 Feb 2000 (AG) adapt to new EVRC; remove faEVRCfill()
// 24 Feb 2000 (AG) fixed gethostname problem
// 29 Feb 2000 (AG) linux -> __linux
//  1 Mar 2000 (CP) introduce faHit() (needs PRCMHITMASK from ONLINE)
//  3 Mar 2000 (AG) add Arte version to ARTE_error()
// 13 Mar 2000 (AG) improve printout
// 21 Mar 2000 (AG) modified faComp()
// 21 Mar 2000 (AG) fixed faDate()
// 26 Mar 2000 (AG) introduce faCompStrg()
// 11 Apr 2000 (AG) improve faPrintNrow()
// 12 Apr 2000 (AG) introduce ARTE_exit()
// 12 Apr 2000 (AG) introduce ARTE_mark()
// 14 Apr 2000 (AG) change initial text in ARTE_MARK_TEXT
// 28 Apr 2000 (AG) introduce ARTE_log()
// 02 May 2000 (AG) changes in ARTE_exit()
// 08 May 2000 (AG) improve printout
// 09 May 2000 (AG) add debug printout
// 10 May 2000 (AG) make sure no memory leaks occur
// 18 May 2000 (AG) add run/exp/evt to output
// 24 May 2000 (AG) change printout of faPrintNrow()
// 26 May 2000 (AG) introduce faDDAQdbKey() and faDDAQdbVersion()
// 31 May 2000 (AG) introduce event count in logfile
//  1 Jun 2000 (AG) corrected undefined variable bug
//  5 Jun 2000 (AG) correction in ARTE_log()
//  7 Jun 2000 (AG) set char arrays to zero at the beginnig
//  8 Jun 2000 (AG) some polishing of printout
// 13 Jun 2000 (AG) nice printout of ipaddr
// 13 Jun 2000 (AG) move all ARTE_ functions to faARTE.C
// 15 Jun 2000 (AG) improve printout
// 16 Jun 2000 (AG) write out flagdst as hex
//
// *****************************************************************************

// standard includes
#include <iostream.h>
#include <strstream.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <math.h>
#include <unistd.h>

// ONLINE includes
#include "esp.h"
#include "runcfg.h"
#include "component.h"

// Arte includes
#include "arte/arte.hh"
#include "arte/ArteFlag.h"
#include "farm/fa.hh"
#include "farm/farm.hh"
#include "reco/eclass.hh"

// defines
#define PRCMHITMASK "HITmask"

// externals
extern int FADEB; // debug flag

// inline function(s)
inline char *IPaddr( int ip )
{
  static char s[5];
  memset( s, '\0', 5 );
  sprintf( s, "%d.%d.%d.%d",
	   ( ip >> 16 ) & 0xff,
	   ( ip >> 24 ) & 0xff,
	   ( ip >>  0 ) & 0xff,
	   ( ip >>  8 ) & 0xff );
  return s;
}

//
// check presence of component COMP_M_XYZ in run
//
int
faComp( int COMP_M_XYZ )
{
  // init
  int compmask = 0;
  int rc       = 0;

  // check RCOMPMASK
  char *rcompmask = getenv( RC_Comp_Mask_ENV );
  if( rcompmask ){
    compmask = atoi( rcompmask ); // convert string to int
    if( compmask & COMP_M_XYZ ){
      rc = 1;  // COMP_M_XYZ present
    }
    else {
      rc = 0; // COMP_M_XYZ NOT present
    }

    // special treatment for DAQ which obviously created RCOMPMASK
    if( COMP_M_XYZ == COMP_M_DAQ ) rc = 1;

  }
  // RCOMPMASK NOT present, return 1 for offline usage
  else {
    rc = 1;
  }

  // print
  if( FADEB ){
    cout << "faComp(): "
	 << "rcompmask="         << rcompmask       << " "
	 << "compmask=0x" << hex << compmask << dec << "=|";
    for( int i=31; i<=0; i-- ){
      if( !(i%4) ) cout << "|";
      int j = 0;
      if( compmask & int( pow( 2, i ) ) ) j = 1;
      cout << j;
    }
    cout << " ";
    cout << "COMP_M_XYZ=0x"      << hex << COMP_M_XYZ << dec << " "
	 << "rc="                << rc                << endl;
  }
  return rc;
}

//
// check presence of component COMP_M_XYZ for Hit production
//
int
faHit( int COMP_M_XYZ )
{
  // init
  int hitmask = 0;
  int rc      = 0;

  // check HITCOMASK
  char *rhitmask = getenv( PRCMHITMASK );
  if( rhitmask ){
    hitmask = atoi( rhitmask ); // convert string to int
    if( hitmask & COMP_M_XYZ ){
      rc = 1;  // COMP_M_XYZ present
    }
    else {
      rc = 0; // COMP_M_XYZ NOT present
    }

    // special treatment for DAQ which obviously created RRECOMASK
    if( COMP_M_XYZ == COMP_M_DAQ ) rc = 1;

  }
  // RHITMASK NOT present, return 1 for offline usage
  else {
    rc = 1;
  }

  // print
  if( FADEB ){
    cout << "faHit(): "
	 << "rhitmask="         << rhitmask       << " "
	 << "hitmask=0x" << hex << hitmask << dec << "=|";
    for( int i=31; i<=0; i-- ){
      if( !(i%4) ) cout << "|";
      int j = 0;
      if( hitmask & int( pow( 2, i ) ) ) j = 1;
      cout << j;
    }
    cout << " ";
    cout << "COMP_M_XYZ=0x"      << hex << COMP_M_XYZ << dec << " "
	 << "rc="                << rc                << endl;
  }
  return rc;
}

//
// check presence of component COMP_M_XYZ for reconstruction
//
int
faReco( int COMP_M_XYZ )
{
  // init
  int recomask = 0;
  int rc       = 0x0000;

  // check RRECOMASK
  char *rrecomask = getenv( PRCMRECOMASK );
  if( rrecomask ){
    recomask = atoi( rrecomask ); // convert string to int
    if( recomask & COMP_M_XYZ ){
      rc = 0xffff;  // COMP_M_XYZ present, return 0xffff
    }
    else {
      rc = 0x0000; // COMP_M_XYZ NOT present, return 0x0000;
    }

    // special treatment for DAQ which obviously created RRECOMASK
    if( COMP_M_XYZ == COMP_M_DAQ ) rc = 0xffff;

  }
  // RRECOMASK NOT present, return 1 for offline usage
  else {
    rc = 0xffff;
  }

  // print
  if( FADEB ){
    cout << "faReco(): "
	 << "rrecomask="         << rrecomask       << " "
	 << "recomask=0x" << hex << recomask << dec << "=|";
    for( int i=31; i<=0; i-- ){
      if( !(i%4) ) cout << "|";
      int j = 0;
      if( recomask & int( pow( 2, i ) ) ) j = 1;
      cout << j;
    }
    cout << " ";
    cout << "COMP_M_XYZ=0x"      << hex << COMP_M_XYZ << dec << " "
	 << "rc="                << rc                << endl;
  }
  return rc;
}

//
// check presence of component COMP_M_XYZ in event
//
int
faComp_evt( int COMP_M_XYZ )
{
  // init
  int compmask = 0;
  int rc       = 0x0000;

  // DIHD-table
  if( ArteTable<DIHD>::size() ){
    ArtePointer<DIHD> dihd( 1 ); // pointer to table
    compmask = dihd->cmsk;       // component mask
    if( compmask & COMP_M_XYZ ){
      rc = 0xffff; // COMP_M_XYZ present, return 0xffff 
    }
    else {
      rc = 0x0000; // COMP_M_XYZ NOT prsent, return 0x0000
    }
  }
  // no DIHD, return -1
  else {
    rc = -1;
  }
  
  // print
  if( FADEB ){
    cout << "faComp_evt(): "
	 << "compmask=0x" << hex << compmask << dec << "=|";
    for( int i=31; i<=0; i-- ){
      if( !(i%4) ) cout << "|";
      int j = 0;
      if( compmask & int( pow( 2, i ) ) ) j = 1;
      cout << j;
    }
    cout << " ";
    cout << "COMP_M_XYZ=0x"      << hex << COMP_M_XYZ << dec << " "
	 << "rc="                << rc                << endl;
  }

  return rc;
}

//
// print string of components from mask
//
char *
faCompStrg( int mask )
{
  static char strg[80];
  memset( strg, 0, 80 );
  ostrstream cstr( strg, 80 );

  cstr << "Mask=" << mask << " components: ";
  if( mask & COMP_M_DAQ     ) cstr << "DAQ ";
  if( mask & COMP_M_SVD     ) cstr << "SVD ";
  if( mask & COMP_M_ITR     ) cstr << "ITR ";
  if( mask & COMP_M_HIPT    ) cstr << "HIPT ";
  if( mask & COMP_M_OTR     ) cstr << "OTR ";
  if( mask & COMP_M_RICH    ) cstr << "RICH ";
  if( mask & COMP_M_TRD     ) cstr << "TRD ";
  if( mask & COMP_M_ECAL    ) cstr << "ECAL ";
  if( mask & COMP_M_MUON    ) cstr << "MUON ";
  if( mask & COMP_M_MPIX    ) cstr << "MPIX ";
  if( mask & COMP_M_MPAD    ) cstr << "MPAD ";
  if( mask & COMP_M_FLT     ) cstr << "FLT ";
  if( mask & COMP_M_SLT     ) cstr << "SLT "; 
  if( mask & COMP_M_FARM    ) cstr << "FARM";
  if( mask & COMP_M_TARG    ) cstr << "TARG ";
  if( mask & COMP_M_FCS     ) cstr << "FCS ";
  if( mask & COMP_M_COMMON  ) cstr << "COMMON ";
  if( mask & COMP_M_MCTRUTH ) cstr << "MCTRUTH "; 

  return strg;
}

//
// print run/event
//
void
faPrintEvtCount( int run, int event, int exp )
{
  if( run > 0 && event > 0 ){
    if( ( event % int( pow( 10, int( log10( event ) ) ) ) ) == 0 ){
      cout << " PRINT: "
	   << "run="   << run   << " "
	   << "event=" << event << " "
	   << "exp="   << exp   << endl;
    }
  }
}

//
// print event classification
//
void
faPrintClass()
{
  unsigned int flagdst = faEVHDflagdst();
  cout << " CLASS: ";
  for( int bit=0; bit<16; bit++ ){
    bool j = ( flagdst >> bit ) & 1;
    if( j ) cout << rcClassName( bit + 1 ) << " ";
  }
  cout << endl;
}

//
// print rows of tables
//
int
faPrintNrow()
{
  //
  // HEAD
  //
  cout << " HEAD: ";
  cout << "EVHD=" << ArteTable<EVHD>::size() * sizeof( EVHD ) << "B ";
  cout << "EVRC=" << ArteTable<EVRC>::size() * sizeof( EVRC ) << "B ";
  cout << "EVFN=" << ArteTable<EVFN>::size() * sizeof( EVFN ) << "B ";
  cout << endl;

  //
  // RAW
  //
  cout << " RAW:  ";
                                cout << "DIHD=" << ArteTable<DIHD>::size() * sizeof( DIHD ) << "B ";
                                cout << "DEVT=" << ArteTable<DEVT>::size() * sizeof( DEVT ) << "B ";
  if( ArteTable<DDAQ>::size() ) cout << "DDAQ=" << ArteTable<DDAQ>::size() * sizeof( DDAQ ) << "B ";
  if( ArteTable<DFLT>::size() ) cout << "DFLT=" << ArteTable<DFLT>::size() * sizeof( DFLT ) << "B ";
  if( ArteTable<DSLT>::size() ) cout << "DSLT=" << ArteTable<DSLT>::size() * sizeof( DSLT ) << "B ";
  if( ArteTable<DTLT>::size() ) cout << "DTLT=" << ArteTable<DTLT>::size() * sizeof( DTLT ) << "B ";
  if( ArteTable<D4LT>::size() ) cout << "D4LT=" << ArteTable<D4LT>::size() * sizeof( D4LT ) << "B ";
  if( ArteTable<DTAR>::size() ) cout << "DTAR=" << ArteTable<DTAR>::size() * sizeof( DTAR ) << "B ";
  if( ArteTable<DTAR>::size() ) cout << "DTA1=" << ArteTable<DTA1>::size() * sizeof( DTA1 ) << "B ";
  if( ArteTable<DFCS>::size() ) cout << "DFCS=" << ArteTable<DFCS>::size() * sizeof( DFCS ) << "B ";
  cout << endl;

  cout << "       ";
  if( ArteTable<DSVD>::size() ) cout << "DSVD=" << ArteTable<DSVD>::size() * sizeof( DSVD ) << "B ";
  if( ArteTable<DITR>::size() ) cout << "DITR=" << ArteTable<DITR>::size() * sizeof( DITR ) << "B ";
  if( ArteTable<DPTC>::size() ) cout << "DPTC=" << ArteTable<DPTC>::size() * sizeof( DPTC ) << "B ";
  if( ArteTable<DOTR>::size() ) cout << "DOTR=" << ArteTable<DOTR>::size() * sizeof( DOTR ) << "B ";
  if( ArteTable<DRIC>::size() ) cout << "DRIC=" << ArteTable<DRIC>::size() * sizeof( DRIC ) << "B ";
  if( ArteTable<DTRD>::size() ) cout << "DTRD=" << ArteTable<DTRD>::size() * sizeof( DTRD ) << "B ";
  if( ArteTable<DCAL>::size() ) cout << "DCAL=" << ArteTable<DCAL>::size() * sizeof( DCAL ) << "B ";
  if( ArteTable<DMUT>::size() ) cout << "DMUT=" << ArteTable<DMUT>::size() * sizeof( DMUT ) << "B ";
  if( ArteTable<DMUX>::size() ) cout << "DMUX=" << ArteTable<DMUX>::size() * sizeof( DMUX ) << "B ";
  if( ArteTable<DMUP>::size() ) cout << "DMUP=" << ArteTable<DMUP>::size() * sizeof( DMUP ) << "B ";
  cout << endl;

  //
  // RECO
  //

  // reco tables
  cout << " RECO: ";

  cout << "EVHI="
       << ArteTable<EVHI>::size() << " (" << ArteTable<EVHI>::size() * sizeof( EVHI ) << "B) ";

  cout << "RTRA=" 
       << ArteTable<RTRA>::size() << " (" << ArteTable<RTRA>::size() * sizeof( RTRA ) << "B) ";

  cout << "RSEG=" 
       << ArteTable<RSEG>::size() << " (" << ArteTable<RSEG>::size() * sizeof( RSEG ) << "B) ";

  cout << "RCCL=" 
       << ArteTable<RCCL>::size() << " (" << ArteTable<RCCL>::size() * sizeof( RCCL ) << "B) ";

  cout << "RVER=" 
       << ArteTable<RVER>::size() << " (" << ArteTable<RVER>::size() * sizeof( RVER ) << "B) ";

  cout << "RREL=" 
       << ArteTable<RREL>::size() << " (" << ArteTable<RREL>::size() * sizeof( RREL ) << "B) ";

  cout << endl;

  return ARTE_GOOD;
}

//
// EVHD ************************************************************************
//

//
// print EVHD-table
//
int
faPrintEVHD()
{
  if( ArteTable<EVHD>::size() ){
    ArtePointer<EVHD> evhd( 1 );
    cout << " EVHD: "
	 << "evt="           << evhd->evt            << " "
	 << "run="           << evhd->run            << " "
	 << "exp="           << evhd->exp            << " "
	 << "date="          << evhd->date           << " "
	 << "time="          << evhd->time           << " "
	 << "vfld="          << evhd->vfld           << " "
	 << "dst=0x"  << hex << evhd->flagdst << dec << " "
	 << "key="           << evhd->dbmajor        << " "
	 << "rev="           << evhd->dbminor        << " "
	 << "flag=0x" << hex << evhd->flag    << dec << endl;
    return ARTE_GOOD;
  }
  else {
    return ARTE_ERR;
  }
}

//
// event number
//
int
faEVHDevt()
{
  if( ArteTable<EVHD>::size() ){
    ArtePointer<EVHD> p( 1 );
    return p->evt;
  }
  else {
    return -1;
  }
}

//
// run number
//
int
faEVHDrun()
{
  if( ArteTable<EVHD>::size() ){
    ArtePointer<EVHD> p( 1 );
    return p->run;
  }
  else {
    return -1;
  }
}

//
// experiment number
//
int
faEVHDexp()
{
  if( ArteTable<EVHD>::size() ){
    ArtePointer<EVHD> p( 1 );
    return p->exp;
  }
  else {
    return -1;
  }
}

//
// date
//
int
faEVHDdate()
{
  if( ArteTable<EVHD>::size() ){
    ArtePointer<EVHD> p( 1 );
    return p->date ;
  }
  else {
    return -1;
  }
}

//
// time
//
int
faEVHDtime()
{
  if( ArteTable<EVHD>::size() ){
    ArtePointer<EVHD> p( 1 );
    return p->time;
  }
  else {
    return -1;
  }
}

//
// vfld
//
float
faEVHDvfld()
{
  if( ArteTable<EVHD>::size() ){
    ArtePointer<EVHD> p( 1 );
    return p->vfld;
  }
  else {
    return -1;
  }
}

//
// flagdst
//
int
faEVHDflagdst()
{
  if( ArteTable<EVHD>::size() ){
    ArtePointer<EVHD> p( 1 );
    return p->flagdst;
  }
  else {
    return -1;
  }
}

//
// dbmajor
//
int
faEVHDdbmajor()
{
  if( ArteTable<EVHD>::size() ){
    ArtePointer<EVHD> p( 1 );
    return p->dbmajor;
  }
  else {
    return -1;
  }
}

//
// dbminor
//
int
faEVHDdbminor()
{
  if( ArteTable<EVHD>::size() ){
    ArtePointer<EVHD> p( 1 );
    return p->dbminor;
  }
  else {
    return -1;
  }
}

//
// flag
//
int
faEVHDflag()
{
  if( ArteTable<EVHD>::size() ){
    ArtePointer<EVHD> p( 1 );
    return p->flag;
  }
  else {
    return -1;
  }
}

//
// EVRC ************************************************************************
//
int
faPrintEVRC()
{
  if( ArteTable<EVRC>::size() ){
    int count = 0;
    for( ArteTable<EVRC>::iterator evrc = ArteTable<EVRC>::begin();
	 evrc != ArteTable<EVRC>::end();
	 ++evrc ){
      count++;
      cout << " EVRC: "
	   << count                                                << " "
	   << "ipaddr="           << IPaddr( evrc->ipaddr )        << " "
	   << "errmask=0x" << hex << evrc->errmask          << dec << " "
	   << "date="             << evrc->date                    << " "
	   << "cputime="          << evrc->cputime                 << " "
	   << "artevers="         << evrc->artevers                << endl;
      cout << "         "
	   << "alca=0x"    << hex << evrc->alca             << dec << " "
	   << "ottmean="          << evrc->ottmean                 << " "
	   << "ottwidth="         << evrc->ottwidth                << endl;
      cout << "         "
           << "cats=0x"    << hex << evrc->cats             << dec << " "
	   << "holmes=0x"  << hex << evrc->holmes           << dec << " "
	   << "ranger=0x"  << hex << evrc->ranger           << dec << " "
	   << "riter=0x"   << hex << evrc->riter            << dec << " "
	   << "rire=0x"    << hex << evrc->rire             << dec << " "
	   << "rise=0x"    << hex << evrc->rise             << dec << " "
	   << "care=0x"    << hex << evrc->care             << dec << " "
	   << "trd=0x"     << hex << evrc->trd              << dec << endl;
      cout << "         "
	   << "mureca=0x"  << hex << evrc->mureca           << dec << " "
	   << "murecb=0x"  << hex << evrc->murecb           << dec << " "
	   << "marple=0x"  << hex << evrc->marple           << dec << endl;   
    }
    return ARTE_GOOD;
  }
  else {
    return ARTE_ERR;
  }
}

//
// ipaddr
//
unsigned int
faEVRCipaddr ( int row )
{
  if( int ( ArteTable<EVRC>::size() ) < row ) return 0;
  ArtePointer<EVRC> evrc( row );
  return evrc->ipaddr;
}

//
// errmask
//
unsigned int
faEVRCerrmask( int row )
{
  if( int ( ArteTable<EVRC>::size() ) < row ) return 0;
  ArtePointer<EVRC> evrc( row );
  return evrc->errmask;
}

//
// date
//
int
faEVRCdate( int row )
{
  if( int ( ArteTable<EVRC>::size() ) < row ) return 0;
  ArtePointer<EVRC> evrc( row );
  return evrc->date;
}

//
// cputime
//
float
faEVRCcputime( int row )
{
  if( int ( ArteTable<EVRC>::size() ) < row ) return 0;
  ArtePointer<EVRC> evrc( row );
  return evrc->cputime;
}

//
// artevers
//
float
faEVRCartevers( int row )
{
  if( int ( ArteTable<EVRC>::size() ) < row ) return 0;
  ArtePointer<EVRC> evrc( row );
  return evrc->artevers;
}

//
// alca
//
unsigned int
faEVRCalca( int row )
{
  if( int ( ArteTable<EVRC>::size() ) < row ) return 0;
  ArtePointer<EVRC> evrc( row );
  return evrc->alca;
}

//
// EVTI ************************************************************************
//

//
// print EVTI-table
//
int
faPrintEVTI()
{
  if( ArteTable<EVTI>::size() ){
    int count = 0;
    for( ArteTable<EVTI>::iterator evti = ArteTable<EVTI>::begin();
	 evti != ArteTable<EVTI>::end();
	 ++evti ){

      // counter
      count++;

      // total time
      float sum =
	evti->vds    +
	evti->ranger +
	evti->refit  +
	evti->rich   +
	evti->ecal   +
	evti->trd    +
	evti->muon   +
	evti->marple +
	evti->pid    +
	evti->vertex;

      cout << " EVTI: "
	   << count                                          << " "
	   << "vds="    << evti->vds    << " "
	   << "ranger=" << evti->ranger << " "
	   << "refit="  << evti->refit  << " "
	   << "rich="   << evti->rich   << " "
	   << "ecal="   << evti->ecal   << " "
	   << "trd="    << evti->trd    << " "
	   << "muon="   << evti->muon   << endl;
      cout << "         "
	   << "marple=" << evti->marple << " "
	   << "pid="    << evti->pid    << " "
	   << "vertex=" << evti->vertex << " "
	   << "sum="    << sum          << endl;
    }
    return ARTE_GOOD;
  }
  else {
    return ARTE_ERR;
  }
}

// VDS
float
faEVTIvds( int row )
{
  if( int ( ArteTable<EVTI>::size() ) < row ) return 0;
  ArtePointer<EVTI> evti( row ); 
  return evti->vds;
}

// Ranger
float
faEVTIranger( int row )
{
  if( int ( ArteTable<EVTI>::size() ) < row ) return 0;
  ArtePointer<EVTI> evti( row ); 
  return evti->ranger;
}

// Refit
float
faEVTIrefit( int row )
{
  if( int ( ArteTable<EVTI>::size() ) < row ) return 0;
  ArtePointer<EVTI> evti( row ); 
  return evti->refit;
}

// RICH
float
faEVTIrich( int row )
{
  if( int ( ArteTable<EVTI>::size() ) < row ) return 0;
  ArtePointer<EVTI> evti( row ); 
  return evti->rich;
}

// ECAL
float
faEVTIecal( int row )
{
  if( int ( ArteTable<EVTI>::size() ) < row ) return 0;
  ArtePointer<EVTI> evti( row ); 
  return evti->ecal;
}

// TRD
float
faEVTItrd( int row )
{
  if( int ( ArteTable<EVTI>::size() ) < row ) return 0;
  ArtePointer<EVTI> evti( row ); 
  return evti->trd;
}

// MUON
float
faEVTImuon( int row )
{
  if( int ( ArteTable<EVTI>::size() ) < row ) return 0;
  ArtePointer<EVTI> evti( row ); 
  return evti->muon;
}

// Marple
float
faEVTImarple( int row )
{
  if( int ( ArteTable<EVTI>::size() ) < row ) return 0;
  ArtePointer<EVTI> evti( row ); 
  return evti->marple;
}

// ParteicleID
float
faEVTIpid( int row )
{
  if( int ( ArteTable<EVTI>::size() ) < row ) return 0;
  ArtePointer<EVTI> evti( row ); 
  return evti->pid;
}

// Vertex
float
faEVTIvertex( int row )
{
  if( int ( ArteTable<EVTI>::size() ) < row ) return 0;
  ArtePointer<EVTI> evti( row ); 
  return evti->vertex;
}

//
// DIHD ************************************************************************
//

//
// print DIHD-table
//
int
faPrintDIHD()
{
  if( ArteTable<DIHD>::size() ){
    ArtePointer<DIHD> dihd( 1 );
    cout << " DIHD: "
	 << "bx="            << dihd->bx          << " "
	 << "flt=0x"  << hex << dihd->flt  << dec << " "
	 << "slt=0x"  << hex << dihd->slt  << dec << " "
	 << "slb=0x"  << hex << dihd->slb  << dec << " "
	 << "cmsk=0x" << hex << dihd->cmsk << dec << " "
	 << "emsk=0x" << hex << dihd->emsk << dec << endl;
    return ARTE_GOOD;
  }
  else {
    return ARTE_ERR;
  }
}

//
// DEVT ************************************************************************
//

//
// event length
//
int
faDEVTlength()
{
  if( ArteTable<DEVT>::size() ){
    ArtePointer<DEVT> devt( 2 );
    int length = 0;
    ESP_get_int( (int *) &devt->bits, &length, 1 ); // [B]
    return length;
  }
  else {
    return -1;
  }
}

//
// DDAQ ************************************************************************
//

//
// print DDAQ-table
//
int
faPrintDDAQ()
{
  if( ArteTable<DDAQ>::size() ){
    ArtePointer<DDAQ> ddaq( 2 );
    struct EVT_DATA *p = (EVT_DATA *)( &ddaq->bits );

    int run_number = 0;
    ESP_get_int( &p->run_number, &run_number, 1 );
    
    int event_time = 0;
    ESP_get_int( &p->event_time, &event_time, 1 );
    
    int event_status = 0;
    ESP_get_int( &p->event_status, &event_status, 1 );
    
    int flt_status = 0;
    ESP_get_int( &p->flt_status, &flt_status, 1 );

    int slp_node = 0;
    ESP_get_int( &p->slp_node, &slp_node, 1 );

    int slt_status = 0;
    ESP_get_int( &p->slt_status, &slt_status, 1 );

    int exp_number = 0;
    ESP_get_int( &p->exp_number, &exp_number, 1 );

    int farm_node = 0;
    ESP_get_int( &p->farm_node, &farm_node, 1 );

    int BX_long = 0;
    ESP_get_int( &p->BX_long, &BX_long, 1 );

    short BX_upper = 0;
    ESP_get_short( &p->BX_upper, &BX_upper, 1 );

    short FCS_tcode = 0;
    ESP_get_short( &p->FCS_tcode, &FCS_tcode, 1 );

    short physBX = 0;
    ESP_get_short( &p->physBX, &physBX, 1 );

    short FLT_BX = 0;
    ESP_get_short( &p->FLT_BX, &FLT_BX, 1 );

    short fltevt = 0;
    ESP_get_short( &p->fltevt, &fltevt, 1 );

    cout << " DDAQ: " 
	 << "run_number="            << run_number                              << " "
	 << "event_time="            << event_time                              << " "
	 << "date="                  << faDate( (time_t *) &event_time )        << " "
	 << "time="                  << faTime( (time_t *) &event_time )        << " "
	 << "event_status=0x" << hex << event_status                     << dec << " "
	 << "flt_status=0x"   << hex << flt_status                       << dec << " "
	 << "slp_node="              << slp_node                                << " "
	 << "slt_status=0x"   << hex << slt_status                       << dec << " "
	 << "exp_number="            << exp_number                              << " "
	 << "farm_node="             << farm_node                               << " "
	 << "BX_long="               << BX_long                                 << " "
	 << "BX_upper="              << BX_upper                                << " "
	 << "FCS_tcode="             << FCS_tcode                               << " "
	 << "physBX="                << physBX                                  << " "
	 << "FLT_BX="                << FLT_BX                                  << " "
	 << "fltevt="                << fltevt                                  << endl;
    return ARTE_GOOD;
  }
  else {
    return ARTE_ERR;
  }
}

//
// daq run_number
//
int
faDDAQrun_number()
{
  if( ArteTable<DDAQ>::size() ){
    ArtePointer<DDAQ> ddaq( 2 );
    struct EVT_DATA *p = (EVT_DATA *)( &ddaq->bits );
    int run_number = 0;
    ESP_get_int( &p->run_number, &run_number, 1 );
    return run_number;
  }
  else {
    return -1;
  }
}

//
// daq event_time
//
int
faDDAQevent_time()
{
  if( ArteTable<DDAQ>::size() ){
    ArtePointer<DDAQ> ddaq( 2 ); 
    struct EVT_DATA *p = (EVT_DATA *)( &ddaq->bits );
    int event_time = 0;
    ESP_get_int( &p->event_time, &event_time, 1 );
    return event_time;
  }
  else {
    return -1;
  }
}

//
// daq event_status
//
int
faDDAQevent_status()
{
  if( ArteTable<DDAQ>::size() ){
    ArtePointer<DDAQ> ddaq( 2 ); 
    struct EVT_DATA *p = (EVT_DATA *)( &ddaq->bits );
    int event_status = 0;
    ESP_get_int( &p->event_status, &event_status, 1 );
    return event_status;
  }
  else {
    return -1;
  }
}

//
// daq flt_status
//
int
faDDAQflt_status()
{
  if( ArteTable<DDAQ>::size() ){
    ArtePointer<DDAQ> ddaq( 2 ); 
    struct EVT_DATA *p = (EVT_DATA *)( &ddaq->bits );
    int flt_status = 0;
    ESP_get_int( &p->flt_status, &flt_status, 1 );
    return flt_status;
  }
  else {
    return -1;
  }
}

//
// daq slp_node
//
int
faDDAQslp_node()
{
  if( ArteTable<DDAQ>::size() ){
    ArtePointer<DDAQ> ddaq( 2 ); 
    struct EVT_DATA *p = (EVT_DATA *)( &ddaq->bits );
    int slp_node = 0;
    ESP_get_int( &p->slp_node, &slp_node, 1 );
    return slp_node;
  }
  else {
    return -1;
  }
}

//
// daq slt_status
//
int
faDDAQslt_status()
{
  if( ArteTable<DDAQ>::size() ){
    ArtePointer<DDAQ> ddaq( 2 ); 
    struct EVT_DATA *p = (EVT_DATA *)( &ddaq->bits );
    int slt_status = 0;
    ESP_get_int( &p->slt_status, &slt_status, 1 );
    return slt_status;
  }
  else {
    return -1;
  }
}

//
// daq exp_number
//
int
faDDAQexp_number()
{
  if( ArteTable<DDAQ>::size() ){
    ArtePointer<DDAQ> ddaq( 2 ); 
    struct EVT_DATA *p = (EVT_DATA *)( &ddaq->bits );
    int exp_number = 0;
    ESP_get_int( &p->exp_number, &exp_number, 1 );
    return exp_number;
  }
  else {
    return -1;
  }
}

//
// daq farm_node
//
int
faDDAQfarm_node()
{
  if( ArteTable<DDAQ>::size() ){
    ArtePointer<DDAQ> ddaq( 2 ); 
    struct EVT_DATA *p = (EVT_DATA *)( &ddaq->bits );
    int farm_node = 0;
    ESP_get_int( &p->farm_node, &farm_node, 1 );
    return farm_node;
  }
  else {
    return -1;
  }
}

//
// daq BX_long
//
int
faDDAQBX_long()
{
  if( ArteTable<DDAQ>::size() ){
    ArtePointer<DDAQ> ddaq( 2 ); 
    struct EVT_DATA *p = (EVT_DATA *)( &ddaq->bits );
    int BX_long = 0;
    ESP_get_int( &p->BX_long, &BX_long, 1 );
    return BX_long;
  }
  else {
    return -1;
  }
}

//
// daq BX_upper
//
short
faDDAQBX_upper()
{
  if( ArteTable<DDAQ>::size() ){
    ArtePointer<DDAQ> ddaq( 2 ); 
    struct EVT_DATA *p = (EVT_DATA *)( &ddaq->bits );
    short BX_upper = 0;
    ESP_get_short( &p->BX_upper, &BX_upper, 1 );
    return BX_upper;
  }
  else {
    return -1;
  }
}

//
// daq FCS_tcode
//
short
faDDAQFCS_tcode()
{
  if( ArteTable<DDAQ>::size() ){
    ArtePointer<DDAQ> ddaq( 2 ); 
    struct EVT_DATA *p = (EVT_DATA *)( &ddaq->bits );
    short FCS_tcode = 0;
    ESP_get_short( &p->FCS_tcode, &FCS_tcode, 1 );
    return FCS_tcode;
  }
  else {
    return -1;
  }
}

//
// daq physBX
//
short
faDDAQphysBX()
{
  if( ArteTable<DDAQ>::size() ){
    ArtePointer<DDAQ> ddaq( 2 ); 
    struct EVT_DATA *p = (EVT_DATA *)( &ddaq->bits );
    short physBX = 0;
    ESP_get_short( &p->physBX, &physBX, 1 );
    return physBX;
  }
  else {
    return 0;
  }
}

//
// daq FLT_BX
//
short
faDDAQFLT_BX()
{
  if( ArteTable<DDAQ>::size() ){
    ArtePointer<DDAQ> ddaq( 2 ); 
    struct EVT_DATA *p = (EVT_DATA *)( &ddaq->bits );
    short FLT_BX = 0;
    ESP_get_short( &p->FLT_BX, &FLT_BX, 1 );
    return FLT_BX;
  }
  else {
    return -1;
  }
}

//
// daq fltevt
//
short
faDDAQfltevt()
{
  if( ArteTable<DDAQ>::size() ){
    ArtePointer<DDAQ> ddaq( 2 ); 
    struct EVT_DATA *p = (EVT_DATA *)( &ddaq->bits );
    short fltevt = 0;
    ESP_get_short( &p->fltevt, &fltevt, 1 );
    return fltevt;
  }
  else {
    return -1;
  }
}

//
// daq dbKey
//
int
faDDAQdbKey()
{
  if( ArteTable<DDAQ>::size() ){
    ArtePointer<DDAQ> ddaq( 2 ); 
    struct EVT_DATA *p = (EVT_DATA *)( &ddaq->bits );
    int dbKey = 0;
    ESP_get_int( &p->dbKey, &dbKey, 1 );
    return dbKey;
  }
  else {
    return -1;
  }
}

//
// daq dbVersion
//
int
faDDAQdbVersion()
{
  if( ArteTable<DDAQ>::size() ){
    ArtePointer<DDAQ> ddaq( 2 ); 
    struct EVT_DATA *p = (EVT_DATA *)( &ddaq->bits );
    int dbVersion = 0;
    ESP_get_int( &p->dbVersion, &dbVersion, 1 );
    return dbVersion;
  }
  else {
    return -1;
  }
}

// DTAR ************************************************************************
//

//
// print DTAR-table
//
int
faPrintDTAR()
{
  if( ArteTable<DTAR>::size() ){
    ArtePointer<DTAR> dtar( 2 ); //  pointer to DTAR
    char *p = (char *) &dtar->bits + sizeof( SLB_SUBHDR );  // pointer to data
    struct TGTS_RAW tgts; // target data structure
    ESP_get_int( (int *) p, (int *) &tgts, sizeof( TGTS_RAW ) / sizeof( int ) ); // swap

    // print
    cout << " DTAR: " 
	 << "date="   << faDate( (time_t *) &tgts.time ) << " "
	 << "time="   << faTime( (time_t *) &tgts.time ) << " "
	 << "ia="     << tgts.ia                         << "MHz "
	 << "bg="     << tgts.bg * 1000                  << "kHz "
	 << "pos[0]=" << tgts.pos[0]                     << "mm "
	 << "pos[1]=" << tgts.pos[1]                     << "mm "
	 << "pos[2]=" << tgts.pos[2]                     << "mm "
	 << "pos[3]=" << tgts.pos[3]                     << "mm "
	 << "pos[4]=" << tgts.pos[4]                     << "mm "
	 << "pos[5]=" << tgts.pos[5]                     << "mm "
	 << "pos[6]=" << tgts.pos[6]                     << "mm "
	 << "pos[7]=" << tgts.pos[7]                     << "mm"   << endl;

    return ARTE_GOOD;
  }
  else {
    return ARTE_ERR;
  }
}

//
// targets ([MHz] and [mm])
//
void
faDTARtgts( TGTS_RAW *tgts )
{
  if( tgts && ArteTable<DTAR>::size() ){
    ArtePointer<DTAR> dtar( 2 ); // pointer to DTAR
    char *p = (char *) &dtar->bits + sizeof( SLB_SUBHDR ); // pointer to data
    ESP_get_int( (int *) p, (int *) tgts, sizeof( TGTS_RAW ) / sizeof( int ) ); //swap
  }
}

//
// interaction rate
//
float
faDTARia()
{
  if( ArteTable<DTAR>::size() ){
    ArtePointer<DTAR> dtar( 2 ); 
    char *p = (char *) &dtar->bits + sizeof( SLB_SUBHDR );
    struct TGTS_RAW tgts;
    ESP_get_int( (int *) p, (int *) &tgts, sizeof( TGTS_RAW ) / sizeof( int ) );  
    return tgts.ia;
  }
  else {
    return -1;
  }
}

//
// interaction rate
//
extern "C" float
faDTARia_()
{
  return faDTARia();
}


//
// background rate
//
float
faDTARbg()
{
  if( ArteTable<DTAR>::size() ){
    ArtePointer<DTAR> dtar( 2 );
    char *p = (char *) &dtar->bits + sizeof( SLB_SUBHDR );
    struct TGTS_RAW tgts;
    ESP_get_int( (int *) p, (int *) &tgts, sizeof( TGTS_RAW ) / sizeof( int ) );     
    return tgts.bg;
  }
  else {
    return -1;
  }
}

//
// background rate
//
extern "C" float
faDTARbg_()
{
  return faDTARbg();
}

// ****************************************************************************
//
// name:      faNameIDInit
//
// purpose:   initialize Name/ID array
//
// input:     void
// return:    int  lenght of array
//
// hints:     component.h
//
// **************************************************************************** 

const int iNameIDLen = 16; // 0 - 15
int iNameID[iNameIDLen];

int
faNameIDInit()
{
  memset( &iNameID, 0, iNameIDLen );
  iNameID[COMP_ID_DAQ /0x0100] = faName2Int( "DDAQ" ); //  0
  iNameID[COMP_ID_SVD /0x0100] = faName2Int( "DSVD" ); //  1
  iNameID[COMP_ID_ITR /0x0100] = faName2Int( "DITR" ); //  2
  iNameID[COMP_ID_HIPT/0x0100] = faName2Int( "DPTC" ); //  3
  iNameID[COMP_ID_OTR /0x0100] = faName2Int( "DOTR" ); //  4
  iNameID[COMP_ID_RICH/0x0100] = faName2Int( "DRIC" ); //  5
  iNameID[COMP_ID_TRD /0x0100] = faName2Int( "DTRD" ); //  6
  iNameID[COMP_ID_ECAL/0x0100] = faName2Int( "DCAL" ); //  7
  iNameID[COMP_ID_MUON/0x0100] = faName2Int( "DMUT" ); //  8
  iNameID[COMP_ID_MPIX/0x0100] = faName2Int( "DMUX" ); //  9
  iNameID[COMP_ID_MPAD/0x0100] = faName2Int( "DMUP" ); // 10
  iNameID[COMP_ID_FLT /0x0100] = faName2Int( "DFLT" ); // 11
  iNameID[COMP_ID_SLT /0x0100] = faName2Int( "DSLT" ); // 12
  iNameID[COMP_ID_TLT /0x0100] = faName2Int( "DTLT" ); // 13
  iNameID[COMP_ID_TARG/0x0100] = faName2Int( "DTAR" ); // 14
  iNameID[COMP_ID_FCS /0x0100] = faName2Int( "DFCS" ); // 15

#ifdef _DEB2_
  if( FADEB ){
    for( int i=0; i<iNameIDLen; i++ ){
      cout << "faNameIDInit:"
	   << "iNameID[" << i      << "]="
	   << iNameID[i]           << " "
	   << faName( iNameID[i] ) << endl;
    }
  }
#endif
  return iNameIDLen;
}

// ****************************************************************************
//
// name:      faID2Name
//
// purpose:   convert ID to table name
//
// input:     ID
// return:    table name
//
// **************************************************************************** 

int
faID2Name( int ID )
{
  // initialize array once
  static int b = 1;
  static int l = 0;
  if( b ){
    b = 0;
    l = faNameIDInit();
  }

  int i = ID / 0x0100; // index
  if( i >= l ){
    cout << "faID2Name(): Fatal index i=" << i << endl;
    return ARTE_ERR;
  }
#ifdef _DEB2_
  if( FADEB ){
    cout << "faID2Name(): "
	 << "ID=0x" << hex << ID << dec << " " 
	 << "iNameID[" << i << "]=" << iNameID[i] << endl;
  }
#endif
  return iNameID[i];
}

//
//  functions
//
int
faName2Int( char *c )
{
  return *( (int *) c );
}

char *
faName( int i )
{
  static char c[5];
  strncpy( c, (char *) &i, 4 );
  return c;
}

// *****************************************************************************
//
// name:      faDate
//
// purpose:   providing current date
//
// called by: anywhere
//
// input:     seconds since Unix 0
// return:    int date yymmdd
//
// *****************************************************************************

int
faDate( time_t *tp )
{
  if( tp == 0 ){
    time_t tmp;
    tp = &tmp;
    time( tp );
  }

  struct tm *t = localtime( tp );
  
  char s[9];
  memset( s, 0, 9 );
  strftime( s, 9, "%Y%m%d", t );

  return atoi( s );
}

// *****************************************************************************
//
// name:      faTime
//
// purpose:   providing current time
//
// called by: anywhere
//
// input:     seconds since Unix 0
// return:    int date hhmmss
//
// *****************************************************************************

int
faTime( time_t *tp )
{
  if( tp == 0 ){
    time_t tmp;
    tp = &tmp;
    time( tp );
  }

  struct tm *t = localtime( tp );
  
  char s[7];
  memset( s, 0, 7 );
  strftime( s, 7, "%H%M%S", t );
  return atoi( s );
}

// *****************************************************************************
//
// name:      fatime_
//
// purpose:   providing timing information
//
// called by: anywhere
//
// input:     void
// return:    float elapsed user_time
//
// *****************************************************************************

extern "C" float
fatime_()
{
  static int first = 1;
  static clock_t clock0;
  static time_t  time0;
  
  if( first ){
    first  = 0;
    clock0 = clock();
    time0  = time( 0 );
  }
  
  // double user_time = ( clock() - clock0 ) / double( CLOCKS_PER_SEC );
  double real_time = difftime( time( 0 ), time0 );
  
  // return float( user_time );
  return float( real_time );
}

// *****************************************************************************
//
// name:      faCopySlb
//
// purpose:   providing timing information
//
// called by: anywhere
//
// input:     void
// return:    float elapsed user_time
//
// *****************************************************************************
int
faCopySlb( int *oldRecID, char *pData , int *newRecID, char *pTable, int nData )
{

#ifdef _DEB2_
  // print data
  cout << "faCopySlb(): oldRecID=0x" << hex << *oldRecID << dec << endl;
  cout << "faCopySlb(): nData="             << nData            << endl;
  cout << "faCopySlb(): Data: ";
  for( int i=0; i<nData; i++ ) {
    cout << "0x" << hex << int( *( pData + i ) ) << dec << " ";
  }
  cout << endl;
#endif

  // pre-set
  int nWrite = 0;

  // no data
  if( nData == 0 ){
    int error = RID_ERROR_SPARSE;
    ESP_put_int( &error, newRecID, 1 );
    return nWrite;
  }

  // rec ID
  ESP_put_int( oldRecID, newRecID, 1 );

  // data
  nWrite = nData;

  // just copy byte-wise
  for( int i=0; i<nData; i++ ) {
    *( pTable + i ) = *( pData + i );
  }

#ifdef _DEB2_
  if( FADEB ){
    int recID = 0;
    ESP_get_int( newRecID, recID, 1 );
    cout << "faCopySlb(): newRecID=0x" 
	 << hex << *recID << dec << endl;
    cout << "faCopySlb(): nWrite="
	 << nWrite << endl;
  }
#endif
  
  return nWrite;
}
