/************************************************************************
* written by Siegmund Nowak, 27/10/94 13.00.07
* changed by Olya Igonkina for fast shower algorithm    
*
* This routines fills the GEANT HIT  banks
*
* called from GUSTEP
*
*     IPROC  variable giving what kind of MIMP/MCAL  will be generated
*            1 - normal MIMP (gean copy number = GEDE bank number)
*            2 - MCAL for calorimeter
*            3 - MCAL for scintillation hodoscopes
*            4 - MIMP for TRD (long version)  
*            5 - MTCP start point of cascades in ECAL
*            6 - modular MIMP (gean copy number packed - valid if moduls exist 
************************************************************************/
#include <vector.h>
#include <stdio.h>
#include "arte/arte.hh"
#include "gean/gctrak.h"
#include "gean/gckine.h"
#include "gean/gcsets.h"
#include "gean/gcvolu.h"
#include "gean/ZGMOTH.h"
#include "gean/zgkine.hh"
#include "gean/zgmimp.hh"

extern vector<hgmtra> *hg_mtra;
extern vector<hgmimp> *normal_mimp; 
extern vector<hgmimp> *modul_mimp;
extern vector<hgmimp> *TRD_mimp;
extern vector<hgmcal> *TR_mimp;
extern vector<hgmtcp> *ECA_ep;
extern vector<hgmcal> *ECA_mimp;

void crhibk(int iproc) {

//    some useful quantity
      vector<hgmtra>::iterator mt = hg_mtra->begin() + gckine_.itra -1;   

//    OTR-, normal and modular mimps 
      if(iproc == 8 || iproc == 1  || iproc == 6 || iproc == 7) {  
         hgmimp mm;
         mm.z = gctrak_.vect[2] - zgmoth_.ZZERO;
         mm.x = gctrak_.vect[0];
         mm.y = gctrak_.vect[1];
         
         if(gctrak_.vect[5]) {
            mm.tx = gctrak_.vect[3]/gctrak_.vect[5];
            mm.ty = gctrak_.vect[4]/gctrak_.vect[5];
            if(gctrak_.vect[5] >= 0.)  mm.p =  gctrak_.vect[6]; 
            else                       mm.p = -gctrak_.vect[6]; 
         }
         else                {
            if(gctrak_.vect[3] >= 0.)    mm.tx =  1.e+10;
            else                         mm.tx = -1.e+10;
            if(gctrak_.vect[4] >= 0.)    mm.ty =  1.e+10;
            else                         mm.ty = -1.e+10;  
            mm.p = gctrak_.vect[6];                      
         }

         mm.time = gctrak_.tofg;
         mm.itra = gckine_.itra;

         if     (iproc == 8) {        // OTR mimps   
            mm.gede = gcsets_.numbv[0]/10;
            normal_mimp->push_back(mm); 
         }
         else if     (iproc == 1) {        // normal mimps   
            mm.gede = gcsets_.numbv[0];
            normal_mimp->push_back(mm); 
         }
         else if(iproc == 6) {        // modular mimps
            int nlev = gcvolu_.nlevel -1;

            unsigned int inum = gcvolu_.number[nlev-2] << 24;
            unsigned int inum1 = gcvolu_.number[nlev-1]&4095;
            inum1 <<= 12;            inum += inum1;
            inum1 = gcvolu_.number[nlev]&4095;
            mm.gede = inum + inum1;
            modul_mimp->push_back(mm); 
         }
         else if(iproc == 7) {        // TRD_mimps
            mm.gede = gcsets_.numbv[0];
            TRD_mimp->push_back(mm);
         } 
      }


//    TR-MIMPS
      else if(iproc == 4)  {       // TR mimps
         if(gctrak_.destep <= 0.) return;

//       check whether volume element is the same as previous TRD_mimp
//       loop over all TR_mimps
         vector<hgmcal>::size_type np = TR_mimp->size();
         vector<hgmcal>::iterator me;
         int n = int(np);  
         for(int i = 1;  i<=n; i++){
            me = TR_mimp->begin() + n - i;      
            if(gcsets_.numbv[0] == me->gcal) {
                me->de += gctrak_.destep * 1.e6; 
                return;
            } 
         }

//    first track or TR_mimp with different volume
            hgmcal mm;
            memset(&mm,0,sizeof(mm));
            mm.de = gctrak_.destep * 1.e6;
	    mm.gcal = gcsets_.numbv[0];
            TR_mimp->push_back(mm);
            mt->nhit++ ;
            return;
      }

//    calorimeter entry points     
      else if(iproc == 5) {
         hgmtcp ep; 
         ep.z = gctrak_.vect[2] - zgmoth_.ZZERO;
         ep.x = gctrak_.vect[0];
         ep.y = gctrak_.vect[1];
         if(gctrak_.vect[5]) {
            ep.tx = gctrak_.vect[3]/gctrak_.vect[5];
            ep.ty = gctrak_.vect[4]/gctrak_.vect[5];
            if(gctrak_.vect[5] >= 0.)  ep.p =  gctrak_.vect[6]; 
            else                       ep.p = -gctrak_.vect[6]; 
         }
         else                {
            if(gctrak_.vect[3] >= 0.)  ep.tx = 1.e+10;  else  ep.tx = -1.e+10;
            if(gctrak_.vect[4] >= 0.)  ep.ty = 1.e+10;  else  ep.ty = -1.e+10;
         } 
         ep.itra = gckine_.itra;
         ECA_ep->push_back(ep);  
         vector<hgmtcp>::size_type np = ECA_ep->size();
         int n = int(np); 
         mt->ep = n;
      } 


//    calorimeter or scintillation counters
      else if(iproc == 2  || iproc == 3) {
         if(gctrak_.destep <= 0.) return;
         vector<hgmcal>::iterator me;     hgmcal ca;       int ivn=0 ; 
//       calorimeter 
         if(iproc == 2) {
            ArtePointer<GCAL> gl(gcsets_.numbv[0]);      
            int nlev = gcvolu_.nlevel -1;
            int inum = gcvolu_.number[nlev-3] << 24; 
            ivn = inum + 100*gcsets_.numbv[0] + gcsets_.numbv[1]*gl->ncl 
                                              + gcsets_.numbv[2];
         }
//          scintillation counters
         else if(iproc == 3) {           
            int nlev = gcvolu_.nlevel -1;
	    unsigned int inum = gcvolu_.number[nlev-1]&4095;    inum <<= 24;    
	    unsigned int inum1 = gcsets_.numbv[0]&4095;  ivn = inum + inum1;
         }

//       check whether track number is the same as previous MCAL
//       loop over all calorimeter hits

         vector<hgmcal>::size_type np = ECA_mimp->size();
         int n = int(np);  
         for(int i = 1;  i<=n; i++){
            me = ECA_mimp->begin() + n - i;      
            if(gckine_.itra != me->itra) {
               ca.de = gctrak_.destep;
               ca.itra = gckine_.itra;
               ca.gcal = ivn;
               ECA_mimp->push_back(ca);
               mt->nhit++ ;
               return;  
            }

//          check whether it is the same volume element, if so add energy
            if(ivn == me->gcal) { 
                me->de += gctrak_.destep; return;
            } 
         }

//       first track, but all MCALs have diff volumina (or first MCAL)
         ca.de = gctrak_.destep;
         ca.itra = gckine_.itra;
         ca.gcal = ivn;
         ECA_mimp->push_back(ca);
     }

     mt->nhit ++;
    
}
