/************************************************************************
*
* SZ2GEO
* written by Siegmund Nowak
*
* This routines is the geometry installation routine for dosimeters
* working together with the Geometry Data Language 
*
*       called from G..MAIN
*
************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <math.h>
#include "arte/arte.hh"
#include "arte/ArteFlag.h"
#include "arte/ArteMessage.hh"
#include "arte/ArteUtil.h"
#include "gean/zgeom.h"
#include "gean/zgvers.hh"
#include "gean/ZGESL.h"


  extern "C" {  
    void sz2geo_(); 
    void gsatt_(char*,char*,int*);  
    void gsdet_(char*,char*,int*,char*,int*,int*,int*,int*,int*,int*);}
    int  read_dat(FILE *,int &);  
    void szh2_arte(const int);   

    FILE  *utfilp(const char*,const char*);

void sz2geo_() {
    extern int first_geo[32];
    static char filnam[80];  static FILE *GEO_inp[32];  
    zgrot ZGROT; zgmed ZGMED; zgvol ZGVOL; zgpos ZGPOS; zgdet ZGDET; 
    zgvers ZGVERS; 

    ArtePointer<NESL> sl(zgmoth_.CURRSL);   
    int cmp = sl->cmp;
    zgeom ZGEOM(cmp);   
    if(first_geo[cmp-1] == 0) {

//     select file name and  open the file
       strcpy(filnam,ZGVERS.Get_vers_datafile(cmp));         
       GEO_inp[cmp-1] = utfilp(filnam,"");
    
//     reserve storage for zgrot, zgvol and zgpos 
       ZGDET.ini(24); ZGROT.ini(6); ZGVOL.ini(32); ZGPOS.ini(128);
       first_geo[cmp-1] = 1;         }

    int icopy =  ArteTable<GCAL>::size();

//  now the card handling begins
    for (;;)   {
//  read in one media, rotation or volume block
     int datagroup=0; int nr=0; 
     datagroup = read_dat(GEO_inp[cmp-1],nr); 

//   no action 
     if (datagroup == 0) continue;                          

//   media
     if (datagroup == 1) {  ZGMED.Create_Gmed();  continue; }

//   rotations
     if (datagroup == 2) { ZGROT.Create_Grot(nr);  continue; }

     char volname[5];
     int nr_vol=nr;    
     strncpy(volname,ZGVOL.Get_volname(nr_vol),5);

//   components or modules
     if (datagroup == 3 ) {   
       if(ZGVOL.Get_sens_flag(nr_vol) != 0 ) {
//      check whether the volume already exists
        int Gvolnum = ZGVOL.Check_Gvol(volname); 
        if (Gvolnum == 0) {
            ZGVOL.Create_Gvol(nr_vol);
            if(ZGVOL.Get_sens_flag(nr_vol) > 2 ) {
               char set[5];  
               if     (cmp==LGESL_HOD2)  strcpy(set ,"SZH2"); 
               int iset,idet; int i1=1, i20=20, i32=32, i50=50; 
               gsdet_(set,volname,&i1,volname,&i32,&cmp,
                      &i50,&i20,&iset,&idet);}}}}


//    superlayers
      if(datagroup == 5 || datagroup == -99)    {
         char cname[5];
         ZGPOS.Gposit_superlayer(cname,nr_vol,icopy);
	 szh2_arte(nr_vol); break; }

     if ( datagroup == -99 )  {                              //end of file
       ZGDET.fin(); ZGROT.fin(); ZGVOL.fin(); ZGPOS.fin();
       break;  }                      
   }}
  
/* sz2geo_ */



