/*************************************************************************
*
* SZH2_ARTE
* written by Siegmund Nowak
*
* This routines defines ARTE-banks for dosimeters
*
*       called from G..MAIN
*
*************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <math.h>
#include "arte/arte.hh"
#include "arte/ArteUtil.h"
#include "arte/ArteMessage.hh"
#include "gean/gclink.h"
#include "gean/gcvolu.h"
#include "gean/zgeom.h"
#include "gean/ZGESL.h"

extern "C" {
    int  matnum_(char*,int);
}

void szh2_arte(const int nr_sl) {
/***********************/

//  define NCAL banks
    struct NCAL gc;
    zgvol ZGVOL;
    zgpos ZGPOS;

//  fill NCAL table
    int type, gnr; 
    float par[3], xp, yp, zp;
    char volume[5], material[13]; volume[4] = '\0'; material[12] = '\0'; 
    int nr0 = nr_sl; 
    int nr = ZGVOL.Get_ncomp(nr0);
    for(int i=0;i<nr;i++) {
       ArtePointer<NESL> currsl(zgmoth_.CURRSL);
       strncpy(volume,ZGPOS.Get_posname(nr0,i+1,type),4);
       int j = ZGVOL.Get_volnr(volume,type);
       if(j>=0)  {ZGVOL.Get_par(j,par);
         ZGPOS.Get_RotPos(nr0,i,xp,yp,zp,gnr);
         int nncal = ArteTable<NCAL>::size() + 1;
         memset(&gc,0,sizeof(gc));  
         if(!currsl->fpl) currsl->fpl =  nncal;
         currsl->npl = currsl->npl +1 ;
         gc.cmp = currsl->cmp;
         gc.xmin = xp - par[0];   gc.xmax = xp + par[0];
         gc.ymin = yp - par[1];   gc.ymax = yp + par[1];
         gc.zmin = zp - par[2];   gc.zmax = zp + par[2];

         ArtePointer<GMAT> gm(ZGVOL.Get_imat(j));
         float xms = (gc.zmax-gc.zmin)/gm->radl; 
         gc.ms = xms*pow(0.0136*(1.+ 0.038*log(double(xms))),2.);         
         gc.ncl = 1;
         ArteTable<NCAL>::push_back(gc);         

         createRelation(currsl,ArtePointer<NCAL>(nncal),&NESL::ncal);          
	 createRelation(gm,    ArtePointer<NCAL>(nncal),&GMAT::ncal);
      }
    } 
  }
      
/* szh2_arte_ */
