
      SUBROUTINE WTLSTR (ISL,ISEC,ZP,ITYP,IVOLS,IRHC)

************************************************************************
* WTLSTR
* written by Siegmund Nowak 3/93
*
* This routine posits single plates of WT-superlayers both in GEANT and
* RECO geometry structures
*
* Arguments  kind         description
*          (in,update,out)
*-----------------------------------------------------------------------
* ISL        in           WT-superlayer
* ISEC       in           sector inside superlayer
* ZP         in           starting z-position
* ITYP       in           = 6*IMFLAG+ITYP0
*                         The following types are valid
*                         ITYP0 = 1 = ITYP1    sensitive wires (10 mm)
*                               = 2 = ITYP1    sensitive wires ( 5 mm)
*                               = 3   ITYP1=2  nonsens   wires ( 5 mm)
*                         IMFLAG = 0     no magnetic field
*                                = 1     magnetic field
*                         LAYTYP = 0     single layers
*                                = 1     double layers
*                         ISEC  sector number
* IVOLS      in           volumen number in Geant conventions
* IRHC       in           flag for rohacel wand
*                         = 1     only front rohacel wall wanted
*                         = 0     front and end rohacel wall wanted
*                         = -1    only end rohacel wall wanted
************************************************************************

      IMPLICIT NONE

      INTEGER IFIRST,IR,ISL,ITYP,ITYP0,IMFLAG,LAYTYP,ITYPX,IWAND,IWTR,
     +        IRHC,JSG(11),K,NPL,IVOLS(*),ISEC,ITYP1,NSL,IUCOMP
      REAL COSANG,SINANG,TANANG,COSX,SINX,PAR(6),THICK,
     +     XP,YP,ZP,XMS,XMIN,XMAX,YMIN,YMAX,XMSROH
      CHARACTER*4 WTWAND(2),MUTT
      SAVE IFIRST
#include "arte/Arte.fi"
#include "gean/gclink.fi"
#include "gean/gcflag.fi"
#include "gean/ZGMOTH.fi"
#include "gean/ZGOTR.fi"
#include "auto/GEO.fi"
      DATA WTWAND /'WTWA','WTWM'/
      DATA IFIRST/0/
#include "arte/Arte.ff"
#include "auto/GEO.ff"

      IF(IFIRST.EQ.0) THEN
         COSANG = COS(ANGWT * 0.01745329)
         SINANG = SIN(ANGWT * 0.01745329)
         TANANG = SINANG/COSANG
         IFIRST = 1
         IWAND  = 0
         CALL VZERO(JSG,11)
      ENDIF

      CALL UHTOC(HNESL_NAME(ISL),4,MUTT,4)
      NSL = IUCOMP(MUTT,DEWT,NSLWT)
      IF(NSL.EQ.0) CALL MEERRO('WTLSTR: '//MUTT//' not found in DEWT')

      NPL = INESL_LAY(ISL)
      IWTR = IROWT(NSL)

      IMFLAG = (ITYP-1)/6
      ITYP0 = ITYP - 6*IMFLAG

      XMIN = XLWT(NSL,ISEC,1)
      XMAX = XRWT(NSL,ISEC,1)
      YMIN = YLWT(NSL,ISEC)
      YMAX = YRWT(NSL,ISEC)

C     front rohacell wand

      IF(IRHC.GE.0) THEN
         PAR(1) = .5 * ((XMAX-XMIN) + ABS(TANANG)*(YMAX-YMIN))
         PAR(2) = .5 * (YMAX-YMIN)
         PAR(3) = .5 * DLWAWT
         ZP = ZP + PAR(3)
         IWAND = IWAND + 1
         CALL GSPOSP(WTWAND(IMFLAG+1),IWAND,MUTT,.5*(XMIN+XMAX),
     +                 .5*(YMIN+YMAX),ZP,0,'ONLY',PAR,3)
         ZP = ZP +  PAR(3)
      ENDIF
      CALL GRADLN(IVOLS(10+IMFLAG),XMSROH)
      IF(IRHC.EQ.0) XMSROH = 2.*XMSROH
      XMSROH = dlwawt/XMSROH/NPL

c     planes

      PAR(2) = .5 * (YMAX-YMIN)
      PAR(5) = 0.
      PAR(6) = 0.
      YP = .5 * (YMIN+YMAX)

C     loop over planes

      DO 100 K=1,NPL
      IR =  IWTR / 10**(NPL-K)
      IWTR = IWTR - IR*10**(NPL-K)
      LAYTYP = IR/3
      IR = IR - 3*LAYTYP + 1
      ITYPX = ITYP
      IF(ITYP0.LE.2) THEN
        THICK = DLTOWT(2*LAYTYP+ITYP0)
        ITYP1 = ITYP0
        IF(LAYTYP.NE.0) ITYPX = ITYP0 + 3
        GO TO 5
      ELSE IF(ITYP0.EQ.3) THEN
        THICK = DLTOWT(2*LAYTYP+2)
        ITYPX = 6*IMFLAG + 3
        IF(LAYTYP.NE.0) ITYPX = 6
        ITYP1 = 2
      ENDIF
 5    CONTINUE
      GO TO (10,20,30),IR

C     ZERO DEGREE CHAMBERS

 10   XMIN = XLWT(NSL,ISEC,1)
      XMAX = XRWT(NSL,ISEC,1)
      XP = .5 * (XMAX+XMIN)
      PAR(1) = .5 * (XMAX-XMIN)
      PAR(4) = 0.
      COSX = 1.
      SINX = 0.
      GO TO 80

C     angwt DEGREE CHAMBERS

 20   XMIN = XLWT(NSL,ISEC,2)
      XMAX = XRWT(NSL,ISEC,2)
      XP = .5 * ((XMIN+XMAX)+TANANG*(YMIN+YMAX))
      PAR(1) = .5 * (XMAX-XMIN)
      PAR(4) = ANGWT
      COSX = COSANG
      SINX = SINANG
      GO TO 80

C      - angwt degree chambers

 30   XMIN = XLWT(NSL,ISEC,2)
      XMAX = XRWT(NSL,ISEC,2)
      XP = .5 * ((XMIN+XMAX)-TANANG*(YMIN+YMAX))
      PAR(1) = .5 * (XMAX-XMIN)
      PAR(4) = -ANGWT
      COSX = COSANG
      SINX = -SINANG

 80   CONTINUE
      JSG(ITYPX) = JSG(ITYPX) + 1
      PAR(3) = 0.5 * THICK
      ZP = ZP + PAR(3)

      IF(ITYP0.GT.2) THEN
      CALL GSPOSP(WTDT(ITYPX),JSG(ITYPX),MUTT,XP,YP,ZP,
     +                                            0,'ONLY',PAR,6)
      ELSE IF (ITYP0.LE.2) THEN

         CALL CRPOS(WTDT(ITYPX),MUTT,K,ISEC,XP,YP,ZP,0,'ONLY',PAR)

         XMS=0.

         Q(QNEDE_ACC_L()) = SIGWT(ITYP1)
         Q(QNEDE_EFF_L()) = EFFWT
         Q(QNEDE_PIT_L()) = PITWT(ITYP1)
         Q(QNEDE_DOD_L()) = DIMXWT(ITYP1)
         Q(QNEDE_SXMN_L()) = FNEDE_XMIN(NNEDE())
         Q(QNEDE_SXMX_L()) = FNEDE_XMAX(NNEDE())
         Q(QNEDE_SYMN_L()) = FNEDE_YMIN(NNEDE())
         Q(QNEDE_SYMX_L()) = FNEDE_YMAX(NNEDE())

         IQ(IQNEDE_NSL_L())  = 2*(LAYTYP+1)
         Q(QNEDE_ZFW_L(1))=FNEDE_ZMIN(NNEDE())+DLWEWT+.5*DLCLWT(ITYP1)
         Q(QNEDE_ZFW_L(2)) = FNEDE_ZFW(1,NNEDE())+ 0.5*DLCLWT(ITYP1)
         IF(XMIN.LE.0.) THEN
            Q(QNEDE_XFW_L(1)) = XMIN*COSX + 0.3333*PITWT(ITYP1)
            Q(QNEDE_XFW_L(2)) = FNEDE_XFW(1,NNEDE())+.5*PITWT(ITYP1)
         ELSE
            Q(QNEDE_XFW_L(2)) = XMIN*COSX + 0.3333*PITWT(ITYP1)
            Q(QNEDE_XFW_L(1)) = FNEDE_XFW(2,NNEDE())+.5*PITWT(ITYP1)
         ENDIF
         IQ(IQNEDE_NWI_L(1)) = COSX*(XMAX-XMIN)/PITWT(ITYP1)-0.1665
         IF(ISEC.GT.2) THEN
            IQ(IQNEDE_NWI_L(2)) = INEDE_NWI(1,NNEDE())
         ELSE
            IQ(IQNEDE_NWI_L(1)) = INEDE_NWI(1,NNEDE()) + 1
            IQ(IQNEDE_NWI_L(2)) = INEDE_NWI(1,NNEDE()) - 1
         ENDIF
         IF(LAYTYP.EQ.1) THEN
            Q(QNEDE_XFW_L(3))  = FNEDE_XFW(1,NNEDE())
            Q(QNEDE_ZFW_L(3))  = FNEDE_ZFW(2,NNEDE())+ 0.5*DLCLWT(ITYP1)
            Q(QNEDE_XFW_L(4))  = FNEDE_XFW(2,NNEDE())
            Q(QNEDE_ZFW_L(4))  = FNEDE_ZFW(3,NNEDE())+ 0.5*DLCLWT(ITYP1)
            IQ(IQNEDE_NWI_L(3)) = INEDE_NWI(1,NNEDE())
            IQ(IQNEDE_NWI_L(4)) = INEDE_NWI(2,NNEDE())
         ENDIF

         CALL GRADLN(IVOLS(ITYPX),XMS)
         XMS = THICK/XMS + XMSROH
         Q(QNEDE_MS_L())= XMS*(.0136*(1.+.038*ALOG(XMS)))**2
      ENDIF

      ZP = ZP + PAR(3)
 100  CONTINUE

C     end rohacell wand
      IF(IRHC.LE.0) THEN
         PAR(1) = .5 * ((XMAX-XMIN) + ABS(TANANG)*(YMAX-YMIN))
         PAR(2) = .5 * (YMAX-YMIN)
         PAR(3) = .5 * DLWAWT
         ZP = ZP + PAR(3)
         IWAND = IWAND + 1
         CALL GSPOSP(WTWAND(IMFLAG+1),IWAND,MUTT,.5*(XMIN+XMAX),
     +             .5*(YMIN+YMAX), ZP,0,'ONLY',PAR,3)
         ZP = ZP + PAR(3)
      ENDIF
 90   RETURN

      END


