#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <vector.h>
#include "arte/arte.hh"
#include "arte/ArteMessage.hh"
#include "gean/zgeom.h"
#include "gean/gclink.h"
#include "gean/gcnum.h"

extern "C" {
    void gsmate_(int*,char*,float*,float*,float*,float*,float*,float*,int*,int);
    void gsmixt_(int&,char*,float[],float[],float&,int&,float[],int);
    void crtmed_(const int&);
    void mmbook_(int &, const int &); 
}

int zgmed::maxmat ;
int zgmed::nmat;

void  zgmed::Set_nmat(int n) {nmat = n;}

void  zgmed::Readmed(char *card, int &nmed) {
      char taga[16]; int flag; int nmix;
      int nitem=sscanf(card,"%s %d %d",taga,&flag,&nmix); 
      if (nitem!=3) {
          arteMsg <<"check data on medium cards" << taga  ;
          arteMsg.error();} 
      fillstr(taga,12);             
      nmed = Check_Gmat(taga);
      if (nmed !=0 ) {
         arteMsg << "material "<<taga<<" exists already - see entry"<< nmed;
         arteMsg.warn();
         if(abs(nmix) > abs(Getncomp(zgmed::nmat))) {
             arteMsg << "number of components in new bigger than in old " 
                   << " define another material name ";
             arteMsg.error();
         } 
      }
      else {
         mmbook_(ArtePointer<GMAT>::getLink().getRef(), 1);
         nmed = ArteTable<GMAT>::size();
      }
      Set_nmat(nmed);
      ArtePointer<GMAT> gmat(nmat); 
      gmat->flag = flag;       
      gmat->nmix  = nmix;
      strncpy(gmat->name,taga,12);
      unsigned int nim = abs(nmix);
      if(nim <= 1) gmat->gmix = 0;
      else gmat->gmix = ArteTable<GMIX>::size() + 1 ; 
}

void  zgmed::Readparam(char *card) {
      char taga[16];
      ArtePointer<GMAT> gmat(nmat); 
      int nitem=sscanf(card,"%f %f %f %f  %f ",&gmat->a,&gmat->z,
                  &gmat->dens, &gmat->radl, &gmat->absl);
      if (nitem!=5) {
         arteMsg <<"check data on medium/parameter cards"  ;
         arteMsg.error();
      } 
      card+=itemlength(card,nitem);
      nitem = sscanf(card," %s ",taga);
      if(nitem== -1 || !strncmp(taga,"!",1) || !strncmp(taga,"//",2)) 
                                         gmat->loss = 0;
      else  nitem = sscanf(card," %f ",&gmat->loss);
}


void  zgmed::Readmcomp(char *card){
      char taga[16]; float mix_prop;
      ArtePointer<GMAT> gmat(nmat);
      int nitem=sscanf(card,"%s %f ",taga,&mix_prop); 
      fillstr(taga,12);
      if (nitem!=2) {
         arteMsg <<"check data on medium/mcomp cards"  ;
         arteMsg.error();
      }
      int nm = Check_Gmat(taga);
      if(nm ==0 && abs(gmat->nmix) > 0){
            arteMsg << "mixture component" << taga <<
            "with volume weights must be defined before";
            arteMsg.error();
      }
      ArtePointer<GMAT> gm(nm);
      mmbook_(ArtePointer<GMIX>::getLink().getRef(), 1);
      int nmix = ArteTable<GMIX>::size();
      ArtePointer<GMIX> gmix(nmix);          
      strncpy(gmix->name,taga,12);  
      gmix->a = gm->a;
      gmix->z = gm->z;
      gmix->w = mix_prop;
}

void zgmed::Create_Gmed() {
      char gname[20];
      float a_mix[10], z_mix[10], w_mix[10], d_mix;
      ArtePointer<GMAT> gmat(nmat);
      if(gmat->flag == 0) return;

      memset(gname,' ',20);
      strncpy(gname,gmat->name,12);
      gname[12] ='$';
      float dens = gmat->dens; 
      unsigned inmix = abs(gmat->nmix);

      if(inmix == 1) {
        float ubuf; int i0=0;         
        gsmate_(&nmat,gname,&gmat->a,&gmat->z,&dens,&gmat->radl,
                &gmat->absl,&ubuf,&i0,20);
      }
      else {
        float weight = 0.;  float vol = 0.;  
        ArtePointer<GMIX> gm0 = gmat->gmix;
        int istart_mix = gm0.index();
        for (unsigned int i=0; i<inmix; i++)  {
          ArtePointer<GMIX> gmix(istart_mix+i); 
          a_mix[i] = gmix->a;
          z_mix[i] = gmix->z;
          w_mix[i] = gmix->w;
          if (gmat->nmix > 1)  { 
            int nm = Check_Gmat(gmix->name);
            if(nm ==0 ) {
               arteMsg << "mixture component" << gmix->name <<
                   "with volume weights must be defined before";
               arteMsg.error();
            }    
            ArtePointer<GMAT> gm(nm);
            d_mix = gm->dens;
            vol += w_mix[i]; 
            w_mix[i] = w_mix[i]* d_mix;
            weight += w_mix[i];
          }
        }
        if(gmat->nmix>1) {
           for (unsigned int i=0;i<inmix;i++) w_mix[i]=w_mix[i]/weight;
            if(gmat->dens == 0.) {
               dens = weight/vol; gmat->dens = dens;
            }
        }
        gsmixt_(nmat,gname,a_mix,z_mix,dens,gmat->nmix,w_mix,20);
        Set_mixpar();
      }
      crtmed_(nmat);  
   }


void  zgmed::Set_mixpar() {
      float aeff, zeff, dens, reff, aleff;
      char taga[13];
      ArtePointer<GMAT> gmat(nmat);
      strncpy(taga,gmat->name,12);
      int jma = *(gcbank_.lq + gclink_.jmate -1 - nmat);
      memcpy(&aeff ,gcbank_.iq + jma + 5,4);
      memcpy(&zeff ,gcbank_.iq + jma + 6,4);
      memcpy(&dens ,gcbank_.iq + jma + 7,4);
      memcpy(&reff ,gcbank_.iq + jma + 8,4);
      memcpy(&aleff,gcbank_.iq + jma + 9,4);
      if(fabs(aeff-gmat->a)/(aeff+gmat->a) > .001 ||
         fabs(zeff-gmat->z)/(zeff+gmat->z) > .001 ||
         fabs(reff-gmat->radl)/(reff+gmat->radl) > .001 ||
         fabs(aleff-gmat->absl)/(aleff+gmat->absl) > .001 ||
         fabs(dens-gmat->dens)/(dens+gmat->dens) > .001) {
         arteMsg <<"for "<< taga <<" new mixture parameters were calculated:";
         arteMsg.warn();
         arteMsg <<"old: " << gmat->a << " " << gmat->z << " " << gmat->dens 
                           << " " << gmat->radl <<  " " << gmat->absl;  
         arteMsg.warn();
         arteMsg <<"new: " << aeff << " " << zeff << " " << dens << " " <<
                              reff << " " << aleff;  
         arteMsg.warn();
      }    
      gmat->a = aeff;
      gmat->z = zeff;
      gmat->radl = reff;
      gmat->absl = aleff;
      gmat->dens = dens; 
}

int   zgmed::Getncomp(int &nr){
      if (nr > nmat) nr = nmat;
      ArtePointer<GMAT> gmat(nmat); 
      return gmat->nmix;
}

int   zgmed::Check_Gmat(char *cinp){
      int nm=0;
      for(ArteTableIterator<GMAT> gmat; gmat.valid(); gmat.next()){
         if(!strncmp(gmat->name,cinp,12)){
            nm = gmat.index();
            break;
         }
      }
      return nm;  
}


void  zgmed::Fill_hg_tmed() {
      extern vector<int*>    hg_tmed;
      int nmat = ArteTable<GMAT>::size();
      hg_tmed.reserve(nmat);
      for (int k = 0; k< nmat; k++) {
         int *ptr = new int[4];
         *ptr = 0;   *(ptr+1) = 0;   *(ptr+2) = 0;   *(ptr+3) = 0; 
         hg_tmed.push_back(ptr);
      }

      for(int imed =1;imed <= gcnum_.ntmed;imed++) {
        int* pjmd = gcbank_.lq + gclink_.jtmed -1 - imed;
        int *pji = gcbank_.iq + *pjmd + 5;
        float *pjx = (float*)pji;
        int imat = int(*pjx);
        vector<int*>::iterator tm = hg_tmed.begin() + imat -1;
        pji = gcbank_.iq + *pjmd + 6;
        pjx = (float*)pji;
        int isvol = int(*pjx);
        if(isvol>0) isvol = 2;
        pji = gcbank_.iq + *pjmd + 7;
        pjx = (float*)pji;
        int ifield = int(*pjx);
        if(ifield > 0) ifield = 1;
        (*tm)[isvol+ifield] = imed;
      }
}

