
/******************************************************************
*                                                                *
*       initialisation routine for HBGEAN grafic commands        *
*                                                                *
*       Author:   S.Nowak                                        *
*                                                                *
*****************************************************************/

#include <stdio.h>
#include <string.h>
#include "arte/arte.hh"
#include "arte/ArteMessage.hh"
#include "arte/ArteFlag.h"
#include "gean/gclink.h"
#include "gean/gcnum.h"
#include "gean/Geant2Arte.hh"
#include "gean/zgpict.hh"
#include "gean/ZGESL.h"


extern "C" {
  void utpris_(char*, int &, int &, const int);
  void iselnt_(int &);
  void iclrwk_(int &, int&); 
  void gsatt_(char*, char*, int&);
  void gfatt_(char*, char*, int&);
  void gdopt_(char*, char*);
  void gdraw_(char*, float&,float&,float&, float&,float&, float&,float&);
  void gdrawt_(float&,float&, char*, float&,float&, int&,int&, const int);
  void  gdrvol_(int &,char *,int *,int &,float &,float &,float &,float &,float &,float &,float &);
  void gdhead_(int &, char*, float &, const int);
  void gdscal_(float &, float&);
  char *ku_getc();
  int  ku_geti();
}   

// initialisation routine for HBGEAN grafic commands
void zgpict::picini() {
  //  if not yet done initialize prism
      int i0=0, i1=1, i2=2,  m112 =-112, ival=0;
      char vol_name[4]; 
      if(arteflag_.prisinit == 0) utpris_("    ",m112,i2,4);
      iselnt_(i1);
      iclrwk_(i0,i1);

      for (int i=0; i<gcnum_.nvolum; i++) {
        memcpy(vol_name,gcbank_.iq+gclink_.jvolum+i,4);
        gfatt_(vol_name,"COLO",ival);
        if(ival == 0) gsatt_(vol_name,"SEEN",i0);
        gfatt_(vol_name,"SEEN",ival);
        if(ival != 0) gsatt_(vol_name,"FILL",i1);
     }
}

/*****************************************************************/


// This routines draw views of the whole detector
void zgpict::picdet(char *name){
  int i0=0, i1=1, i2 =2, m2=-2, m7=-7;
  float f0=0.,  f10 =10.,  f75=75.,  f90=90.,  f150 =150.,  f180=180.;  
  char pers[2];
  strncpy(pers,ku_getc(),2);
  gsatt_("MOTH","LWID",m7);
  if(!strncmp(name,"MOTH",4) || !strncmp(name,"RPOT",4)) {
     gsatt_("VCYL","LWID",i1);
     gsatt_("VCYL","FILL",i0);
     gsatt_("VCON","LWID",i1);
     gsatt_("VCON","FILL",i0);
  }
  float xsc =0, ysc =0, zsc=0; 
  for(ArteTableIterator<NESL> ns ; ns.valid(); ns.next()) {
     if(ns->cmp == LGESL_SVD  ||  ns->cmp == LGESL_OTR  ||  
        ns->cmp == LGESL_ITR  ||  ns->cmp == LGESL_PTC  || 
        ns->cmp == LGESL_MUO  ||  ns->cmp == LGESL_MUX  ||  
        ns->cmp == LGESL_TRD  ||  ns->cmp == LGESL_ECA) {
	gsatt_(ns->name,"SEEN",m2);
        gsatt_(ns->name,"FILL",i1);
     }  
     if(!strncmp(name,ns->name,4)) {
        xsc = 18./(ns->x2-ns->x1 + 2.*ns->dx); 
        ysc = 18./(ns->y2-ns->y1 + 2.*ns->dy);
        zsc = 18./(ns->z2-ns->z1); 
     }
  }

  gdopt_("HIDE","ON  ");
  gdopt_("SHAD","ON  ");

  if(!strncmp(pers,"P",1)) {
     gdraw_(name,f75,f150,f0,f10,f10,zsc,zsc);
     float txth =.3; 
     if(!strncmp(name,"MOTH",4)) { 
        float xp = 1.8, yp =12.4;
        gdrawt_(xp,yp,"RPOT WITH SVD",txth,f0,i2,i0,13);
        xp = 4.5, yp = 7.9;    
        gdrawt_(xp,yp,"MT01-05",txth,f0,i2,i0,7);
        xp = 5.0, yp = 14.2;
        gdrawt_(xp,yp,"MAGNET",txth,f0,i2,i0,6);
        xp = 5.6, yp = 13.4; 
        gdrawt_(xp,yp,"PT01-03",txth,f0,i2,i0,7);
        xp = 6.2, yp = 7.2;
        gdrawt_(xp,yp,"MC06-08",txth,f0,i2,i0,7);
        xp = 6.8, yp =6.6;
        gdrawt_(xp,yp,"PC01-04",txth,f0,i2,i0,7);
        xp = 10.0, yp =15.0;   
        gdrawt_(xp,yp,"RICH",txth,f0,i2,i0,4);
        xp =10.4, yp =6.1;
        gdrawt_(xp,yp,"TC01-02",txth,f0,i2,i0,7);
        xp = 12.8, yp=13.;
        gdrawt_(xp,yp,"ECAL",txth,f0,i2,i0,4);
        xp = 13.5, yp = 4.5;
        gdrawt_(xp,yp,"MUON SYSTEM",txth,f0,i2,i0,11);
     }
  }
  else if(!strncmp(pers,"CX",2)) 
                  gdraw_(name,f90,f180,f0,f10,f10,zsc,ysc);
  else if(!strncmp(pers,"CY",2)) 
                  gdraw_(name,f90,f90 ,f0,f10,f10,zsc,xsc);
 int isel = 1001;
 float txth = 0.6;
 gdhead_(isel,"HERA-B",txth,6);
 float xp =10., yp=2.;  
 gdscal_(xp,yp);
 }



/*****************************************************************/
// this routines draws a view of one si superlayer
void zgpict::picsil() {

   int lcopy[4];
   char pers[3], name[5], lvols[16];
   strncpy(pers,ku_getc(),2);
   strncpy(name,ku_getc(),4);
   name[4] = '\0';

   int isl = 0;

   ArteTableIterator<NESL> ns;
   for(; ns.valid(); ns.next()){
      if(!strncmp(ns->name,name,4)) {
         isl = ns.index();     break;
      }
   }
   if(!isl) {
      arteMsg << "superlayer " << name  << " not found";
      arteMsg.warn();
      return;
   }   

   gdopt_("HIDE","ON  ");
   gdopt_("SHAD","ON  "); 

   float xsc = 18./(ns->x2-ns->x1);
   float ysc = 18./(ns->y2-ns->y1);
   float zsc = 18./(ns->z2-ns->z1);
   int in = 3;
   //   if(ns->cmp == LGESL_ITR) in = 3;
   int lay = ku_geti();
   int sec = ku_geti();
   int i0 =0;
   float f0 = 0., f90=90., f10=10., f180=180.;

   for(int ipl = ns->fpl;ipl != ns->fpl+ns->npl; ipl++) {
      ArtePointer<NEDE> nd(ipl); 
      if(lay != 0 && nd->lay != lay) continue;
      if(sec != 0 && nd->sec != sec) continue;
      Geant2Arte G2A;
      G2A.GetModule(ipl,lvols,lcopy); 
      //      gsatt_(&lvols[4],"SEEN",i0);

      if(!strncmp(pers,"CZ",2))
                 gdrvol_(in,lvols,lcopy,i0,f0,f90,f0,f10,f10,xsc,ysc);      
      else if (!strncmp(pers,"CX",2))
                 gdrvol_(in,lvols,lcopy,i0,f90,f180,f0,f10,f10,zsc,ysc);
      if(lay !=0 && sec !=0) break;
   }        
    
/*

//    carrier
      name(2) = mtda(2)
      nume(2) = ipl;
c      call gdrvol(2,name,nume,0,150.,150.,0.,15.,14.,.12,.12)
      call gdrvol(2,name,nume,0,0.,90.,0.,6.,13.,.25,.25)

c     silicon
      name(2) = mtda(1)
      nume(2) = ifpl + k
c      call gdrvol(2,name,nume,0,150.,150.,0.,10.,10.5,.12,.12)
c      call gdrvol(2,name,nume,0,180.,180.,0.,12., 3.,.08,.08)
       gdrvol(2,name,nume,0,0.,90.,0.,6.,13.,.25,.25) 
c      nume(2) = nume(2) + 2
c      call gdrvol(2,name,nume,0,180.,180.,0.,17., 6.,.08,.08)

      name(2) = mtda(3)
      nume(2) = ifpl +k
c      call gdrvol(2,name,nume,0,150.,150.,0.,10.,10.5,.12,.12)
c      call gdrvol(2,name,nume,0,180.,180.,0.,12., 3.,.08,.08)
      call gdrvol(2,name,nume,0,0.,90.,0.,6.,13.,.25,.25)
c      nume(2) = nume(2) + 2
c      call gdrvol(2,name,nume,0,180.,180.,0.,17., 6.,.08,.08)


      if(k.eq.13) call gdscal(16.,2.) */
      

}
/*****************************************************************/
// this routines draws a view of one itr superlayer
void zgpict::picitr() {

   int lcopy[4];
   char pers[3], name[5], lvols[16];
   strncpy(pers,ku_getc(),2);
   strncpy(name,ku_getc(),4);
   name[4] = '\0';

   int isl = 0;

   ArteTableIterator<NESL> ns;
   for(; ns.valid(); ns.next()){
      if(!strncmp(ns->name,name,4)) {
         isl = ns.index();     break;
      }
   }
   if(!isl) {
      arteMsg << "superlayer " << name  << " not found";
      arteMsg.warn();
      return;
   }   

   gdopt_("HIDE","OFF ");
   gdopt_("SHAD","ON  "); 

   float xsc = 18./(ns->x2-ns->x1);
   float ysc = 18./(ns->y2-ns->y1);
   float zsc = 18./(ns->z2-ns->z1);
   int in = 3;

   int lay = ku_geti();
   int sec = ku_geti();
   int i0 =0, i1=1 ;
   float f0 = 0., f90=90., f10=10., f180=180.;

   for(int ipl = ns->fpl;ipl != ns->fpl+ns->npl; ipl=ipl+2) {
      ArtePointer<NEDE> nd(ipl); 
      if(lay != 0 && nd->lay != lay) continue;
      if(sec != 0 && nd->sec != sec) continue;
      Geant2Arte G2A;
      G2A.GetModule(ipl,lvols,lcopy); 
      gsatt_(&lvols[4],"SEEN",i0);

      int jvos = *(gcbank_.lq + gclink_.jvolum -1 -
                                     G2A.GetGvolNumber(&lvols[4]));
      int *pji = gcbank_.iq + jvos + 2;
      float *pjx = (float*)pji; 
      int ndau = int(*pjx);
      for (int i = 0; i<ndau; i++){
         int jin = *(gcbank_.lq + jvos -2 -i);
         pji = gcbank_.iq + jin + 2;
         float *pjx = (float*)pji;
         lcopy[2] = int(*pjx);
         pji = gcbank_.iq + jin + 1;
         pjx = (float*)pji;  
         memcpy(&lvols[8], gcbank_.iq+gclink_.jvolum - 1 + int(*pjx),4);
         printf("a %.12s %d %d %d %d \n",lvols,lcopy[0],lcopy[1],lcopy[2],i);
         gsatt_(&lvols[8],"FILL",i1);
         if(!strncmp(pers,"CZ",2))
             gdrvol_(in,lvols,lcopy,i0,f0,f90,f0,f10,f10,xsc,ysc);      
         else if (!strncmp(pers,"CX",2))
                 gdrvol_(in,lvols,lcopy,i0,f90,f180,f0,f10,f10,zsc,ysc);
         else if (!strncmp(pers,"CZ",2)) 
                 gdrvol_(in,lvols,lcopy,i0,f0,f0,f0,f10,f10,xsc,ysc);
      }
      if(lay !=0 && sec !=0) break;
   }        
    
/*

//    carrier
      name(2) = mtda(2)
      nume(2) = ipl;
c      call gdrvol(2,name,nume,0,150.,150.,0.,15.,14.,.12,.12)
      call gdrvol(2,name,nume,0,0.,90.,0.,6.,13.,.25,.25)

c     silicon
      name(2) = mtda(1)
      nume(2) = ifpl + k
c      call gdrvol(2,name,nume,0,150.,150.,0.,10.,10.5,.12,.12)
c      call gdrvol(2,name,nume,0,180.,180.,0.,12., 3.,.08,.08)
       gdrvol(2,name,nume,0,0.,90.,0.,6.,13.,.25,.25) 
c      nume(2) = nume(2) + 2
c      call gdrvol(2,name,nume,0,180.,180.,0.,17., 6.,.08,.08)

      name(2) = mtda(3)
      nume(2) = ifpl +k
c      call gdrvol(2,name,nume,0,150.,150.,0.,10.,10.5,.12,.12)
c      call gdrvol(2,name,nume,0,180.,180.,0.,12., 3.,.08,.08)
      call gdrvol(2,name,nume,0,0.,90.,0.,6.,13.,.25,.25)
c      nume(2) = nume(2) + 2
c      call gdrvol(2,name,nume,0,180.,180.,0.,17., 6.,.08,.08)


      if(k.eq.13) call gdscal(16.,2.) */
      

}


