// changed 05/07/99; 10/09/99 Torsten Zeuner
// last changes 10/03/00 TZ

#include  <stdio.h>

// ARTE
#include "arte/arte.hh"
#include "farm/farm.hh"

// libitr
#include "aframelib.h"
#include "ITR_silraw.h"
#include "ITR_arte.h"
#include "ITR_marvin.h"
#include "itrleda.h"
#include "ITR_EVinfo.h"

// return codes
#include  "grr.h"

// sparse event definition
#include "event.h"
#include "etb.h"

// some arguments for aframe
extern       int    nArg;
extern const char **vArg;

//-----------------------------------------------------------------

int ItrInit()
{
  extern int ITR_STEERDB_MAJ;
  extern int ITR_STEERDB_MIN;
  extern int ITR_CLUSTER_ALG;
  extern int ITR_DYNTHRESH;
  extern int ITR_NEVT_MON;
  extern int ITR_OFFS_1;
  extern int ITR_OFFS_2;
  extern int ITR_OFFS_3;
  extern int ITR_OFFS_4;
  extern int ITR_OFFS_5;
  extern int ITR_OFFS_6;
  extern int ITR_OFFS_7;
  extern int ITR_OFFS_8;
  extern int ITR_OFFS_9;
  extern int ITR_OFFS_10;
  extern float ITR_THRESHOLD_A;
  extern float ITR_THRESHOLD_B;
  extern float ITR_HOTCUT;
  extern float ITR_MAXSIGNAL;
  
  char *routine = "ItrInit";
  int rc;
  int dbmajor, dbminor;
  static int ITR_first = 1;

  // initialize Steering DB

  if (ITR_STEERDB_MAJ == -999 || ITR_STEERDB_MIN == -999) 
    if( ArteTable<GEHD>::size() ){
      ArtePointer<GEHD> gehd( 1 );  
      if (int(gehd->gevs[2]) >  0) {ITR_STEERDB_MAJ=2; ITR_STEERDB_MIN=2;}
      if (int(gehd->gevs[2]) == 0) {ITR_STEERDB_MAJ=2000; ITR_STEERDB_MIN=1;}
    }

  // Initilialize some control words

  aframe_store_program_name(vArg[0]);
  aframe_mess(routine,GRR_SINFO,"ITR_arte started");  

 // return when called more then once

  if ( !ITR_first ) {
    aframe_mess(routine,GRR_SINFO,"ItrInit already called - skipping...");
    return GRR_OK;
  }
  ITR_first = 0;
  
  // 
  // no ITR Hitpreparation in case of MC
  //
  // if ( ITR_CheckMC() ) return GRR_OK;
  
  //
  // REAL DATA
  //
  // call ITR_fillgedeindex()

  rc = ITR_fillgedeindex();
 
  // get event pointer

  // EVENT_HDR *event = ARTE_get_event();

  // if ( !event ) {
  //   aframe_mess(routine,GRR_SERROR,"Could not find event!");
  //   return GRR_FISHY;
  // }   

  // initialize parameters of other packages

  aframe_param_init();
  ITR_leda_param_init();
  ITR_silraw_param_init();
  ITR_arte_param_init();
  ITR_marvin_param_init();

  // preferences are loaded here

  /* get command line arguments indicating database table */
  
  dbmajor = ITR_STEERDB_MAJ;
  dbminor = ITR_STEERDB_MIN;   
    
  iol_scan_preferences_from_db ("/ITR_Setup:ITR/Steering", dbmajor,
                                dbminor);

  if ( aframe_process_args(nArg, vArg, AFRAME_CLI) != AFRAME_OK ) {
    aframe_mess(routine,GRR_SWARNING,"Error processing arguments.");
  }
   

  // Overwriting IOL parameters by kumac

  int module_matches, verb_matches;
  char values[300];

  if (ITR_OFFS_1 != 0) {
    sprintf(values,"%d",ITR_OFFS_1);
    rc = iol_load_value_from_mod_verb(values,"ITR_silraw",
                                      "datavalid offset sl1",&module_matches,&verb_matches);
  }
  if (ITR_OFFS_2 != 0) {  
    sprintf(values,"%d",ITR_OFFS_2);
    rc = iol_load_value_from_mod_verb(values,"ITR_silraw",
                                      "datavalid offset sl2",&module_matches,&verb_matches); 
  }
  if (ITR_OFFS_3 != 0) {  
    sprintf(values,"%d",ITR_OFFS_3);
    rc = iol_load_value_from_mod_verb(values,"ITR_silraw",
                                      "datavalid offset sl3",&module_matches,&verb_matches); 
  }
  if (ITR_OFFS_4 != 0) {  
    sprintf(values,"%d",ITR_OFFS_4);
    rc = iol_load_value_from_mod_verb(values,"ITR_silraw",
                                      "datavalid offset sl4",&module_matches,&verb_matches); 
  }
  if (ITR_OFFS_5 != 0) {  
    sprintf(values,"%d",ITR_OFFS_5);
    rc = iol_load_value_from_mod_verb(values,"ITR_silraw",
                                      "datavalid offset sl5",&module_matches,&verb_matches); 
  }
  if (ITR_OFFS_6 != 0) {  
    sprintf(values,"%d",ITR_OFFS_6);
    rc = iol_load_value_from_mod_verb(values,"ITR_silraw",
                                      "datavalid offset sl6",&module_matches,&verb_matches); 
  }
  if (ITR_OFFS_7 != 0) {  
    sprintf(values,"%d",ITR_OFFS_7);
    rc = iol_load_value_from_mod_verb(values,"ITR_silraw",
                                      "datavalid offset sl7",&module_matches,&verb_matches); 
  }
  if (ITR_OFFS_8 != 0) {  
    sprintf(values,"%d",ITR_OFFS_8);
    rc = iol_load_value_from_mod_verb(values,"ITR_silraw",
                                      "datavalid offset sl8",&module_matches,&verb_matches); 
  }
  if (ITR_OFFS_9 != 0) {  
    sprintf(values,"%d",ITR_OFFS_9);
    rc = iol_load_value_from_mod_verb(values,"ITR_silraw",
                                      "datavalid offset sl9",&module_matches,&verb_matches); 
  }
  if (ITR_OFFS_10 != 0) {  
    sprintf(values,"%d",ITR_OFFS_10);
    rc = iol_load_value_from_mod_verb(values,"ITR_silraw",
                                      "datavalid offset sl10",&module_matches,&verb_matches); 
  } 

  if (ITR_CLUSTER_ALG==1 || ITR_CLUSTER_ALG==2) {
    sprintf(values,"%d",ITR_CLUSTER_ALG);
    rc = iol_load_value_from_mod_verb(values,"ITR_marvin",
                                      "clusterfinder",&module_matches,&verb_matches); 
    if (ITR_CLUSTER_ALG==1) {
       sprintf(values,"%f",ITR_THRESHOLD_A);
       rc=iol_load_value_from_mod_verb(values,"ITR_marvin",
                                        "minDiff",&module_matches,&verb_matches);
       printf("%f\n",ITR_THRESHOLD_B);
       sprintf(values,"%f",ITR_THRESHOLD_B);
       rc=iol_load_value_from_mod_verb(values,"ITR_marvin",
                                        "pulseHeight",&module_matches,&verb_matches);
    }
    if (ITR_CLUSTER_ALG==2) {
       sprintf(values,"%f",ITR_THRESHOLD_A);
       rc=iol_load_value_from_mod_verb(values,"ITR_marvin",
                                       "seedcut",&module_matches,&verb_matches);
       sprintf(values,"%f",ITR_THRESHOLD_B);
       rc=iol_load_value_from_mod_verb(values,"ITR_marvin",
                                       "chi2cut",&module_matches,&verb_matches);
       cout << ">>>>>>  S/Nstr = " << ITR_THRESHOLD_A << "   :   S/Nclu = " << ITR_THRESHOLD_B << endl;
    }
  }
  if (ITR_DYNTHRESH !=0) {
    sprintf(values,"%d",ITR_DYNTHRESH);
    rc=iol_load_value_from_mod_verb(values,"ITR_marvin",
                                    "dynamic threshold",&module_matches,&verb_matches); 
  }
  if (ITR_NEVT_MON > 1) {
    sprintf(values,"%d",ITR_NEVT_MON);
    rc=iol_load_value_from_mod_verb(values,"ITR_marvin",
                                    "nevmon",&module_matches,&verb_matches); 
  }

  if (ITR_HOTCUT !=0) {
    sprintf(values,"%f",ITR_HOTCUT);
    rc=iol_load_value_from_mod_verb(values,"ITR_marvin",
                                    "hotcut",&module_matches,&verb_matches); 
  }    

  if (ITR_MAXSIGNAL !=0) {
    sprintf(values,"%f",ITR_MAXSIGNAL);
    rc=iol_load_value_from_mod_verb(values,"ITR_marvin",
                                    "svar maxSignal",&module_matches,&verb_matches); 
  }
  // int ndaq;

  int major = 0;
  int minor = 0;

  // get daq block stored in EVT_DATA_RECORD daq, ndaq number of found daq blocks

  // EVT_DATA_RECORD daq;
  // ETB_get_daqs(event, 1, &daq, &ndaq);

  // if ( ndaq > 0 ) {
  //   major = daq.data.event_time;
  //   minor = 0;
  // }

  rc = ITR_leda_startprogram( major, minor );
  if ( rc != GRR_OK ) {
    cout << "*** ITR Initialisation failed : No ITR Hitpreparation ! ***" << endl;
    return rc;
  }

  // start ITR_silraw

  rc = ITR_silraw_startprogram();
  if ( rc != GRR_OK ) return rc;

  // start ITR_marvin

  rc = ITR_marvin_init();
  if ( rc != GRR_OK ) return rc;

  rc = ITR_fillgedeindex();
  if ( rc != GRR_OK ) return rc;
 
  rc = ITR_arte_alignment();
  if ( rc != GRR_OK ) return rc;

  return GRR_OK;
}

