
// *****************************************************************************
//
// source:    l4tInit.C
//
// type:      source code
//
// created:   14 Jan 1998
//
// author:    Dr. Andreas Gellrich
//            HERA-B Collaboration
//            Humboldt-University Berlin
//            Physics Department
//            Invalidenstr. 110
//            10115 Berlin
//            Germany
//            email: Andreas.Gellrich@desy.de
//
// symbol:    l4tInit
//
// called by: 
//
// changes:
// 24 Nov 1999 (AG) move to l4tInit()
// 25 Jan 2000 (AG) check trigger settings
// 21 Feb 2000 (AG) add kumac steering
// 22 Feb 2000 (AG) improve kumac steering
// 27 Apr 2000 (AG) use ARTE_mark()
// 23 Jun 2000 (AG) add printout of classification bits
//
// *****************************************************************************

// standard includes
#include <iostream.h>
#include <stdlib.h>

// ONLINE includes
#include "runcfg.h"

// ARTE includes
#include "arte/ArteProto.hh"
#include "farm/farm.hh"
#include "l4trig/l4t.hh"
#include "reco/eclass.hh"

// globals
extern int FATRG;

// prototypes
extern "C" void l4kuip_();

//
// initialize the Xyz kuip commands
//
static class L4Initer
{
public:
  L4Initer() { atregkuip( l4kuip_, 1 ); } 
} _l4Initer;

//
// program code
//
int
l4tInit()
{
  // show up
  ARTE_mark( "l4tInit", 0 );
  cout << "l4tInit()" << endl;
  
  // configuration
  char *configuration = getenv( PRCMRUNCFG );
  unsigned int confMask = 0;
  int l4Trigger = 1;
  //cout << "l4tInit(): configuration=" << configuration << endl;
  if( configuration ){
    confMask = atoi( configuration );

    // settings
    int trigger = RCFG_GET_TRIGGERS( confMask );
    cout << "TRIGGER: "
	 << "Settings (RUNcfg):" << trigger << " "
	 << "FLT=" << bool( trigger & RCFG_TL_FIRST )  << " "
	 << "SLT=" << bool( trigger & RCFG_TL_SECOND ) << " "
	 << "TLT=" << bool( trigger & RCFG_TL_THIRD )  << " "
	 << "4LT=" << bool( trigger & RCFG_TL_FOURTH ) << endl;

    // l4 trigger
    l4Trigger = bool( trigger & RCFG_TL_FOURTH );
  }
  //cout << "l4tInit(): l4Trigger=" << l4Trigger << endl;
  
  // update flag
  if( !l4Trigger ) FATRG = L4_OFF; 
  cout << "TRIGGER: FATRG=" << FATRG << endl;

  // printout
  for( int bit=1; bit<=16; bit++ ){
    cout << "CLASS: event classification bits" << endl;
    cout << "       " << bit   << " " << rcClassName( bit ) << endl;
  }
  
  return 0;
}
