
//**********************************************************************/
// Created: 21.04.98 by   Olya Igonkina 
//
// implementation of SegClusBuilder class
//
//**********************************************************************/

#include <math.h>
#include <arte/arte.hh>
#include "arte/RSEG_cmp.hh"

#include "marple/marple.hh"
#include "marple/RcMcFunction.hh"
#include "marple/RcSegClusBuilder.hh"

#include "marple/RcHbook.hh"
#include "marple/RcTuples.h"

RcSegClusBuilder::RcSegClusBuilder() :
  RcMatchBuilder<RcSegPoint,RcClusPoint>(), fptRcConstBrem(0),
  fNmtra(0),  fNelectron(0),  fNjpsiElectron(0),
  fNmtraBrem(0),  fNelectronBrem(0),  fNjpsiElectronBrem(0),
  frNmtra(0),  frNelectron(0),  frNjpsiElectron(0),
  frNmtraBrem(0),  frNelectronBrem(0),  frNjpsiElectronBrem(0), 
  fNbrem(0), fNbadBrem(0)
{}

RcSegClusBuilder::RcSegClusBuilder( std::vector<RcSegPoint  *> vecLeft,
				std::vector<RcClusPoint *> vecRight,
				RcMatchConst & rcConst,
				RcMatchConst & rcConstBrem)
  : RcMatchBuilder<RcSegPoint, RcClusPoint>(vecLeft, vecRight, rcConst),
    fptRcConstBrem(&rcConstBrem),
    fNmtra(0),  fNelectron(0), fNjpsiElectron(0), 
    fNmtraBrem(0),  fNelectronBrem(0),  fNjpsiElectronBrem(0),
    frNmtra(0),  frNelectron(0),  frNjpsiElectron(0),
    frNmtraBrem(0),  frNelectronBrem(0),  frNjpsiElectronBrem(0),
    fNbrem(0), fNbadBrem(0)
{}

RcSegClusBuilder::~RcSegClusBuilder()
{
  fptRcConstBrem     = 0;
  //RcMatchBuilder<RcSegPoint,RcClusPoint>::~RcMatchPoint(*this);
}

//------------------------------------------------------
void RcSegClusBuilder::doMatch()
{
  RcMatchBuilder<RcSegPoint, RcClusPoint>::doMatch();
  statMC();
}
//------------------------------------------------------
void RcSegClusBuilder::doMatch(  MatchFunction matchFunction,
				 int markCombination )
{

  // is there anything to match ?
  if( vectorLeft().size() == 0  || vectorRight().size() == 0 )
    return;

  int baseMark = 0;
  MatchFunction bremFunc = &RcSegClusBuilder::matchBrem;
  RcMatchConst * ptConst = 0;
  if( matchFunction != bremFunc ) 
    ptConst = ptToMatchConst();
  else
    ptConst = ptToBremConst();

  if( markCombination ) 
    {
      for( unsigned int iLeft=0; iLeft < vectorLeft().size(); iLeft++)
	vectorLeft()[ iLeft ]->setmark( ptConst->n_pair_left);
      for( unsigned int iRight=0; iRight < vectorRight().size(); iRight++)
	vectorRight()[ iRight ]->setmark( ptConst->n_pair_right);
    }
  for( unsigned int iLeft=0; iLeft < vectorLeft().size(); iLeft++)
    {
      RcSegPoint * segment = vectorLeft()[ iLeft ];
      for( unsigned int iRight=0; iRight < vectorRight().size(); iRight++)
        {
	  RcClusPoint * cluster = vectorRight()[iRight];
	  
	  RcMatchPoint * matchPoint = 0;
	  // matchFunction will create 'new' object!! if not 0
	  //segment, cluster, matchPoint));
	  if(  ((*this).*matchFunction)(segment, cluster, matchPoint) )
	    {
	      // suppose all match points of this type
	      // should have the same mark
	      baseMark = matchPoint->mark(); 
	      fmatchList.push_back(matchPoint);
	      matchPoint = 0;
	    }
	}
    }	
  
  //**********************************************************************/
  // check the number of combinations with given RcPoints
  //**********************************************************************/
  
  // sort vector of RcMatchPoints by chi2() 
  if( ptConst->chi2 > 0 &&  
      ptConst->chi2 < RcMatchConst::maxConsideredChi2 )
    {
      sort( fmatchList.begin(), fmatchList.end(), comp_chi2 );
      
      if( ptConst->use_pair_cut ) 
	{
	  for( unsigned int iMatch=0; iMatch < fmatchList.size(); iMatch++ )
	    {
	      if( matchList()[iMatch]->mark() != baseMark ) continue;
	      int checkOk = 1;
	      RcSegPoint  * ptLeft   =  leftOrigin(iMatch);
	      RcClusPoint * ptRight  =  rightOrigin(iMatch);
	      
	      if( !(ptConst->pair_cut_ok( *ptLeft  )) ) checkOk = 0 ;
	      if( !(ptConst->pair_cut_ok( *ptRight )) ) checkOk = 0 ;
	      if( !checkOk )
		{ 
		  delete( fmatchList[iMatch]);
		  fmatchList.erase( &fmatchList[iMatch]) ; 
		  iMatch--;
		}else{
		  // mark origin
		  if( markCombination )
		    {
		      int mark = ptLeft->mark();
		      mark--;
		      ptLeft->setmark(mark );
		      mark = ptRight->mark();
		      mark--;
		      ptRight->setmark(mark );
		    }
		}
	    }
	}
    }
  statMC();
}
//
void RcSegClusBuilder::matchBrem( int markCombination )
{
  doMatch(&RcSegClusBuilder::matchBrem, markCombination);
}

int RcSegClusBuilder::matchBrem( RcSegPoint  * segment, 
				 RcClusPoint * cluster,
				 RcMatchPoint* &result)
{
  result = 0;
  if ( ptToBremConst() == 0 ) return 0;

  RcSegPoint tmpPoint = *segment;
  // check for bremstralung, take if it is there
  if( ptToBremConst()->use_delta_cut ) 
    {
      tmpPoint.lineswim(cluster->z() );
      // doesn't pass delta cut, so return 0;
      if( ptToBremConst()->use_delta_cut == 2 &&
	  !ptToBremConst()->energy_delta_cut_ok( tmpPoint, *cluster) )
	return 0; 
      else if( !ptToBremConst()->delta_cut_ok( tmpPoint, *cluster) )
	return 0;
      //takeCombination = 0 ;       
    }

  int lineswim = 1; 

  result = new  RcMatchPoint( *segment,
			      *cluster,
			      ptToBremConst()->flag,
			      lineswim);
  
  if( result->mark() ||                // check mark of matching
      ( ptToBremConst()->chi2 < RcMatchConst::maxConsideredChi2 &&
	result->chi2() > ptToBremConst()->chi2 ) ) // bad chi2
    {
      delete result;
      result = 0;
      return 0;
    }
  result->setmark(3); // mark as bremstrahlung
  return 1;
}
//-------------------------------------------------------------
void RcSegClusBuilder::matchWithMomentum( int markCombination )
{
  doMatch(&RcSegClusBuilder::matchWithMomentum, markCombination);
}

int RcSegClusBuilder::matchWithMomentum( RcSegPoint  * segment, 
					 RcClusPoint * cluster,
					 RcMatchPoint* &result)
{
  result = 0;
  if ( ptToMatchConst() == 0 ) return 0;

  // first check that simple cuts are fine

  if( ptToMatchConst()->use_delta_cut ) 
    {
      float segY = segment->y()+segment->ty()*(cluster->z()-segment->z());
      if( fabs(segY-cluster->y() ) >
	  ptToMatchConst()->delta_y*1.1 ) return 0; // y direction doesn't fit
    }

  HepDouble peOld = segment->pe();
  HepDouble covOld55 = segment->cov()(5,5);
  HepDouble covOld15 = segment->cov()(1,5);
  HepDouble covOld25 = segment->cov()(2,5);
  
  int iErr = 0;
  // from ECAL note 
  float peEstimation = (cluster->x()- segment->x()
			- segment->tx()*(cluster->z()-segment->z() ) ) / 560;

  // !!!!!!!!!!!!!!!! let be carefull in this question! check it! 
  if( fabs(peEstimation) > 2 ) return 0;// less then Pt added by magnetic field

  static int epcut = 0;
  if( epcut < 1 )
    {
      if( MARPLE_SOFT )
	epcut = 3;
      else
	epcut = 2;
    }
  if( fabs(peEstimation) > epcut*cluster->pe() ) // 3*(1/E) < (1/P) => E/P >3
	return 0;

  segment->setpe(peEstimation);
  segment->estimatePEfromTAR(cluster->x(), cluster->y(),
			     cluster->z(), 
			     cluster->cov()(1,1),
			     cluster->cov()(2,2),
			     cluster->cov()(1,2),
			     3, iErr );
  if( iErr ) return 0; 

  // now we should make more presice estimations 

  //if( fabs(segment->pe()) < 0.0033 ) return 0; // !!!!!!!!!!!!!!!
  
  if( ptToMatchConst()->use_delta_cut ) 
    {
      RcSegPoint tmpPoint = *segment;
      tmpPoint.prolongate( cluster->z(), iErr );
      if( iErr ) return 0;
      tmpPoint.setpe( fabs(tmpPoint.pe()) );
      if( ptToMatchConst()->use_delta_cut == 2 &&
	  !ptToMatchConst()->energy_delta_cut_ok( tmpPoint, *cluster) )
	return 0; 
      else if( !ptToMatchConst()->delta_cut_ok( tmpPoint, *cluster) )
	return 0;
    }

  // save combination passed cuts
  if( segment->pe() < 0 ) cluster->setpe(-cluster->pe());
  result = new RcMatchPoint( *segment, *cluster,ptToMatchConst()->flag);
  result->setpe( segment->pe() );
  result->setcov(5,5, segment->cov()(5,5) );
  result->setcov(1,5, segment->cov()(1,5) );
  result->setcov(2,5, segment->cov()(2,5) );
  if( segment->pe() < 0 ) cluster->setpe(-cluster->pe());

  segment->setpe(peOld);
  segment->setcov(5,5,covOld55);
  segment->setcov(1,5,covOld15);
  segment->setcov(2,5,covOld25);

  if(  result->mark() || 
       ( ptToMatchConst()->chi2 < RcMatchConst::maxConsideredChi2 && 
	 result->chi2() > ptToMatchConst()->chi2 ) )
    { 
      delete result;
      result = 0;
      return 0;
    }
  result->setmark(1);
  return 1;
}

//-------------------------------------------------------
void RcSegClusBuilder::matchWithEcluster(int markCombination )
{
  doMatch(&RcSegClusBuilder::matchWithEcluster, markCombination);
}

int RcSegClusBuilder::matchWithEcluster( RcSegPoint  * segment, 
					 RcClusPoint * cluster,
					 RcMatchPoint* &result)
{
  result = 0;
  if ( ptToMatchConst() == 0 ) return 0;

  int iErr = 0;
  HepDouble peOld = segment->pe();
  HepDouble covOld = segment->cov()(5,5);
  
  RcSegPoint tmpPoint = *segment;
  tmpPoint.lineswim(cluster->z() );
	  /*
	  if( (*ptConst).use_delta_cut && 
	      fabs(tmpPoint.y() - cluster->y() ) >
	      (*ptConst).delta_y )
	    {
	      if(RcMtraOk(segment->mtraList(), 
			  RcElepton(cluster->mtraList()) ))
		cout << " lost pair by lineswim: tmpPoint = " 
		     <<tmpPoint.y() << " cluster " << cluster->y()
		     << " delta_y " << (*ptConst).delta_y << endl;
	      continue;
	    }
	    */
  if( tmpPoint.x() < cluster->x() )
    {
      segment->setpe( cluster->pe() );
      segment->setcov(5,5, cluster->cov()(5,5) );
    }else{
      segment->setpe( -cluster->pe() );
      segment->setcov(5,5, cluster->cov()(5,5) );
    }
  tmpPoint = *segment;

  if( ptToMatchConst()->use_delta_cut ) 
    {
      tmpPoint.prolongate( cluster->z(), iErr );
      if( iErr ) return 0;
      if( ptToMatchConst()->use_delta_cut == 2 &&
	  !ptToMatchConst()->energy_delta_cut_ok( tmpPoint, *cluster) )
	return 0; 
      else if( !ptToMatchConst()->delta_cut_ok( tmpPoint, *cluster) )
	return 0;
    }
  if( segment->pe() < 0 ) cluster->setpe(-cluster->pe());
  result =  new RcMatchPoint( *segment, *cluster,ptToMatchConst()->flag);
  result->setpe( cluster->pe() );
  if( segment->pe() < 0 ) cluster->setpe(-cluster->pe());
  segment->setpe(peOld);
  segment->setcov(5,5,covOld);

  if(  result->mark() || 
       ( ptToMatchConst()->chi2 < RcMatchConst::maxConsideredChi2 && 
	 result->chi2() > ptToMatchConst()->chi2 ) )
    { 
      delete result;
      result = 0;
      return 0;
    }
  result->setmark(2);
  return 1;
}	

//------------------------------------------------------
void RcSegClusBuilder::matchRings( int markCombination )
{
  if( maMagnetON() )
    {
      int iErr = 0;
      if( ptToMatchConst()->flag.ifmarked_TX() ){
	for( unsigned int iRight=0; iRight < vectorRight().size(); iRight++)
	  vectorRight()[iRight]->estimateAngles(1,iErr);
      }
      doMatch(&RcSegClusBuilder::matchRingsMagnet, markCombination);
    }
  else
    doMatch(&RcSegClusBuilder::matchRingsNoMagnet, markCombination);
}

int RcSegClusBuilder::matchRingsMagnet( RcSegPoint  * segment, 
					RcClusPoint * cluster,
					RcMatchPoint* &result)
{
  result = 0;
  if ( ptToMatchConst() == 0 ) return 0;

  int iErr = 0;
  int lineswim = 1;

  HepDouble zold = segment->z();
  segment->setZ( cluster->z() );


  //========================================================
  // algorithm with match based on both slopes
  //========================================================  
  if( ptToMatchConst()->flag.ifmarked_TX() )
    {
      HepDouble txOld = cluster->tx();
      HepDouble covtxOld = cluster->cov()(3,3);
      HepDouble pener = fabs(cluster->pe());

      // estimate TX for +charge hypothesis
      cluster->estimateAngles(2,iErr);
      int flag1 = 1;
      HepDouble tx1 = cluster->tx();
      HepDouble covtx1 = cluster->cov()(3,3);
      
      // estimate TX for +charge hypothesis
      cluster->settx( txOld );
      cluster->setcov(3,3, covtxOld );
      cluster->setpe( -pener );
      cluster->estimateAngles(2,iErr);
      HepDouble tx2 = cluster->tx();
      HepDouble covtx2 = cluster->cov()(3,3);

      int flag2 = 1;
      
      // check for rings, take if it is there
      if( ptToMatchConst()->use_delta_cut ) 
	{
	  if( ptToMatchConst()->use_delta_cut == 2 &&
	      !ptToMatchConst()->energy_delta_cut_ok( *segment, *cluster) )
	    flag2 = 0;
	  else if( !ptToMatchConst()->delta_cut_ok( *segment, *cluster) )
	    flag2 = 0;
	  
	  cluster->setpe( pener );
	  cluster->settx( tx1 );
	  cluster->setcov(3,3, covtx1 );	  
	  if( ptToMatchConst()->use_delta_cut == 2 &&
	      !ptToMatchConst()->energy_delta_cut_ok( *segment, *cluster) )
	    flag1 = 0;
	  else if( !ptToMatchConst()->delta_cut_ok( *segment, *cluster) )
	    flag1 = 0;
	  
	  if( !flag1 && !flag2) return 0 ;       
	}
      

      HepDouble chi22 = MARPLE_INFINITY;
      HepDouble chi21 = MARPLE_INFINITY;
      cluster->settx( tx2 );
      cluster->setcov(3,3, covtx2 );	  
      cluster->setpe( -pener );
      RcMatchPoint * matchPair2 =  new RcMatchPoint( *segment,
						     *cluster, 
						     ptToMatchConst()->flag,
						     lineswim);  
      if( matchPair2->mark() ) flag2= 0;
      cluster->setpe( pener );
      cluster->settx( tx1 );
      cluster->setcov(3,3, covtx1 );	  
      RcMatchPoint * matchPair1 =  new RcMatchPoint( *segment,
						     *cluster, 
						     ptToMatchConst()->flag,
						     lineswim);
      segment->setZ( zold );
      cluster->setpe( pener );
      cluster->settx( txOld );
      cluster->setcov(3,3, covtxOld );
      
      if( matchPair1->mark() ) flag1= 0;
      if( flag2 ) chi22 = matchPair2->chi2();
      if( flag1 ) chi21 = matchPair1->chi2();
      if( chi22 <= chi21 )
	{
	  result = matchPair2;
	  delete matchPair1;
	  result->setmark(40);
	  result->setpe( -pener);
	}else{
	  result = matchPair1;
	  delete matchPair2;
	  result->setmark(4);
	  result->setpe( pener);
	}              
      if( ptToMatchConst()->chi2 < RcMatchConst::maxConsideredChi2 &&
	  result->chi2() > ptToMatchConst()->chi2 )
	{ 
	  delete result;
	  result = 0;
	  return 0;
	}
      matchPair1 = 0;
      matchPair2 = 0;
      //============================================================
      // Match on vertical component. Get momentum from horizontal 
      // and remove all combinations with too small momentum
      //============================================================
    }else{  // it is not needed to estimateAngles, we just create
            // match from cluster and segement
      if( ptToMatchConst()->use_delta_cut ) 
	{
	  if( ptToMatchConst()->use_delta_cut == 2 &&
	      !ptToMatchConst()->energy_delta_cut_ok( *segment, *cluster) )
	    return 0;
	  else if( !ptToMatchConst()->delta_cut_ok( *segment, *cluster) )
	    return 0;
	}

      //------------------------------------------------------------
      // check that direction of segment constarined from cluster-rich_segment
      // crosses magnet field
      static float Zmagnet = 450;
      static float Rmagnet2 = 140*140; // square of magnet radius
      HepDouble AA = cluster->x() - segment->tx()*(cluster->z() - Zmagnet) ;
      if( AA*AA > Rmagnet2*(1+segment->tx()*segment->tx()) ) return 0;


       result =  new RcMatchPoint( *segment,
				   *cluster, 
				   ptToMatchConst()->flag,
				   lineswim);
      
       if(  ptToMatchConst()->chi2 < RcMatchConst::maxConsideredChi2 && 
	    result->chi2() > ptToMatchConst()->chi2 )
	 { 
	  delete result;
	  result = 0;
	  return 0;
	}

      //------------------------------------------------------------
      // estimate momentum 
      // initial variable from trivial formula
      static float bp = 288;
      float overMomentum = (cluster->z()*segment->tx() - cluster->x())/bp;

      if( fabs(overMomentum) > 2 ) // P < 0.5 GeV
	{ 
	  delete result;
	  result = 0;
	  return 0;
	}

      static int epcut = 0;
      if( epcut < 1 )
	{
	  if( MARPLE_SOFT )
	    epcut = 3;
	  else
	    epcut = 2;
	}
      // 3*(1/E) < (1/P) => E/P >3
      if( fabs(overMomentum) > epcut*cluster->pe() ) 
	{ 
	  delete result;
	  result = 0;
	  return 0;
	}

      result->setZ( cluster->z() );
      result->setX( cluster->x() ); 
      result->setY( cluster->y() ); 
      result->setcov(1,1, cluster->cov()(1,1) ); 
      result->setcov(1,2, cluster->cov()(1,2) ); 
      result->setcov(2,2, cluster->cov()(2,2) ); 
      
      /*
      HepDouble tyOld = result->ty();
      HepDouble covOld = result->cov()(4,4);

      result->setty( segment->ty() ); //!!!the best estimation from match?
      result->setcov(4,4,segment->cov()(4,4) );
      */

      result->settx( segment->tx() );
      result->setcov(3,3,segment->cov()(3,3) );
      result->setcov(3,4,segment->cov()(3,4) );

      // 2 GeV < P < 200 GeV
      if( fabs(overMomentum) < 0.5 && fabs(overMomentum) > 0.005 ) 
	{
	  result->setpe( overMomentum );
	  //float resol = overMomentum*overMomentum; 
	  result->setcov(5,5, 10 );	  
	  result->estimatePEfromTAR(10, iErr);      
	}
      else
	{
	  result->setpe( 0.001 );
	  result->setcov(5,5, 10 ); // deviation ~= variable	  
	  result->estimatePEfromTAR(10, iErr);      
	}
      // E/P cut 
      if( iErr || fabs(result->pe()) >2 || // momentum less than 0.5
	  fabs(result->pe()) > 3*fabs(cluster->pe()) )
	{ 
	  delete result;
	  result = 0;
	  return 0;
	}

      /*
      // check that it does point to vertex we search only for such tracks

      RcPoint tmpPoint = *result;
      tmpPoint.fastSwim(-5, iErr); // 
      if( tmpPoint.x()
      */
      result->setmark(5);
    }

  return 1;
}
//------------------------------------------------------------
int RcSegClusBuilder::matchRingsNoMagnet(RcSegPoint  * segment, 
					 RcClusPoint * cluster,
					 RcMatchPoint* &result)
{
  result = 0;
  if ( ptToMatchConst() == 0 ) return 0;

  if( ptToMatchConst()->use_delta_cut ) 
    {
      if( ptToMatchConst()->use_delta_cut == 2 &&
	  !ptToMatchConst()->energy_delta_cut_ok( *segment, *cluster) )
	return 0;
      else if( !ptToMatchConst()->delta_cut_ok( *segment, *cluster) )
	return 0;
    }

  HepDouble zold = segment->z();
  segment->setZ( cluster->z() );
  int lineswim = 1;
  result =  new RcMatchPoint( *segment,
			      *cluster, 
			      ptToMatchConst()->flag,
			      lineswim);
  segment->setZ(zold);
  if( result->mark() ||                // check mark of matching
      (ptToMatchConst()->chi2 < RcMatchConst::maxConsideredChi2 && 
       result->chi2() > ptToMatchConst()->chi2 )) // bad chi2
    {
      delete result;
      result = 0;
      return 0;
    }
  result->setmark(4);
  return 1;
}


//----------------------------------------------------
void RcSegClusBuilder::statMC()
{
  fNrealPair          = 0;
  fNfastReal          = 0;
  frNmtra             = 0;
  frNelectron         = 0;
  frNjpsiElectron     = 0;
  frNmtraBrem         = 0;
  frNelectronBrem     = 0;
  frNjpsiElectronBrem = 0;

  if( ArteTable<MTRA>::size() == 0 ) return;

  for( unsigned int iLeft=0; iLeft < vectorLeft().size(); ++iLeft)
    {
      std::vector<ArtePointer<MTRA> > segMtraList = 
	(*vectorLeft()[ iLeft ]).mtraList();
      std::vector<ArtePointer<MTRA> > segBremList = 
	RcMtraListWithBrem(segMtraList);
      for( unsigned int iRight=0; iRight < vectorRight().size(); ++iRight)
        {
	  std::vector<ArtePointer<MTRA> > clusMtraList = 
	    (*vectorRight()[ iRight ]).mtraList();
	  std::vector<ArtePointer<MTRA> > clusBremList = 
	    RcMtraListWithBrem(clusMtraList);
	  std::vector<ArtePointer<MTRA> > clusOnlyBremList = 
	    RcMtraListOnlyBrem(clusMtraList);

	  ArtePointer<MTRA> imtra = 
	    RcMtraPointer( segMtraList, clusMtraList );
	  if( imtra != 0 )
	    {	      
	      fNrealPair++; 
	      frNmtra++;
	      if( RcMtraReference(imtra) )
		{
		  fNfastReal++;
		}
	      if( RcMtraReferenceOk( RcElepton(segMtraList), 
				     RcElepton(clusMtraList)) )frNelectron++;
	      if( RcMtraReferenceOk( RcEleptonFromJpsi(segMtraList),
				     RcEleptonFromJpsi(clusMtraList)) ) 
		frNjpsiElectron++;
	    }else{
	      imtra = RcMtraPointer(segBremList, clusOnlyBremList);
	      if( imtra != 0 ) 
		{ 
		  fNrealPair++; 
		  frNmtraBrem++;
		  if( RcMtraOk( RcElepton(segBremList), 
				RcElepton(clusOnlyBremList)) )
		    frNelectronBrem++;
		  if( RcMtraOk( RcEleptonFromJpsi(segBremList), 
				RcEleptonFromJpsi(clusOnlyBremList)))
		    frNjpsiElectronBrem++;
		}
	    }
	}
    }
  
  fNbrem = 0;
  fNmtra             = 0;
  fNelectron         = 0;
  fNjpsiElectron     = 0;
  fNmtraBrem         = 0;
  fNelectronBrem     = 0;
  fNjpsiElectronBrem = 0;

  int nFoundRef = 0; 

  for( unsigned int iMatch=0; iMatch < matchList().size(); ++iMatch )
    {
      std::vector<ArtePointer<MTRA> > segMtraList = 
	(*leftOrigin(iMatch)).mtraList();
      std::vector<ArtePointer<MTRA> > clusMtraList = 
	(*rightOrigin(iMatch)).mtraList();

      // non-bremstrahlung pairs
      if( matchList()[iMatch]->mark() != 3 )
	{
	  ArtePointer<MTRA> imtra = RcMtraPointer( segMtraList, 
						   clusMtraList );
	  if( imtra != 0 )
	    { 
	      fNmtra++;
	      if( RcMtraReference(imtra) )
		{
		  nFoundRef++;
		  if( RcMtraReferenceOk( RcElepton(segMtraList), 
					 RcElepton(clusMtraList)) )fNelectron++;
		  if( RcMtraReferenceOk( RcEleptonFromJpsi(segMtraList),
					 RcEleptonFromJpsi(clusMtraList)) )
		    fNjpsiElectron++;
		}
	    }
	}else{
	  // bremstrahlung 
	  fNbrem++;
      
	  std::vector<ArtePointer<MTRA> > segBremList = 
	    RcMtraListWithBrem(segMtraList);
	  std::vector<ArtePointer<MTRA> > clusBremList = 
	    RcMtraListWithBrem(clusMtraList);
	  std::vector<ArtePointer<MTRA> > clusOnlyBremList = 
	    RcMtraListOnlyBrem(clusMtraList);

	  if( RcMtraOk(segBremList, clusOnlyBremList)  )
	    {
	      fNmtraBrem++;
	      if( RcMtraOk( RcElepton(segBremList), 
			    RcElepton(clusOnlyBremList)) )fNelectronBrem++;
	      if( RcMtraOk( RcEleptonFromJpsi(segBremList), 
			    RcEleptonFromJpsi(clusOnlyBremList)))
		fNjpsiElectronBrem++;
	    }
	}
    }
  int NgoodRef       = nFoundRef + NmtraBrem();
  fNbadBrem          = Nbrem() - NmtraBrem();
  fNbad              = matchList().size() - Nbrem() - Nmtra() ;
  fNlost             = NrefPair() - NgoodRef;
}


//--------------------------------------------------------------
void RcSegClusBuilder::stat() const
{
  if( MARPLE_DEBUG > 0 && MARPLE_DEBUG != 10 && MARPLE_DEBUG != 20 )
    {
      static int first = 1;
      if( first )
	{
	  stat(1,0);
	  first = 0;
	}
      stat(MARPLE_DEBUG,0);
    }
}
//--------------------------------------------------------------
void RcSegClusBuilder::stat( int iLong, int iType ) const
{
  unsigned int i;
  switch (iLong)
    {
    case 2 :
      arteMsg << "Left Points :" << vectorLeft().size(); 
      arteMsg.allp();
      for( i=0; i< vectorLeft().size(); ++i)
        {  cout << *(vectorLeft()[i]) ; }
      
      arteMsg << "Right Points :" << vectorRight().size(); 
      arteMsg.allp();
      for( i=0; i< vectorRight().size(); ++i)
        {  cout << *(vectorRight()[i]) ;  }
      
      arteMsg << "Match Points : "<< matchList().size(); 
      arteMsg.allp();
      for( i=0; i< matchList().size(); ++i)
        {  cout << *(matchList()[i]) ;  }

    case 1 :
      cout << "MARPLE: *** constants used for matching ****" << endl 
	   << *ptToMatchConst(); 
      cout << "=== bremstralung constants " << endl
	   << *ptToBremConst(); 

    case 0 :
    default :     
      arteMsg << "*********** result of matching *******";
      arteMsg.allp();
      arteMsg << "Left Points      : " << vectorLeft().size(); 
      arteMsg.allp();
      arteMsg << "Right Points     : " << vectorRight().size(); 
      arteMsg.allp();
      arteMsg << "All pairs with the  same MTRA: " << NrealPair() ; 
      arteMsg.allp();
      if( iType ==  0 )
	{
      arteMsg << "All Match Points     : " << matchList().size();
      arteMsg.allp();
      arteMsg << "Brem Match Points    : " << Nbrem();
      arteMsg.allp();
	}else if(iType == -1 ){
      arteMsg << "Brem Match Points    : " << Nbrem();
      arteMsg.allp();
	}else if(iType == 1 ){
      arteMsg << "Non-Brem Match Points: " << matchList().size() - Nbrem();
      arteMsg.allp();
	}
      if( iType ==  0 || iType == 1)
	{
      arteMsg << "=======================  real = found  = lost" ;
      arteMsg.allp();
      arteMsg << "== same ref MTRA           " << NrefPair() << "   " 
	      << 1-Nlost() ;
      arteMsg.allp();
      arteMsg << "== same electron           " << rNelectron() << "   "
	      << Nelectron();
      arteMsg.allp();
      arteMsg << "== same Jpsi electron      " << rNjpsiElectron() 
	      << "   "<< NjpsiElectron();
      arteMsg.allp();
	}
      if( iType ==  0 || iType == -1)
	{
      arteMsg << "=bremstralung =========   real = found  = lost" ;
      arteMsg.allp();
      arteMsg << "== same brem MTRA          " << rNmtraBrem() << "   "
	      << NmtraBrem();
      arteMsg.allp();
      arteMsg << "== same brem electron      " << rNelectronBrem() << "   "
	      << NelectronBrem();
      arteMsg.allp();
      arteMsg << "== same brem Jpsi-electron " << rNjpsiElectronBrem() << "   "
	      << NjpsiElectronBrem();
      arteMsg.allp();
	}
      if( iType ==  0 )
	{
      arteMsg << "Bad  pairs       : " << Nbad() ; 
      arteMsg.allp();
      arteMsg << "Bad  Brem pairs  : " << NbadBrem() ; 
      arteMsg.allp();
      arteMsg << "Lost pairs       : " << Nlost(); 
      arteMsg.allp();
	}
      if( iType == 1)
	{
      arteMsg << "Bad  Non-Brem pairs  : " << Nbad() - NbadBrem() ; 
      arteMsg.allp();
	}
      if( iType == -1)
	{
      arteMsg << "Bad Brem pairs  : " << NbadBrem() ; 
      arteMsg.allp();
	}
    }
}

//----------------------------------------------------------
//----------------------------------------------------------
void RcSegClusBuilder::saveRSEG(int & iErr ) const
{
  iErr = 0;
  for( unsigned int iMatch=0; iMatch < fmatchList.size(); ++iMatch )
    {
      RSEG rseg;
      RcMatchPoint * matchPoint = fmatchList[iMatch];      

      rseg.zf  = matchPoint->z();
      rseg.xf  = matchPoint->x();
      rseg.yf  = matchPoint->y();
      rseg.txf = matchPoint->tx();
      rseg.tyf = matchPoint->ty();
      rseg.pf  = matchPoint->pe();

      for( unsigned int i = 0; i < 5 ; ++i )
	for( unsigned int j = i; j < 5 ; ++j )
	  rseg.cvf[c_matrix_pack[i][j]] = matchPoint->cov()(i+1,j+1);

      rseg.chi = matchPoint->chi2();
      rseg.fit = 7;
      rseg.dtim = matchPoint->mark();

      RcSegPoint* ptRp1   =  leftOrigin(iMatch);
      RcClusPoint* ptRp2  =  rightOrigin(iMatch);
      switch ( matchPoint->mark() )
        {
        case 5 :          // rings with estimation of momentum  
          rseg.time = 888846;
          break;          
        case -4 :          // rings for negative charge
          rseg.time = 888846;
          break;
        case 4 :          // rings
          rseg.time = 888846;
          break;
        case 3 :          // bremstralung 
          rseg.time = 888817;
          break;
        case 2 :
        case -2 :          // matching using energy of cluster
          rseg.time = 888816;
          break;
        case -1 :
        case 1 :          // matching with segment momentum estimation
          rseg.time = 888816;
          break;
        case 0  :
        default :
          rseg.time = 888888;
          break;
        }
 

      rseg.cmp   = ptRp1->link()->cmp | Rsegc::ecal;

      rseg.xe  =  matchPoint->deltaX();
      rseg.ye  =  matchPoint->deltaY();
      rseg.txe =  matchPoint->deltaTX();
      rseg.tye =  matchPoint->deltaTY();
      rseg.pe  =  matchPoint->deltaPE();

      for( unsigned int i = 0; i < 15 ; ++i )
	rseg.cve[i] = 0;
      
      rseg.cve[0] = ptRp1->link() - *ArteTable<RSEG>::begin()+1;
      rseg.cve[1] = ptRp2->link() - *ArteTable<RCCL>::begin()+1;


      ArtePointer<RSEG> Rseg=ArteTable<RSEG>::push_back(rseg);
      int iErr1 = 0;
      if(!MARPLE_BKGR)saveRelationWithMatch( ptRp1, ptRp2, Rseg, iErr1 );
      if( iErr1 ) iErr = iErr1;

      // fill MTRA information
      if( ArteTable<MTRA>::size() != 0 )
	{
	  ArtePointer<MTRA> imtra = RcMtraPointer( ptRp1->mtraList(), 
						   ptRp2->mtraList());
	  if( imtra != 0 )
	    createRelation(imtra,Rseg,&RSEG::mtra);
	}

    } 
}


//----------------------------------------------------------
//----------------------------------------------------------
void RcSegClusBuilder::fillMatch( int iNtup) const
{
  if( ! FORTRAN_ROUTINE( hexist )(&iNtup) ) 
    {
      int type = 2;  
      FORTRAN_ROUTINE( mamakent )(&iNtup, &type);  
    }
  
  ArtePointer<EVHD> event;
  event.setInt(1);
  FORTRAN_COMMON(matchnt).Nevt   = event->evt;
  FORTRAN_COMMON(matchnt).Nleft  = vectorLeft().size();
  FORTRAN_COMMON(matchnt).Nright = vectorRight().size();
  FORTRAN_COMMON(matchnt).Nmatch = matchList().size();
  FORTRAN_COMMON(matchnt).Nreal  = NrealPair();
  FORTRAN_COMMON(matchnt).Nbad   = Nbad() + NbadBrem();
  FORTRAN_COMMON(matchnt).Nlost  = Nlost() ;

  if( NbadBrem() )
    {
      FORTRAN_COMMON(matchnt).RbadB = float(NbadBrem()) /(float)Nbrem();
    }else{
      FORTRAN_COMMON(matchnt).RbadB     = 0;
    }
  
  if( matchList().size()-Nbrem() )
    {
      FORTRAN_COMMON(matchnt).Rbad = float(Nbad()) 
	/ float(matchList().size()-Nbrem());
    }else{
      FORTRAN_COMMON(matchnt).Rbad     = 0;
    }
  
  if( NrefPair() != 0)
    {
      FORTRAN_COMMON(matchnt).Rlost = (float)Nlost()/ (float)NrefPair();
    }else{
      FORTRAN_COMMON(matchnt).Rlost = 0;
    }
  if( rNmtra() != 0)
    {
      FORTRAN_COMMON(matchnt).Rlmtra = 1-
	(float)Nmtra()/ (float)rNmtra();
    }else{
      FORTRAN_COMMON(matchnt).Rlmtra = 0;
    }
  if( rNelectron() != 0)
    {
      FORTRAN_COMMON(matchnt).Rlelec = 1- 
	(float)Nelectron()/(float)rNelectron();
    }else{
      FORTRAN_COMMON(matchnt).Rlelec = 0;
    }
  if( rNjpsiElectron() != 0)
    {
      FORTRAN_COMMON(matchnt).Rlelecj = 1 - (float)NjpsiElectron()/ 
	(float)rNjpsiElectron();
    }else{
      FORTRAN_COMMON(matchnt).Rlelecj = 0;
    }
  if( rNmtraBrem() != 0)
    {
      FORTRAN_COMMON(matchnt).RlmtraB = 1 - 
	(float)NmtraBrem()/ (float)rNmtraBrem();
    }else{
      FORTRAN_COMMON(matchnt).RlmtraB = 0;
    }
  if( rNelectronBrem() != 0)
    {
      FORTRAN_COMMON(matchnt).RlelecB = 1 -
	(float)NelectronBrem()/ (float)rNelectronBrem();
    }else{
      FORTRAN_COMMON(matchnt).RlelecB = 0;
    }
  if( rNjpsiElectronBrem() != 0)
    {
      FORTRAN_COMMON(matchnt).RlelecjB = 1 - 
	(float)NjpsiElectronBrem()/ (float)rNjpsiElectronBrem();
    }else{
      FORTRAN_COMMON(matchnt).RlelecjB = 0;
    }



  FORTRAN_COMMON(matchnt).fromJpsi = NjpsiElectron();

  if( iNtup )  FORTRAN_ROUTINE( hfnt)(&iNtup); 
}


//---------------------------------------------------------
void RcSegClusBuilder::fillDeltas(int iNtup, int pulsNtup) const
{
  if( ! FORTRAN_ROUTINE( hexist )(&iNtup) ) 
    {
      int type = 3;  
      FORTRAN_ROUTINE( mamakent )(&iNtup, &type);  
    }
  if( pulsNtup && ! FORTRAN_ROUTINE( hexist )(&pulsNtup) ) 
    {
      int type = 4;
      FORTRAN_ROUTINE( mamakent )(&pulsNtup, &type);  
    }

  for( unsigned int iMatch=0; iMatch < matchList().size(); iMatch++ )
    {
      RcMatchPoint * ptMatchPoint = matchList()[iMatch];
      FORTRAN_COMMON(deltas).chi2 = (*ptMatchPoint).chi2();
      FORTRAN_COMMON(deltas).quality = (*ptMatchPoint).quality();

      FORTRAN_COMMON(deltas).matchX  = ptMatchPoint->x();
      FORTRAN_COMMON(deltas).matchY  = ptMatchPoint->y();
      FORTRAN_COMMON(deltas).matchTX = ptMatchPoint->ty();
      FORTRAN_COMMON(deltas).matchTY = ptMatchPoint->tx();
      FORTRAN_COMMON(deltas).matchPE = ptMatchPoint->pe();
      
      RcPoint * ptRp1 = leftOrigin(iMatch);
      RcClusPoint * ptRp2 = rightOrigin(iMatch);

      int iErr;

      FORTRAN_COMMON(deltas).deltaX  = ptMatchPoint->deltaX();
      FORTRAN_COMMON(deltas).deltaY  = ptMatchPoint->deltaY();
      FORTRAN_COMMON(deltas).deltaTX = ptMatchPoint->deltaTX();
      FORTRAN_COMMON(deltas).deltaTY = ptMatchPoint->deltaTY();
      FORTRAN_COMMON(deltas).deltaPE = ptMatchPoint->deltaPE();

      if( ptToMatchConst()->use_delta_cut == 2 )
	{
	  HepDouble energyCorr = sqrt(fabs(ptRp2->pe()));
	  if( energyCorr > MARPLE_INFINITY || 
	      energyCorr < MARPLE_NEGLIGIBLE ) energyCorr = 1;

	  FORTRAN_COMMON(deltas).deltaX  *= energyCorr ;
	  FORTRAN_COMMON(deltas).deltaY  *= energyCorr ;
	  FORTRAN_COMMON(deltas).deltaTX *= energyCorr ;
	  FORTRAN_COMMON(deltas).deltaTY *= energyCorr ;
	  FORTRAN_COMMON(deltas).deltaPE *= energyCorr ;
	}

      //---------------------------------------------------------------------
      // MC information
      if( ArteTable<MTRA>::size() != 0 )
	{
	  std::vector<ArtePointer<MTRA> > segMtraList = (*ptRp1).mtraList();
	  std::vector<ArtePointer<MTRA> > segBremList = 
	    RcMtraListWithBrem(segMtraList);

	  std::vector<ArtePointer<MTRA> > clusMtraList = (*ptRp2).mtraList();
	  std::vector<ArtePointer<MTRA> > clusBremList = 
	    RcMtraListWithBrem(clusMtraList);
	  std::vector<ArtePointer<MTRA> > clusOnlyBremList = 
	    RcMtraListOnlyBrem(clusMtraList);


	  if( matchList()[iMatch]->mark() != 3 )
	    {
	      ArtePointer<MTRA> imtra = RcMtraPointer( segMtraList, 
						       clusMtraList );
	      if( imtra != 0 )
		{ 
		  if( ! RcMtraReference( imtra ) )
		    FORTRAN_COMMON(deltas).flag    = 10; // good pair
		  else
		    {		
		      if( RcMtraReferenceOk( RcEleptonFromJpsi(segMtraList),
					     RcEleptonFromJpsi(clusMtraList)) )
			FORTRAN_COMMON(deltas).flag    = 13; // ref J/psi elec
		      else if( RcMtraReferenceOk( RcElepton(segMtraList), 
						  RcElepton(clusMtraList)) )
			FORTRAN_COMMON(deltas).flag    = 12; // ref electron
		      else 
			FORTRAN_COMMON(deltas).flag    = 11; // ref mtra
		    }
		}else{
		  FORTRAN_COMMON(deltas).flag    = 0;
		}
	    }else{
	      if( RcMtraOk(segBremList, clusOnlyBremList)  )
		{
		  FORTRAN_COMMON(deltas).flag    = 1;
		  if( RcMtraOk( RcElepton(segBremList), 
				RcElepton(clusOnlyBremList)) )
		    FORTRAN_COMMON(deltas).flag    = 2;
		  if( RcMtraOk( RcEleptonFromJpsi(segBremList), 
				RcEleptonFromJpsi(clusOnlyBremList)))
		    FORTRAN_COMMON(deltas).flag    = 3;
		}else{
		  FORTRAN_COMMON(deltas).flag    = -1;
		}
	    }
	  
	  int ipart;
	  ArtePointer<MTRA> mtra;
	  if( ptRp1->mtraList().size() !=0)
	    {
	      mtra  = ptRp1->mtraList()[0];
	      float momentum = sqrt( mtra->px*mtra->px + mtra->py*mtra->py +
				     mtra->pz*mtra->pz );
	      FORTRAN_COMMON(deltas).leftp = momentum;
	      ipart = mtra->topp->lund;
	      FORTRAN_COMMON(deltas).ipart1  = ipart;
	      mtra  = mtra->btra;
	      ipart = mtra->topp->lund;
	      FORTRAN_COMMON(deltas).imoth1  = ipart;
	      
	    }else{
	      FORTRAN_COMMON(deltas).ipart1  = 0;
	      FORTRAN_COMMON(deltas).imoth1  = 0;
	      if( fabs( ptRp1->pe() )> 0.1e-20 )
		FORTRAN_COMMON(deltas).leftp   = 1/ ptRp1->pe();
	      else
		FORTRAN_COMMON(deltas).leftp   = 0;
	    }
	  
	  
	  if( ((*ptRp2).mtraList()).size() != 0 )
	    {
	      mtra  = (*ptRp2).mtraList()[0];
	      ipart = mtra->topp->lund;
	      FORTRAN_COMMON(deltas).ipart2  = ipart;
	      mtra  = mtra->btra;
	      ipart = mtra->topp->lund;
	      FORTRAN_COMMON(deltas).imoth2  = ipart;
	    }else{
	      FORTRAN_COMMON(deltas).ipart2  = 0;
	      FORTRAN_COMMON(deltas).imoth2  = 0;
	    }
	  
	}else{
	  FORTRAN_COMMON(deltas).flag = ptMatchPoint->mark();
	  if( fabs( ptRp1->pe() )> MARPLE_NEGLIGIBLE )
	    FORTRAN_COMMON(deltas).leftp   = 1/ ptRp1->pe();
	  else
	    FORTRAN_COMMON(deltas).leftp   = 0;
	}

      if( fabs( ptRp2->pe() ) > MARPLE_NEGLIGIBLE ) 
	FORTRAN_COMMON(deltas).rightp   = 1/ptRp2->pe(); // energy of cluster
      else 
	FORTRAN_COMMON(deltas).rightp   = 0; // energy of cluster

      if( iNtup ) FORTRAN_ROUTINE( hfnt)(&iNtup); 

      //--------------------------------------------------
      // fill Pulses if specified
      //--------------------------------------------------

      if( pulsNtup == 0 )   continue;

      RcSegPoint tmpPoint =  *ptRp1;
      RcClusPoint cluster =  *ptRp2;

      //HepDouble txOld = cluster.tx();
      //HepDouble covtxOld = cluster.cov()(3,3);
      switch ( ptMatchPoint->mark() )
	{
	case 5 :
	  // rings with estimation of momentum	
	  tmpPoint.setpe( ptMatchPoint->pe());
	  //if( tmpPoint.pe() < 0 ) cluster.setpe( -cluster.pe() );
	  //cluster.estimateAngles(2,iErr);
	  //if( tmpPoint.pe() < 0 ) cluster.setpe( -cluster.pe() );
	  break;	  
	case 40 :
	  // rings for negative charge
	  tmpPoint.setZ( cluster.z() );
	  cluster.setpe( -cluster.pe() );
	  cluster.estimateAngles(2,iErr);
	  break;
	case 4 :
	  // rings
	  tmpPoint.setZ( cluster.z() );
	  cluster.estimateAngles(2,iErr);
	  break;
	case 3 :
	  // bremstralung 
	  tmpPoint.lineswim( ptRp2->z() );
	  break;
	case 2 :
	case -2 :
	  // matching using energy of cluster
	  tmpPoint.lineswim( ptRp2->z() );      
	  if( tmpPoint.x() < ptRp2->x() )
	    {
	      tmpPoint = *ptRp1;
	      tmpPoint.setpe( ptRp2->pe() );
	      tmpPoint.setcov(5,5, ptRp2->cov()(5,5) );
	    }else{
	      tmpPoint = *ptRp1;
	      tmpPoint.setpe( -ptRp2->pe() );
	      tmpPoint.setcov(5,5, ptRp2->cov()(5,5) );
	    }
	  tmpPoint.prolongate(ptRp2->z(), iErr);
	  break;
	case -1 :
	case 1 :
	  // matching with segment momentum estimation
	  tmpPoint.setpe( ptMatchPoint->pe() );
	  /*tmpPoint.estimatePEfromTAR(ptRp2->x(), ptRp2->y(),
				     ptRp2->z(), 
				     ptRp2->cov()(1,1),
				     ptRp2->cov()(2,2),
				     ptRp2->cov()(1,2),
				     3, iErr );
	  if( iErr ) continue;
				     */
	  tmpPoint.prolongate(ptRp2->z(), iErr);
	  break;
	case 0  :
	default :
	  tmpPoint.prolongate(ptRp2->z(), iErr);
	  break;
	}

      if( ptToMatchConst()->flag.ifmarked_X() )
	{
	  FORTRAN_COMMON(deltas).lpulsX = ( ptMatchPoint->x() - tmpPoint.x() )
	    / sqrt(tmpPoint.cov()(1,1));
	  FORTRAN_COMMON(deltas).rpulsX = ( ptMatchPoint->x() - cluster.x() )
	    / sqrt(cluster.cov()(1,1));
	}else{
	  FORTRAN_COMMON(deltas).lpulsX = 0;
	  FORTRAN_COMMON(deltas).rpulsX = 0;
	}

      if( ptToMatchConst()->flag.ifmarked_Y() )
	{
	  FORTRAN_COMMON(deltas).lpulsY = ( ptMatchPoint->y() - tmpPoint.y() )
	    / sqrt(tmpPoint.cov()(2,2));
	  FORTRAN_COMMON(deltas).rpulsY = ( ptMatchPoint->y() - cluster.y() )
	    / sqrt(cluster.cov()(2,2));
	}else{
	  FORTRAN_COMMON(deltas).lpulsY = 0;
	  FORTRAN_COMMON(deltas).rpulsY = 0;
	}

      if( ptToMatchConst()->flag.ifmarked_TX() )
	{
	  FORTRAN_COMMON(deltas).lpulsTX = ( ptMatchPoint->tx() - tmpPoint.tx())
	    / sqrt(tmpPoint.cov()(3,3));
	  FORTRAN_COMMON(deltas).rpulsTX = ( ptMatchPoint->tx() - cluster.tx())
	    / sqrt(cluster.cov()(3,3));
	}else{
	  FORTRAN_COMMON(deltas).lpulsTX = 0;
	  FORTRAN_COMMON(deltas).rpulsTX = 0;
	}

      if( ptToMatchConst()->flag.ifmarked_TY() )
	{
	  FORTRAN_COMMON(deltas).lpulsTY = ( ptMatchPoint->ty() - tmpPoint.ty())
	    / sqrt(tmpPoint.cov()(4,4));
	  FORTRAN_COMMON(deltas).rpulsTY = ( ptMatchPoint->ty() - cluster.ty())
	    / sqrt(cluster.cov()(4,4));
	}else{
	  FORTRAN_COMMON(deltas).lpulsTY = 0;
	  FORTRAN_COMMON(deltas).rpulsTY = 0;
	}

      if( ptToMatchConst()->flag.ifmarked_PE() )
	{
	  FORTRAN_COMMON(deltas).lpulsPE = ( ptMatchPoint->pe() - tmpPoint.pe())
	    / sqrt(tmpPoint.cov()(5,5));
	  FORTRAN_COMMON(deltas).rpulsPE = ( ptMatchPoint->pe() - cluster.pe())
	    / sqrt(cluster.cov()(5,5));
	}else{
	  FORTRAN_COMMON(deltas).lpulsPE = 0;
	  FORTRAN_COMMON(deltas).rpulsPE = 0;
	}
      
      FORTRAN_ROUTINE( hfnt)(&pulsNtup);      

    } 
}

