//**********************************************************************/
// Created: 23.10.97,   Olya Igonkina   for   MARPLE package
//
// implementation of saveRelation() functions of RcMatchBuilder class
// for special casses 
// at the moment implemented function for :
//               RSEG_RSEG (RSEG_RBEG) relation
//               RTRA_RSEG     relation
//               RTRA_RCCL     relation
//               RSEG_RCCL     relation
//
//**********************************************************************/

#include "arte/arte.hh"
#include "arte/ArteMessage.hh"
#include "marple/marple.hh"
#include "marple/RcMatchBuilder.hh"

/*
extern "C" 
{
  void  FORTRAN_ROUTINE(mmbrel)(int &, 
				int *, 
				int *);
}
*/
//-----------------------------------------------------------------
// RcPoint-RcPoint relation  - nothing is done, type of ARTE bank is unknown
//-----------------------------------------------------------------

template<>
void RcMatchBuilder< RcPoint, RcPoint>::saveRelation( 
                                                 const RcPoint * ptPoint1,
						 const RcPoint * ptPoint2,
						 int & iErr) const
{
  iErr = 1;
  return ;
}
//-----------------------------------------------------------------
// RSEG-RSEG relation 
//-----------------------------------------------------------------
/*void createRelation(  ArtePointer<T1> el1, ArtePointer<T2> el2,
                      ArteManRelationPointer<Relation,T2> T1::* e
                      )
{
  mmbrel_(  ArteManRelationPointer<Relation,T2>::getLink().getInt(),
            el2, el1 );
}
*/
template<>
void RcMatchBuilder< RcSegPoint, RcSegPoint>::saveRelation( 
                                                 const RcSegPoint * ptPoint1,
						 const RcSegPoint * ptPoint2,
						 int & iErr) const
{
  iErr = 0;
  if( ptPoint1->link()->zf < ptPoint2->link()->zf )
    {
      mmbrel_( ArteManRelationPointer<RSRS,RSEG>::getLink().getInt(),
	       ptPoint1->link(), ptPoint2->link());
    }else{
      mmbrel_( ArteManRelationPointer<RSRS,RSEG>::getLink().getInt(),
	       ptPoint2->link(), ptPoint1->link());
    }
  return ;
}

//-----------------------------------------------------------------
// RSEG-RCCL relation 
//-----------------------------------------------------------------

template<>
void RcMatchBuilder< RcSegPoint, RcClusPoint>::saveRelation( 
                                               const RcSegPoint * ptPoint1,
					       const RcClusPoint * ptPoint2,
					       int & iErr) const
{
  createRelation( ptPoint1->link(), ptPoint2->link(), &RSEG::rccl);
  iErr = 0;
  return ;
}

template<>
void RcMatchBuilder< RcClusPoint, RcSegPoint>::saveRelation( 
                                               const RcClusPoint * ptPoint1,
					       const RcSegPoint * ptPoint2,
					       int & iErr) const
{
  createRelation( ptPoint1->link(), ptPoint2->link(), &RSEG::rccl);
  iErr = 0;
  return ;
}

//-----------------------------------------------------------------
// RTRA-RSEG relation 
//-----------------------------------------------------------------

template<>
void RcMatchBuilder< RcTraPoint, RcSegPoint>::saveRelation( 
                                              const RcTraPoint * ptPoint1,
					      const RcSegPoint * ptPoint2,
					      int & iErr) const
{
  createRelation( ptPoint1->link(), ptPoint2->link(), &RTRA::rseg);
  iErr = 0;
  return ;
}

template<>
void RcMatchBuilder< RcSegPoint, RcTraPoint>::saveRelation( 
                                                 const RcSegPoint * ptPoint1,
						 const RcTraPoint * ptPoint2,
						 int & iErr) const
{
  createRelation( ptPoint1->link(), ptPoint2->link(), &RTRA::rseg);
  iErr = 0;
  return ;
}

//-----------------------------------------------------------------
// RTRA-RCCL relation 
//-----------------------------------------------------------------

template<>
void RcMatchBuilder< RcTraPoint, RcClusPoint>::saveRelation( 
                                              const RcTraPoint * ptPoint1,
					      const RcClusPoint * ptPoint2,
					      int & iErr) const
{
  createRelation( ptPoint1->link(), ptPoint2->link(), &RTRA::rccl);
  iErr = 0;
  return ;
}

template<>
void RcMatchBuilder< RcClusPoint, RcTraPoint>::saveRelation( 
                                              const RcClusPoint * ptPoint1,
					      const RcTraPoint * ptPoint2,
					      int & iErr) const
{
  createRelation( ptPoint1->link(), ptPoint2->link(), &RTRA::rccl);
  iErr = 0;
  return ;
}
//-------------------------------------------------------------
//-------------------------------------------------------------
//
//  Now we save relations between matched objectes 
//  and also between them and match rseg
//
//-------------------------------------------------------------
//-------------------------------------------------------------


template<>
void RcMatchBuilder< RcSegPoint, RcSegPoint>::saveRelationWithMatch( 
                                                 const RcSegPoint * ptPoint1,
						 const RcSegPoint * ptPoint2,
						 ArtePointer<RSEG> rseg,
						 int & iErr) const
{
  iErr = 0;
  mmbrel_( ArteManRelationPointer<RSRS,RSEG>::getLink().getInt(),
	   rseg, ptPoint1->link());
  mmbrel_( ArteManRelationPointer<RSRS,RSEG>::getLink().getInt(),
	   rseg, ptPoint2->link());
  return ;
}

//-----------------------------------------------------------------
// RSEG-RCCL relation 
//-----------------------------------------------------------------

template<>
void RcMatchBuilder< RcSegPoint, RcClusPoint>::saveRelationWithMatch( 
                                             const RcSegPoint * ptPoint1,
					     const RcClusPoint * ptPoint2,
					     ArtePointer<RSEG> rseg,
					     int & iErr) const
{
  mmbrel_( ArteManRelationPointer<RSRS,RSEG>::getLink().getInt(),
	   rseg, ptPoint1->link());
  createRelation( rseg, ptPoint2->link(), &RSEG::rccl);
  iErr = 0;
  return ;
}

template<>
void RcMatchBuilder< RcClusPoint, RcSegPoint>::saveRelationWithMatch( 
                                             const RcClusPoint * ptPoint1,
					     const RcSegPoint * ptPoint2,
					     ArtePointer<RSEG> rseg, 
					     int & iErr) const
{
  mmbrel_( ArteManRelationPointer<RSRS,RSEG>::getLink().getInt(),
	   rseg, ptPoint2->link());
  createRelation( rseg, ptPoint1->link(), &RSEG::rccl);
  iErr = 0;
  return ;
}

//-----------------------------------------------------------------
// RTRA-RSEG relation 
//-----------------------------------------------------------------

template<>
void RcMatchBuilder< RcTraPoint, RcSegPoint>::saveRelationWithMatch( 
                                                 const RcTraPoint * ptPoint1,
						 const RcSegPoint * ptPoint2,
						 ArtePointer<RSEG> rseg, 
						 int & iErr) const
{
  createRelation( rseg, ptPoint1->link(),  &RTRA::rseg);
  mmbrel_( ArteManRelationPointer<RSRS,RSEG>::getLink().getInt(),
	   rseg, ptPoint2->link());
  iErr = 0;
  return ;
}

template<>
void RcMatchBuilder< RcSegPoint, RcTraPoint>::saveRelationWithMatch( 
                                                 const RcSegPoint * ptPoint1,
						 const RcTraPoint * ptPoint2,
						 ArtePointer<RSEG> rseg, 
						 int & iErr) const
{
  // replace ptPoint1 and ptPoint2
  createRelation( rseg, ptPoint2->link(),  &RTRA::rseg);
  mmbrel_( ArteManRelationPointer<RSRS,RSEG>::getLink().getInt(),
	   rseg, ptPoint1->link());
  iErr = 0;
  return ;
}

//-----------------------------------------------------------------
// RTRA-RCCL relation 
//-----------------------------------------------------------------

template<>
void RcMatchBuilder< RcTraPoint, RcClusPoint>::saveRelationWithMatch( 
				 const RcTraPoint * ptPoint1,
				 const RcClusPoint * ptPoint2,
				 ArtePointer<RSEG> rseg, 
				 int & iErr) const
{
  createRelation( rseg, ptPoint1->link(),  &RTRA::rseg);
  createRelation( rseg, ptPoint2->link(),  &RCCL::rseg);
  iErr = 0;
  return ;
}

template<>
void RcMatchBuilder< RcClusPoint, RcTraPoint>::saveRelationWithMatch( 
                                            const RcClusPoint * ptPoint1,
					    const RcTraPoint * ptPoint2,
					    ArtePointer<RSEG> rseg, 
					    int & iErr) const
{
  //replace ptPoint1 and ptPoint2
  createRelation( rseg, ptPoint2->link(),  &RTRA::rseg);
  createRelation( rseg, ptPoint1->link(),  &RCCL::rseg);
  iErr = 0;
  return ;
}

