// 
//  Author:  Rainer Mankel                     Created:  07-Mar-99
//           Humboldt University Berlin        Modified:  
//  Email:   mankel@ifh.de                     HERA-B Collaboration
// 


// rgdom
//
// This is the heart of ranger. Change it only if you know what you
// are doing. 
//
// Builds complete domain structure with new navigator.
// This version for full modular geometry.
// The domain structure *rangerDO must already exist with default
// domain list defined (this is normally done before by rgdef).
// This is intended to allow the user to change interactively
// the list of domains to be used.
// If *rangerDO is already defined and containing domains, rgdom will
// quit without action.

#include <map.h>
#include "rang/DomObj.hh"
#include "rang/Domain.hh"
#include "rang/DomWin.hh"
#include "rang/DomLay.hh"
#include "rang/DomSeg.hh"
#include "rang/SNM.hh"
#include "rang/SNMList.hh"
#include "rang/SeedTableau.hh"
#include "rang/SeedSet.hh"
#include "rang/rginit.hh"
#include "rang/Geom.hh"
#include "rang/NavItem.hh"
#include "rang/XPro.hh"
#include "rang/XWin.hh"
#include "rang/VLTSet.hh"
#include "rang/VLT.hh"
#include "rang/CompLay.hh"
#include "rang/CompLaySet.hh"
#include "rang/rgzone.hh"

#undef PREVPC1
#undef PROXY

extern rangerDomObj* rangerDO;   // global DO pointer


// helper routine for creating horizontal and vertical seeds 
// in pattern tracker

// PC01 and PC04
void rangerMakeSeed(string dName,const int direction) {
  vector<rangerDomain*>::iterator di=rangerDO->findDom(dName);
  if (di != rangerDO->dom().end()) {
    rangerSeedTableau* st = new  rangerSeedTableau(*di,direction);
    if (st) {
      for (vector <rangerDomWin*>::iterator dw=(*di)->win().begin();
	   dw != (*di)->win().end(); ++dw) {

	if ((*dw)->cmp() == 2) {  // outer tracker seeds
	  rangerSeedSet* ss = new rangerSeedSet(*dw,1,direction,1,2,3);
	  if (ss && ss->OK()) {
	    st->appSeedSet(ss);
	  }
	  else {
	    delete ss;
	  }
	  ss = new rangerSeedSet(*dw,1,direction,1,2,4);
	  if (ss && ss->OK()) {
	    st->appSeedSet(ss);
	  }
	  else {
	    delete ss;
	  }

#ifdef ALL_COMBINATION_PC14
          ss = new rangerSeedSet(*dw,1,direction,1,3,4);
          if (ss && ss->OK()) {
            st->appSeedSet(ss);
          }
          else {
            delete ss;
          }
           ss = new rangerSeedSet(*dw,1,direction,2,3,4);
          if (ss && ss->OK()) {
            st->appSeedSet(ss);
          }
          else {
            delete ss;
          }
#endif
	  ss = new rangerSeedSet(*dw,2,direction,1,2,4);
	  if (ss && ss->OK()) {
	    st->appSeedSet(ss);
	  }
	  else {
	    delete ss;
	  }

	  ss = new rangerSeedSet(*dw,2,direction,1,3,4);
	  if (ss && ss->OK()) {
	    st->appSeedSet(ss);
	  }
	  else {
	    delete ss;
	  }
	}
	else {      // inner tracker seeds
	  rangerSeedSet* ss = new rangerSeedSet(*dw,1,direction,1,3,4);
	  if (ss && ss->OK()) {
	    st->appSeedSet(ss);
	  }
	  else {
	    delete ss;
	  }
	  ss = new rangerSeedSet(*dw,1,direction,2,3,4);
	  if (ss && ss->OK()) {
	    st->appSeedSet(ss);
	  }
	  else {
	    delete ss;
	  }
	  ss = new rangerSeedSet(*dw,2,direction,1,2,4);
	  if (ss && ss->OK()) {
	    st->appSeedSet(ss);
	  }
	  else {
	    delete ss;
	  }
	  ss = new rangerSeedSet(*dw,2,direction,1,3,4);
	  if (ss && ss->OK()) {
	    st->appSeedSet(ss);
	  }
	  else {
	    delete ss;
	  }
	}
      }
      rangerDO->appSeedTab(st);
    }
  }
}

//  PC2 and PC3 
void rangerMakeSeed1(string dName,const int direction) {
  vector<rangerDomain*>::iterator di=rangerDO->findDom(dName);
  if (di != rangerDO->dom().end()) {
    rangerSeedTableau* st = new  rangerSeedTableau(*di,direction);
    if (st) {
      for (vector <rangerDomWin*>::iterator dw=(*di)->win().begin();
	   dw != (*di)->win().end(); ++dw) {

	if ((*dw)->cmp() == 2) {  // outer tracker seeds
	  rangerSeedSet* ss = new rangerSeedSet(*dw,1,direction,1,2,3);
	  if (ss && ss->OK()) {
	    st->appSeedSet(ss);
	  }
	  else {
	    delete ss;
	  }
	  ss = new rangerSeedSet(*dw,2,direction,1,2,3);
	  if (ss && ss->OK()) {
	    st->appSeedSet(ss);
	  }
	  else {
	    delete ss;
	  }
	}
	else {      // inner tracker seeds
#ifdef SEEDS_ITR_IN_PC23 
	  rangerSeedSet* ss = new rangerSeedSet(*dw,1,direction,1,2,3);
	  if (ss && ss->OK()) {
	    st->appSeedSet(ss);
	  }
	  else {
	    delete ss;
	  }
	  ss = new rangerSeedSet(*dw,2,direction,1,2,3);
	  if (ss && ss->OK()) {
	    st->appSeedSet(ss);
	  }
	  else {
	    delete ss;
	  }
#endif
	}
      }
      rangerDO->appSeedTab(st);
    }
  }
}



// now comes the real rgdom

extern "C" {
  void rgdom_() {

    // verify that rangerDO exists
    if (!rangerDO) {
      cout << "rgdom: rangerDO=" << rangerDO << endl;
      return;
    }

    // check if domains already defined
    if (rangerDO->dom().size() > 0) {
      cout << "rgdom: domains are already defined..." << endl;
      return;
    }

    // test initialization
    rginit_();
    rangerGeom::condUpdate();

    // loop over default domains
    for (vector<rangerDomain*>::iterator di=rangerDO->def().begin();
	 di != rangerDO->def().end(); ++di) {
      rangerDomain* dn = (*di)->cloneSl();  // clone default domain
      dn->updSls();                         // set the superlayers
      // windows are created by accumulating GEDEs according
      // to cmp and sector numbers
      dn->addWin(1,1,8,-0.060,0.060,1);     // add windows Si (cmp=1) 
      dn->addWin(2,1,12,-0.002,0.002,20);   // add windows OT (cmp=2)
      dn->addWin(3,1,8,-0.002,0.002,1);     // add windows IT (cmp=3)
      rangerDO->appDom(dn);                 // append this domain
      if (dn->sco() == 3) {
	rangerDomSeg* ds = new rangerDomSeg(dn);
	rangerDO->appDomSeg(ds);            // add domain segments
	dn->setDomSeg(ds);                  // for pattern tracker domains
      }                                     // to store x projections
      //  cout << "rgdom: appended domain " << *dn << endl;
#ifdef DOMPRINT
      dn->dump();
#endif
      //...
    }

    // define sector neighbourhood masks and store them in a list.
    // The SNMs are used to define which GEDEs belong to a domain layer.
    // The list should contain only one SNM per cmp/sec combination.
    rangerSNMList SNMs;

    // outer tracker sectors
    SNMs.add(new rangerSNM(2,1,2,9,2,7));
    SNMs.add(new rangerSNM(2,2,2,8,2,10));
    SNMs.add(new rangerSNM(2,3,2,7,3,2));
    SNMs.add(new rangerSNM(2,4,2,8,3,4));
    SNMs.add(new rangerSNM(2,5,2,9,3,7));
    SNMs.add(new rangerSNM(2,6,2,10,3,5));
    SNMs.add(new rangerSNM(2,7,2,1,2,3,2,11));
    SNMs.add(new rangerSNM(2,8,2,2,2,4,2,11));
    SNMs.add(new rangerSNM(2,9,2,1,2,5,2,12));
    SNMs.add(new rangerSNM(2,10,2,2,2,6,2,12));
    SNMs.add(new rangerSNM(2,11,2,7,2,8));
    SNMs.add(new rangerSNM(2,12,2,9,2,10));
    // inner tracker sectors
    SNMs.add(new rangerSNM(3,1,3,2,3,8));
    SNMs.add(new rangerSNM(3,2,2,3,3,1));
    SNMs.add(new rangerSNM(3,3,3,4,3,6));
    SNMs.add(new rangerSNM(3,4,2,4,3,3));
    SNMs.add(new rangerSNM(3,5,2,6,3,6));
    SNMs.add(new rangerSNM(3,6,3,3,3,5));
    SNMs.add(new rangerSNM(3,7,2,5,3,8));
    SNMs.add(new rangerSNM(3,8,3,1,3,7));
    // vds sectors
    SNMs.add(new rangerSNM(1,1));
    SNMs.add(new rangerSNM(1,2));
    SNMs.add(new rangerSNM(1,3));
    SNMs.add(new rangerSNM(1,4));
    
//     cout << " SNM test..." << endl;
//     SNMs.test(3);
//     cout << " .............." << endl;

    // create VLTs
//     rangerVLTSet vs1;
//     vs1.append(new rangerVLT(2,1,2,3,4,5,6));
//     vs1.append(new rangerVLT(3,1,6,2,3,5,6));

//     rangerVLTSet vs2;
//     vs2.append(new rangerVLT(2,1,3,4,6,5,2));
//     vs2.append(new rangerVLT(3,1,2,3,4,5,6));

    // map to contain pointers of defined composite layers
    map<string,rangerCompLaySet*> clMap;

    // define composite layers for more complex layer combinations
    // A composite layer combines layers from different cmp types,
    // typically cmp=2 (OT), cmp=3 (IT) or cmp=1 (Si)

    rangerCompLaySet clset_MC1;   // MC1-type composite layer
    clset_MC1.append
      ( new rangerCompLay( make_pair(1,rangerSubLayerSet(3,1)),
			   make_pair(1,rangerSubLayerSet(1,1)) ) );
    clset_MC1.append
      ( new rangerCompLay( make_pair(1,rangerSubLayerSet(2,1)),
                           make_pair(3,rangerSubLayerSet(3,3)),
			   make_pair(3,rangerSubLayerSet(1,3)) ) );
    clset_MC1.append
      ( new rangerCompLay( make_pair(2,rangerSubLayerSet(2,2)),
                           make_pair(4,rangerSubLayerSet(3,4)),
			   make_pair(4,rangerSubLayerSet(1,2)) ) );
    clset_MC1.append
      ( new rangerCompLay( make_pair(3,rangerSubLayerSet(2,3)),
                           make_pair(2,rangerSubLayerSet(3,2)),
			   make_pair(2,rangerSubLayerSet(1,2)) ) );
    clMap["MC1"] = &clset_MC1;


    rangerCompLaySet clset_MC3;   // MC3-type composite layer
    clset_MC3.append
      ( new rangerCompLay( make_pair(1,rangerSubLayerSet(2,1)) ) );
    clset_MC3.append
      ( new rangerCompLay( make_pair(2,rangerSubLayerSet(2,2)),
			   make_pair(1,rangerSubLayerSet(3,1)) ) );
    clset_MC3.append
      ( new rangerCompLay( make_pair(3,rangerSubLayerSet(2,3)),
			   make_pair(2,rangerSubLayerSet(3,2)) ) );
    clMap["MC3"] = &clset_MC3;
    clMap["MC6"] = &clset_MC3;    // MC6 has same structure as MC3


    rangerCompLaySet clset_MC5;   // MC5-type composite layer
    clset_MC5.append
      ( new rangerCompLay( make_pair(1,rangerSubLayerSet(2,1)),
			   make_pair(1,rangerSubLayerSet(3,1)) ) );
    clset_MC5.append
      ( new rangerCompLay( make_pair(2,rangerSubLayerSet(3,2)) ) );
    clMap["MC5"] = &clset_MC5;


    rangerCompLaySet clset_PC1;   // PC1-type composite layer
    clset_PC1.append
      ( new rangerCompLay( make_pair(1,rangerSubLayerSet(2,1)),
			   make_pair(1,rangerSubLayerSet(3,1)) ) );
    clset_PC1.append
      ( new rangerCompLay( make_pair(2,rangerSubLayerSet(2,2)),
			   make_pair(3,rangerSubLayerSet(3,3)) ) );
    clset_PC1.append
      ( new rangerCompLay( make_pair(3,rangerSubLayerSet(2,3)),
			   make_pair(2,rangerSubLayerSet(3,2)) ) );
    clset_PC1.append
      ( new rangerCompLay( make_pair(4,rangerSubLayerSet(2,4,1,2)),
			   make_pair(6,rangerSubLayerSet(3,6)) ) );
    clset_PC1.append
      ( new rangerCompLay( make_pair(5,rangerSubLayerSet(2,4,3,4)),
			   make_pair(7,rangerSubLayerSet(3,7)) ) );
    clset_PC1.append
      ( new rangerCompLay( make_pair(6,rangerSubLayerSet(2,5,1,2)),
			   make_pair(5,rangerSubLayerSet(3,5)) ) );
    clset_PC1.append
      ( new rangerCompLay( make_pair(7,rangerSubLayerSet(2,5,3,4)),
			   make_pair(8,rangerSubLayerSet(3,8)) ) );
    clset_PC1.append
      ( new rangerCompLay( make_pair(8,rangerSubLayerSet(2,6,1,4)),
			   make_pair(4,rangerSubLayerSet(3,4)) ) );
    clMap["PC1"] = &clset_PC1;

    //clset_PC1.dump(9);


    rangerCompLaySet clset_PC2;   // PC2-type composite layer
    clset_PC2.append
      ( new rangerCompLay( make_pair(1,rangerSubLayerSet(2,1)),
			   make_pair(1,rangerSubLayerSet(3,1)) ) );
    clset_PC2.append
      ( new rangerCompLay( make_pair(2,rangerSubLayerSet(2,2)),
			   make_pair(3,rangerSubLayerSet(3,3)) ) );
    clset_PC2.append
      ( new rangerCompLay( make_pair(3,rangerSubLayerSet(2,3)),
			   make_pair(4,rangerSubLayerSet(3,4)) ) );
    clset_PC2.append
      ( new rangerCompLay( make_pair(4,rangerSubLayerSet(2,4)),
			   make_pair(2,rangerSubLayerSet(3,2)) ) );
    clset_PC2.append
      ( new rangerCompLay( make_pair(5,rangerSubLayerSet(2,5)) ) );
    clset_PC2.append
      ( new rangerCompLay( make_pair(6,rangerSubLayerSet(2,6)) ) );
    clMap["PC2"] = &clset_PC2;
    clMap["PC3"] = &clset_PC2;    // PC3 has same structure as PC2

    rangerCompLaySet clset_PC4;   // PC4-type composite layer
    clset_PC4.append
      ( new rangerCompLay( make_pair(1,rangerSubLayerSet(2,1)),
			   make_pair(1,rangerSubLayerSet(3,1)) ) );
    clset_PC4.append
      ( new rangerCompLay( make_pair(2,rangerSubLayerSet(2,2)),
			   make_pair(3,rangerSubLayerSet(3,3)) ) );
    clset_PC4.append
      ( new rangerCompLay( make_pair(3,rangerSubLayerSet(2,3)),
			   make_pair(2,rangerSubLayerSet(3,2)) ) );
    clset_PC4.append
      ( new rangerCompLay( make_pair(4,rangerSubLayerSet(2,4,1,2)),
			   make_pair(6,rangerSubLayerSet(3,6)) ) );
    clset_PC4.append
      ( new rangerCompLay( make_pair(5,rangerSubLayerSet(2,4,3,4)),
			   make_pair(7,rangerSubLayerSet(3,7)) ) );
    clset_PC4.append
      ( new rangerCompLay( make_pair(6,rangerSubLayerSet(2,5,1,2)),
			   make_pair(5,rangerSubLayerSet(3,5)) ) );
    clset_PC4.append
      ( new rangerCompLay( make_pair(7,rangerSubLayerSet(2,5,3,4)),
			   make_pair(8,rangerSubLayerSet(3,8)) ) );
    clset_PC4.append
      ( new rangerCompLay( make_pair(8,rangerSubLayerSet(2,6,1,4)),
			   make_pair(4,rangerSubLayerSet(3,4)) ) );
    clMap["PC4"] = &clset_PC4;
    //clset_PC4.dump(9);
	
    rangerCompLaySet clset_TC1;   // TC1-type composite layer
    clset_TC1.append
      ( new rangerCompLay( make_pair(1,rangerSubLayerSet(2,1,1,2)),
			   make_pair(3,rangerSubLayerSet(3,3)) ) );
    clset_TC1.append
      ( new rangerCompLay( make_pair(2,rangerSubLayerSet(2,1,3,4)),
			   make_pair(4,rangerSubLayerSet(3,4)) ) );
    clset_TC1.append
      ( new rangerCompLay( make_pair(3,rangerSubLayerSet(2,2,1,2)),
			   make_pair(1,rangerSubLayerSet(3,1)) ) );
    clset_TC1.append
      ( new rangerCompLay( make_pair(4,rangerSubLayerSet(2,2,3,4)),
			   make_pair(2,rangerSubLayerSet(3,2)) ) );
    clset_TC1.append
      ( new rangerCompLay( make_pair(5,rangerSubLayerSet(2,3,1,2)),
			   make_pair(5,rangerSubLayerSet(3,5)) ) );
    clset_TC1.append
      ( new rangerCompLay( make_pair(6,rangerSubLayerSet(2,3,3,4)),
			   make_pair(6,rangerSubLayerSet(3,6)) ) );
    clMap["TC1"] = &clset_TC1;
    clMap["TC2"] = &clset_TC1;    // TC2 has same structure as TC1
    //clset_TC1.dump(9);

    // create the domain layers for each domain.
    // Use the previously defined composite layer sets and
    // sector neighbourhood masks.
    for (vector<rangerDomain*>::iterator di=rangerDO->dom().begin();
	 di != rangerDO->dom().end(); ++di) {   // loop over all domains
#ifdef DOMPRINT
      cout << "Make domain layers for " << *(*di) << endl;
#endif
      for (vector <rangerDomWin*>::iterator wi=(*di)->win().begin();
	   wi != (*di)->win().end(); ++wi) {    // loop over domain windows
	rangerSNM* s = SNMs.getSNM((*wi)->cmp(),(*wi)->sec()); // find SNM
	if (s) {
	  // loop over thinkable GEDE layer numbers, try to create
	  // a domain layer
	  for (int gLayer=1; gLayer<=10; ++gLayer) {
	    // tentative creation of new (still empty) domain layer
	    rangerDomLay* dl = new rangerDomLay(*wi);
	    if (! dl) break;
	    // collect planes for a layer
	    int nAcc = 0;
	    
	    map<string,rangerCompLaySet*>::iterator clmi
	      = clMap.find((*di)->nam()); // find composite layer for domain
	    if (clmi != clMap.end()) {
	      // composite layer is defined, use it to create domain layer
	      nAcc = dl->accumComp(*wi,gLayer,s,*((*clmi).second),-1);
#ifdef VERBOSE
	      cout << "rgdom: layer planes for " << (*di)->nam()
		   << " win " << *(*wi)
		   << " gLayer " << gLayer << " nAcc=" << nAcc << endl;
#endif
	    }
	    else {
	      // use simpler layer combinatorics otherwise
	      nAcc = dl->accum(*wi,gLayer,s,-1);
	    }

	    // check native layer planes (those really having the 
	    // nominal sector number)
	    int nNative = dl->cntLP((*wi)->cmp(),(*wi)->sec());
	    if (nAcc && nNative) {
	      // domain layer creation successful (native layer planes found)
	      (*wi)->addDomLay(dl);  // keep dl and add to window
	      dl->setHL(rangerDO->makeHL(dl));  // prepare hitlist entity
// 	      cout << "  added " << nAcc << " domain layer planes" << endl;
	    }
	    else {
	      // domain layer creation failed (no native layer planes found)
	      delete dl;          // destroy
	      break;
	    }
	  }
	}
// 	cout << "call updLsn for this window" << endl;
	(*wi)->updLsn();    // update window-internal layer sequence number 
	                    // bookkeeping

	// add entrance/exit layers for navigation
	(*wi)->addEntrance(+1,new rangerDomLay(*wi));
	(*wi)->addEntrance(-1,new rangerDomLay(*wi));

// 	(*wi)->dump(5);
      }

#ifdef PROXY
      // add proxy windows and layers
      if ((*di)->sco() >=2 && (*di)->sco() <=4) { // only magnet, pattern, tc
	int nProxy = 0;
	nProxy += (*di)->addProxy(2,9,2,5.,2,10); // proxies for 10mm sectors
	nProxy += (*di)->addProxy(2,10,1,5.,2,9);
	nProxy += (*di)->addProxy(2,7,2,5.,2,8);
	nProxy += (*di)->addProxy(2,8,1,5.,2,7);
	nProxy += (*di)->addProxy(2,5,2,5.,2,6);  // proxies for 5mm middle
	nProxy += (*di)->addProxy(2,6,1,5.,2,5);
	nProxy += (*di)->addProxy(2,3,2,5.,2,4);
	nProxy += (*di)->addProxy(2,4,1,5.,2,3);

	nProxy += (*di)->addProxy(2,1,2,5.,2,5);  // proxies for 4fold division
	nProxy += (*di)->addProxy(2,5,1,5.,2,1);
	nProxy += (*di)->addProxy(2,1,2,5.,2,3);
	nProxy += (*di)->addProxy(2,3,1,5.,2,1);

	nProxy += (*di)->addProxy(2,6,2,5.,2,2);
	nProxy += (*di)->addProxy(2,2,1,5.,2,6);
	nProxy += (*di)->addProxy(2,4,2,5.,2,2);
	nProxy += (*di)->addProxy(2,2,1,5.,2,4);

	nProxy += (*di)->addProxy(2,1,2,5.,3,1);  // proxies outer/inner tr
	nProxy += (*di)->addProxy(2,1,2,5.,3,2);  // above
	nProxy += (*di)->addProxy(2,1,2,5.,3,7);
	nProxy += (*di)->addProxy(2,1,2,5.,3,8);

	nProxy += (*di)->addProxy(3,1,1,5.,2,1);  // reverse
	nProxy += (*di)->addProxy(3,2,1,5.,2,1);
	nProxy += (*di)->addProxy(3,7,1,5.,2,1);
	nProxy += (*di)->addProxy(3,8,1,5.,2,1);

	nProxy += (*di)->addProxy(2,2,1,5.,3,3);  // proxies outer/inner tr
	nProxy += (*di)->addProxy(2,2,1,5.,3,4);  // below
	nProxy += (*di)->addProxy(2,2,1,5.,3,5);
	nProxy += (*di)->addProxy(2,2,1,5.,3,6);

	nProxy += (*di)->addProxy(3,3,2,5.,2,2);  // reverse
	nProxy += (*di)->addProxy(3,4,2,5.,2,2);
	nProxy += (*di)->addProxy(3,5,2,5.,2,2);
	nProxy += (*di)->addProxy(3,6,2,5.,2,2);

	nProxy += (*di)->addProxy(3,2,2,5.,3,4);  // proxies for inner tracker
	nProxy += (*di)->addProxy(3,4,1,5.,3,2);
	nProxy += (*di)->addProxy(3,7,2,5.,3,5);
	nProxy += (*di)->addProxy(3,5,1,5.,3,7);

	int nProxyLay = (*di)->setProxyLayers();
	cout << " Domain " << (*di)->nam() << " proxy windows created "
	     << nProxy << " layers " << nProxyLay << endl;
      }
#endif
    }

    // create domain window navigation table

    // define navigation mask (the SNM class is used)
    // store in a list
    rangerSNMList NNMs;

    // outer tracker sectors
    NNMs.add(new rangerSNM(2,1,2,9,2,7,2,3,2,5,3,7,3,8,3,1,3,2));
    NNMs.add(new rangerSNM(2,2,2,8,2,10,2,4,2,6,3,5,3,6,3,3,3,4));
    NNMs.add(new rangerSNM(2,3,2,7,3,2,2,4,2,1));
    NNMs.add(new rangerSNM(2,4,2,8,3,4,2,3,2,2));
    NNMs.add(new rangerSNM(2,5,2,9,3,7,2,6,2,1));
    NNMs.add(new rangerSNM(2,6,2,10,3,5,2,5,2,2));
    NNMs.add(new rangerSNM(2,7,2,1,2,3,2,11,2,8,2,4));
    NNMs.add(new rangerSNM(2,8,2,2,2,4,2,11,2,7,2,3));
    NNMs.add(new rangerSNM(2,9,2,1,2,5,2,12,2,10,2,6));
    NNMs.add(new rangerSNM(2,10,2,2,2,6,2,12,2,9,2,5));
    NNMs.add(new rangerSNM(2,11,2,7,2,8));
    NNMs.add(new rangerSNM(2,12,2,9,2,10));
    // inner tracker sectors
    NNMs.add(new rangerSNM(3,1,3,2,3,8,2,1,1,1,1,2));
    NNMs.add(new rangerSNM(3,2,2,3,3,1,2,1,3,8,3,4,2,4,1,1,1,2));
    NNMs.add(new rangerSNM(3,3,3,4,3,6,2,2,1,3,1,2));
    NNMs.add(new rangerSNM(3,4,2,4,3,3,2,2,3,6,3,2,2,3,1,2,1,3));
    NNMs.add(new rangerSNM(3,5,2,6,3,6,2,2,3,3,3,7,2,5,1,3,1,4));
    NNMs.add(new rangerSNM(3,6,3,3,3,5,2,2,1,3,1,4));
    NNMs.add(new rangerSNM(3,7,2,5,3,8,2,1,3,1,3,5,2,6,1,4,1,1));
    NNMs.add(new rangerSNM(3,8,3,1,3,7,2,1,1,1,1,4));
    // vds sectors
    NNMs.add(new rangerSNM(1,1,1,2,1,4));
    NNMs.add(new rangerSNM(1,2,1,1,1,3));
    NNMs.add(new rangerSNM(1,3,1,2,1,4));
    NNMs.add(new rangerSNM(1,4,1,3,1,1));

//     NNMs.test(3);
//     NNMs.dump(3);

    // create empty navigation items for each domain window
    for (vector<rangerDomain*>::iterator di=rangerDO->dom().begin();
	 di != rangerDO->dom().end(); ++di) {       // loop over domains
      for (vector <rangerDomWin*>::iterator wIt=(*di)->win().begin();
	   wIt != (*di)->win().end(); ++wIt) {      // loop over windows
	rangerNavItem* ni = new rangerNavItem(*wIt);
	if (! ni) break;
	rangerDO->appNav(ni);             // append to navigation item list
	(*wIt)->setNI(ni);                // set navigation item in window
      }
    }

#ifdef DOMPRINT
    cout << " rgdom: number of NavItems " << rangerDO->nav().size()
	 << endl;
#endif
    // set navigation table for each domain window
    for (vector<rangerNavItem*>::iterator nii=rangerDO->nav().begin();
	 nii != rangerDO->nav().end(); ++nii) {
      if ((*nii)->win()->dom()->sco() == 1) {
	// vertex detector navigation table should be self-contained
	(*nii)->setNavTable(rangerDO->dom().begin(),rangerDO->dom().end(),
			    NNMs,1,1,5);
      }
      else {
        // main tracker navigation table should be self-contained
	(*nii)->setNavTable(rangerDO->dom().begin(),rangerDO->dom().end(),
			    NNMs,2,4,5);
      }
//       (*nii)->dump(2);
    }
    

    // Prepare seeds for ranger pattern tracker analysis
    rangerMakeSeed("PC4",-1);  // PC4 seeds for backward propagation
    rangerMakeSeed("PC1",+1);  // PC1 seeds for forward  propagation
    rangerMakeSeed1("PC3",-1);  // PC3 seeds for backward propagation
    rangerMakeSeed1("PC2",+1);  // PC2 seeds for forward  propagation

#ifdef NAVCHECK
    // check the navigation table

    // locate domain PC4
    for (vector<rangerDomain*>::iterator di=rangerDO->dom().begin();
	 di != rangerDO->dom().end(); ++di) {
      if ((*di)->nam() == "PC4") {
	int method = 0;
	int dir = +1;
	int zone = 6;
	rangerDomLay* dl = (*(*di)->win().begin())->firstDL(method,1);
	cout << " look at window " << endl;
	(*(((*di)->win()).begin()))->dump();
	if (dl) {
	  dl->dump();
	  rangerDomLay* targetVec[100];
	  int nTarget = 0;
	  rgzone_(dl,method,zone,dir,3,3,1,100,targetVec,nTarget);
	  cout << " number of targets=" << nTarget << endl;
	  for (int i=0; i<nTarget && i<100; ++i) {
	    targetVec[i]->win()->dump();
	    targetVec[i]->dump();
	  }
	}
      }
    }
#endif

    // build the x projection structure (only for pattern tracker domains)
    for (vector<rangerDomain*>::iterator di=rangerDO->dom().begin();
	 di != rangerDO->dom().end(); ++di) {
      if ((*di)->sco() == 3) {
	rangerXWin* xwi = new rangerXWin((*di));
	rangerDO->xpro()->appXWin(xwi);
	(*di)->setXWin(xwi);
      }
    }

    // fill the exclusion ranges from OTDR table
    int nExcl = 0;
    for (ArteTable<OTDR>::iterator dri=ArteTable<OTDR>::begin();
	 dri != ArteTable<OTDR>::end(); ++dri) {
      switch ((*dri)->sub) {
      case 1:
	nExcl += rangerDO->addExclusionRange((*dri)->gede,(*dri)->xmin,
					     (*dri)->xmax,1,2);
	break;
      case 3:
	nExcl += rangerDO->addExclusionRange((*dri)->gede,(*dri)->xmin,
					     (*dri)->xmax,3,4);
	break;
      }
    }

#ifdef DOMPRINT
    cout << "rgdom: number of domains: " << rangerDO->dom().size() << endl;
    int nWin = 0;
    for (vector<rangerDomain*>::iterator di=rangerDO->dom().begin();
	 di != rangerDO->dom().end(); ++di) {
      nWin += (*di)->win().size();
    }
    cout << "rgdom: number of windows: " << nWin  << endl;
    cout << "rgdom: number of hitlist items:" 
	 << (rangerDO->hls()).size() << endl;
    cout << "rgdom: number of seed tableaus : " 
	 << (rangerDO->stab()).size() 
	 << " seed sets : " << rangerDO->nSeedSets() << endl;
    cout << "rgdom: number of XWin items : "
	 << (rangerDO->xpro()->xws()).size() << endl;
    cout << "rgdom: number of exclusion ranges : " << nExcl << endl;
#endif
    //exit(1);
    //rangerDO->dump(5);
  }
}

    
