// 
//  Author:  Rainer Mankel                     Created:  29-Sep-98
//           Humboldt University Berlin        Modified:  
//  Email:   mankel@ifh.de                     HERA-B Collaboration
// 
#include <iostream.h>
#include <math.h>
#include "arte/arte.hh"
#include "rang/Fit.hh"
#include "rang/seg2track.hh"
#include "rang/tfmt2seg.hh"
#include "rang/tffhist.hh"
#include "rang/Geom.hh"
#include "rang/rginit.hh"

// rgfit_
// Fortran interface to int rangerFit(ArtePointer<RSEG> rseg,
// rangerParticleType pType).
// Applies ranger fit to a RSEG entry.
// Output is written back to the corresponding RSEG/RHIT/RPNT entities.
// Second parameter specifies if electron radiation correction is wanted.
// Self-initialized.

extern "C" {
  int rgfit_(const int& rs,  // row number of RSEG to be fitted 
	     const int &elec_corr){  // electron correction mode
                                     //    1 - on
                                     //    0 - off
    if (ArtePointer<RSEG>::nrow() <= 0) return 0;

    // geometry initialization
    rginit_();
    rangerGeom::condUpdate();

    ArtePointer<RSEG> rseg(rs);
    int fitStatus = (elec_corr) ? rangerFit(rseg,ELECTRON) : rangerFit(rseg);
    return fitStatus;
  }
}

