
// 
//  Author:  A. Spiridonov                     Created:  04-Okt-99
//           DESY/Zeuthen                      Modified:  
//  Email:   spiridonl@ifh.de                  HERA-B Collaboration
// 

// rc99AOTR.C
// reconstruction of OTR
//---------------------------
//      modifications:                     
//        - added momentum estimation for otr-segments with zero momentum
//
//                                                   15/06/00 D. Samtleben
//        - momentum estimation done for all otr-segments 
//                                                   20/06/00 D. Samtleben

#include "reco/rc99Areco.hh"
#include "arte/arte.hh"
#include "arte/RSEG_cmp.hh"
#include "Otr/OtrEvent.h"
#include <iostream.h>
#include <stdlib.h>

#include "marple/RcSegPoint.hh"


// Prototypes

void waxevnt();
int rc99removeBadRseg();

extern "C" {
  void tfevntr_();
}

//static int otrfirst=-1;
static int maxhits=500000;
static int hitprepflag=-1;

int rc99AOTR(int level) { 

//==================================================================
// 0. use steering variables if set.
//==================================================================
  if (hitprepflag==-1) {
    char *f;
    
/*                    hit preparation for OTR  */
    if ((f=getenv("OtrHitPreparation"))) hitprepflag = atoi(f);
    else hitprepflag=1;

/* not supported anymore, just for backward compability */    
    if ((f=getenv("WAX_OCCUPCUT"))) maxhits = atoi(f);

/* use this one instead */    
    if ((f=getenv("OtrOccupancyCut"))) maxhits = atoi(f);
   
    if (maxhits<500000) {    
      printf("rc99AOTR[warning]: Occupancy cut!!! Number of hits lower %d\n",
          maxhits);
      }
       
    }
    
  //==================================================================
  // 1. create hits:  fill HITB table with OTR hits
  //==================================================================

  if (level ==1 || level ==3 ) {
    int notrhits = ArteTable<HITB>::size();
    switch  (hitprepflag) {
      case 0:
        OtrEventUpdate();
        break;
      case 1:
        waxevnt();
        break;
      case 2:
        waxevnt();
        OtrEventUpdate();
        break;
    }

    notrhits = ArteTable<HITB>::size() - notrhits;
    if (notrhits > maxhits) return -1;
  }



//                    ranger tracking

  if ( level > 1 ) {
    tfevntr_();
    //int removeflag=
    rc99removeBadRseg();

//-- include momentum estimation

  for( ArteTable<RSEG>::iterator irseg = ArteTable<RSEG>::begin();
       irseg != ArteTable<RSEG>::end(); ++irseg ) 
    {
      if ( (*irseg)->fit != 0 ) continue;
      if( ((*irseg)->cmp & Rsegc::bitsign) != 0 ) continue;
//      if(  (*irseg)->cmp & Rsegc::patt && fabs((*irseg)->pf)< 1.e-5 )
      if(  ((*irseg)->cmp & Rsegc::patt) && !((*irseg)->cmp & Rsegc::magt) )
        {
          RcSegPoint  otr( (*irseg), 0 ); //first point
          // initial value from kick :
          float overMomentum = ( otr.tx()*otr.z() - otr.x() ) / 288;
          if( fabs(overMomentum) <  2 ) // Pt > 0.5 GeV
            {
              otr.setpe( overMomentum);
              int iErr;
              otr.estimatePEfromTAR( 5, iErr);
              (*irseg)->pf = otr.pe();
            }else{
              (*irseg)->pf = overMomentum;
            }    
        }
    }


  }
  return 0;
}
