#ifndef __SPHERICALMIRROR_H__
#define __SPHERICALMIRROR_H__

#include "rich/SingleSphericalMirror.h"

// ------------------------------------------------------------------------
/// description of the spherical mirror
/**

*/
class SphericalMirror {

protected:

double r,alphah,alphah0,alphav,alphav0;
double radiusprotonhole,radiuselectronhole;
int nummir;
int speed;
int loss;

SingleSphericalMirror **usm,**lsm,*upper,*lower;
Table  refl;
double dx,dy;


inline int index(int i,int j) { return i*10+j;}

/// constructor (only for special purposes) 
  inline SphericalMirror() {}

  void SingleMirrorNeighbours();
  
public:
/// returns the radius of the hole around the proton beam shroud
  inline double  RadiusProtonHole()   const { return radiusprotonhole; }
/// returns the radius of the hole around the electron beam shroud
  inline double  RadiusElectronHole() const { return radiuselectronhole; }
/// return the width of the mirror (dimension in x-direction)
  inline double  xdim() const { return dx; }
/// return the height of the mirror (dimension in y-direction)
  inline double  ydim() const { return dy; }
/// return the reflectivity curve (at the moment common for a single mirrors)
  inline const Table&  reflectivity() const { return refl; }
  inline       Table&  reflectivity()       { return refl; }
  inline double        reflectivity(double l) { return refl.f(l); }
/// returns the number of rows
  inline int numrow() const {  return 8; }
/// returns the number of columns
  inline int numcol() const {  return 10; }
// returns the number of mirrors
  inline int nmirror() const { return nummir; }
/// returns the SingleSphericalMirror at position (row,column)
  inline SingleSphericalMirror* SingleMirror(int row,int col) {
    SingleSphericalMirror *sm=0;
    if (col>=0&&col<numcol()&&row>=0&&row<numrow()) { 
      if (row<numrow()/2) sm=lsm[index(3-row,col)];
      else sm=usm[index(row,col)-nummir/2];
      }
    return sm;
    }
  SingleSphericalMirror* GetSingleMirror(const Vec3& x);
//@Man: reflection
//@{
/// reflexion of a photon. Return 1, if there are no losses.
/**
  Reflexion of a photon. If no losses happen the photon
  position is changed to the intersection point on the mirror
  and the flight direction is turned to the reflected one.
  In this case the function returns 1. If losses occure the
  function returns 0 and one can analyse the reason using
  the functions ReflexionLoss and GeometricalLoss.
  */
  int reflect(Photon* pp);
/// fast reflexion  
  int fullmirror_intersection(const StraightLine& p,double *path);
  int fullmirror_reflectonly(Photon *p);
  int fullmirror_reflect(Photon *p);
/// return upper mirror  
  inline SingleSphericalMirror& UpperMirror() { return *upper; }
/// return lower mirror  
  inline SingleSphericalMirror& LowerMirror() { return *lower; }
/// reflexion especially for tracing the track direction to the detector  
  inline int upper_reflect(StraightLine *t) { return upper->reflect(t); }
/// reflexion especially for tracing the track direction to the detector
  inline int lower_reflect(StraightLine *t) { return lower->reflect(t); }
/// raytrace with only geometrical losses
  int geometry_trace(Photon *pp) ;
/// switch to full simulation in function reflect (default)
  inline void FullSimulation() { speed=0; }
/// switch to fast simulation in function reflect : no subdivision of the mirrors
  inline void FastSimulation() { speed=1; }
//@}      

/// provide a kumac-file, which can be used to plot the mirror in paw.
  void xyview();

/// print the contents of the class to an ASCII file.
  void print(FILE *fp);
/// constructor reading the information from an ASCII-FILE. 
  SphericalMirror(ScriptFile& f);
/// constructor reading the information from thew CSMI ARTE-table. (not ready)
  SphericalMirror(ArtePointer<CSMI> a);
/// destructor
  ~SphericalMirror();
  };

#endif
