//-----------------------------------------------------------------------//
//
//  MC ringsearch efficiency
//  Author: Denis Dujmic
//  Created: '99
//
//  Modifications:
//    29 May (DD) code polishing
//
//-----------------------------------------------------------------------//
#include "rise/RingSearchEfficiency.h"
#include "arte/arte.hh"
#include "rich/PhiLambda.h"
#include "rich/PhotonDetection.h"
#include "arte/RSEG_cmp.hh"

#ifdef RISE_ROOT

RingSearchEfficiency *RICH_Efficiency=0;


RingSearchEfficiency::RingSearchEfficiency() {
  char *fname=getenv("ROOTFILE");
  if (!fname) fprintf(stderr,"RingSearchEfficiency: [Error]: Variable ROOTFILE not defined!!! Exiting... \n");
  char rname[256];
  rname[0]=0;
  sprintf(rname,"%srun%d_rise_efficiency.root",fname,ArtePointer<EVHD>(1)->run);
  ring_file = new TFile(rname,"RECREATE","Ringsearch efficiency File");   
  events=new TH1F("events","",2,-.5,1.5);

  ring_tree=new TTree("ring_tree","Ringsearch efficiency tree");

  // -- stand-alone
  ring_tree->Branch("ring",&ring,"tx:ty:rad:txf:tyf:radf:x:y:z:Tx:Ty:p:pho:phof:bg:f:type:moth:multi:x0:y0:z0:Rb:S:good/I:assign:fit");
  dphi=new TH1F("dphi","real - corrected photon phi", 100, -.0025,.0025);
  dlam=new TH1F("dlam","real - corrected photon lam", 100, -.0025,.0025);
  raw_dphi=new TH1F("raw_dphi","real - uncorrected photon phi", 100, -.0025,.0025);
  raw_dlam=new TH1F("raw_dlam","real - uncorrected photon lam", 100, -.0025,.0025);
  pho_assign=new TH1F("pho_assign","Multiplicity for photon assignment", 10, -.5, 9.5);
  pho_assign_sa=new TH1F("pho_assign_sa","Multiplicity for photon assignment in stand alone search", 10, -.5, 9.5);
  pho_assign_seed=new TH1F("pho_assign_seed","Multiplicity for photon assignment in seeded search", 10, -.5, 9.5);

  // -- id
  id_tree=new TTree("id_tree","Ringsearch identification tree");
  id_tree->Branch("id",&ringid,"tx/F:ty:rad:p:pf:lre:lrmu:lrpi:lrk:lrp:sn:Rb:S:type/I:moth:hsi:hin:hou:hri");
}

RingSearchEfficiency::~RingSearchEfficiency() {
  cout<<" Writing Ringsearch efficiency histos ... ";
  ring_file->Write();
  cout<<"done"<<endl;
}


void RingSearchEfficiency::fillNtuple() {

  static int errcnt=0;
  if (!ArteTable<MTRA>::size()) {
    if (errcnt<10) { 
      cerr<<"Class RingSearchEfficiency can be used in MC only"<<endl;
      errcnt++;
    }
    return;
  }

  events->Fill(0);

  RICHrings *rich_MC = new RICHrings; 
  RICHreco  *rich_RC = new RICHreco; 


  for (rich_MC->begin(); rich_MC->valid(); rich_MC->next()) {
    // -- get MC truth
    ring.mc_tx  =rich_MC->tx();
    ring.mc_ty  =rich_MC->ty();
    ring.mc_rad =rich_MC->radius();
    ring.mc_p   =rich_MC->momentum();
    ring.mc_pho =rich_MC->DetectedPhotons();
    ring.mc_x[0]=rich_MC->x().x();
    ring.mc_x[1]=rich_MC->x().y();
    ring.mc_x[2]=rich_MC->x().z();
    ring.mc_type=(float)rich_MC->topp()->lund;
    ring.mc_moth=(float)rich_MC->mother()->lund;
    ring.mc_Tx  =rich_MC->T().x();
    ring.mc_Ty  =rich_MC->T().y();

    ArtePointer<MVER> mver=rich_MC->mtra()->bver;
    ring.mc_x0  = mver->x;
    ring.mc_y0  = mver->y;
    ring.mc_z0  = mver->z;

    // -- phi-lambda test
    /*if (rich_MC->DetectedPhotons()>10 && false) {
      //cout<<"detected photons="<<rich_MC->DetectedPhotons()<<endl;
      RICH_LambdaPhi->Correct()->newRing(rich_MC->phi(), rich_MC->lambda(), rich_MC->radius(), 
					 rich_MC->T().x()*.01, rich_MC->T().y()*.01);
      for (rich_MC->firstPhoton(rich_MC->mtra().index()); rich_MC->validPhoton(); rich_MC->nextPhoton()) {
	if (!rich_MC->GetPhotons()->detected()) continue;
	if (ring.mc_z0>10) continue;
	ArtePointer<HITB> hitb=rich_MC->GetPhotons()->hitb();
	ArtePointer<MIMP> mimp=rich_MC->GetPhotons()->mimp();
	int id=rich_MC->GetPhotons()->id();
	if (!RICH_Detector->ChannelExist(id)) continue;
	if (RICH_Detector->pm().numchannel()!=16) continue;
	float ddphi=RICH_LambdaPhi->Correct()->dPhi   ( hitb->q, hitb->r, id/6000000);
	float phi = ddphi + rich_MC->phi();
	float ddlam=RICH_LambdaPhi->Correct()->dLambda( hitb->q, hitb->r, id/6000000); 
	float lam = ddlam + rich_MC->lambda();

	float mctx =mimp->tx/mimp->p;
	float mcty =mimp->ty/mimp->p;
	float mcphi=RICH_LambdaPhi->phi   (mctx,mcty);
	float mclam=RICH_LambdaPhi->lambda(mctx,mcty);
	dphi->Fill(mcphi - phi);
	dlam->Fill(mclam - lam);
	raw_dphi->Fill(mcphi-hitb->q);
	raw_dlam->Fill(mclam-hitb->r);
      }
    }*/

    // -- reco
    ring.rc_tx=ring.rc_ty=ring.rc_rad=ring.rc_pho=ring.multi=ring.Rb=ring.S=0;
    ring.good=ring.assigned=0;
    if (rich_RC->find(rich_MC->mtra().index())) {
      for (;rich_RC->valid();rich_RC->next()) {
	if (rich_MC->mtra()!=rich_RC->mtra()) {
	  fprintf(stderr,"RingSearchEfficiency: [Error] rc MTRA != mc MTRA,  skipping ring...\n");
	  continue;
	}
	ring.multi++;
	ArtePointer<RSEG> rseg=rich_RC->rseg();
	ring.rc_tx =rich_RC->tx();
	ring.rc_ty =rich_RC->ty();
	ring.rc_rad=rich_RC->radius();
	ring.rc_pho=rich_RC->photons();
	ring.rc_background=rich_RC->background();
	ring.rc_f=rich_RC->f();
	ring.Rb=rseg->txe;
	ring.S=rseg->tye;
	ring.fit=rseg->fit;
	// -- find assigned photons
	unsigned int iMtra=rich_MC->mtra().index();
	for (ArteRelationIterator<HITB> ihitb=rseg->hitb.begin(); 
	     ihitb!=rseg->hitb.end(); ++ihitb) {
	  ring.assigned++;
	  for (ArteRelationIterator<MIMP> imimp=(*ihitb)->mimp.begin();
	       imimp!=(*ihitb)->mimp.end(); ++imimp) {
	    if ((*imimp)->mtra.index()==iMtra) { ring.good++; break; }
	  }
	}

	ring_tree->Fill();
      }
    }
    else ring_tree->Fill();
  }

  delete rich_MC;
  delete rich_RC;

  MultiPhoton();
}


void  RingSearchEfficiency::MultiPhoton() {
  // -- avoid double entries per event
  static int old_evt=0;
  if (old_evt==ArtePointer<EVHD>(1)->evt) return;

  for (ArteTable<HITB>::iterator ihitb=RichRun::gede->hit1;
       ihitb!=RichRun::gede->hitl; ++ihitb) {
    int n=0, n_seed=0, n_sa=0;
    for (ArteRelationIterator<RSEG> irseg=(*ihitb)->rseg.begin();
	 irseg!=(*ihitb)->rseg.end(); ++irseg) {
      switch ((*irseg)->fit) {
      case  8:
      case 10:
	n_sa++;
	break;
      case 12:
	n_seed++;
	break;
      }
      n++;
    }
    pho_assign->Fill(n);
    pho_assign_seed->Fill(n_seed);
    pho_assign_sa->Fill(n_sa);
  }
}


void  RingSearchEfficiency::fillNtupleID() {
  static int errcnt=0;
  if (!ArteTable<MTRA>::size()) {
    if (errcnt<10) { 
      cerr<<"Class RingSearchEfficiency can be used in MC only"<<endl;
      errcnt++;
    }
    return;
  }

  for (ArteTable<RTRA>::iterator irtra=ArteTable<RTRA>::begin();
       irtra!=ArteTable<RTRA>::end(); ++irtra) {
    if ((*irtra)->mtra.begin()==(*irtra)->mtra.end()) continue;

    memset(&ringid,0,sizeof(ringid));
    ArtePointer<MTRA> mtra=(*(*irtra)->mtra.begin());

    ArteRelationIterator<RSEG> irseg=(*irtra)->rseg.begin();
    for (;irseg!=(*irtra)->rseg.end(); ++irseg) {
      if ((*irseg)->cmp==Rsegc::rich && (*irseg)->fit==12)  break; 
    }
    ringid.mc_p  =sqrt(mtra->px*mtra->px + mtra->py*mtra->py + mtra->pz*mtra->pz);
    ringid.type  =mtra->topp->lund;
    ringid.mother=mtra->btra->topp->lund;
    ringid.hsi   =(*irtra)->hsi;
    ringid.hin   =(*irtra)->hin;
    ringid.hou   =(*irtra)->hou;
    ringid.hri   =(*irtra)->hri;
    ringid.lre   =(*irtra)->lre;
    ringid.lrmu  =(*irtra)->lrmu;
    ringid.lrpi  =(*irtra)->lrpi;
    ringid.lrk   =(*irtra)->lrk;
    ringid.lrp   =(*irtra)->lrp;
    ringid.rc_p  =(*irtra)->pf;

    if (irseg==(*irtra)->rseg.end()) { 
      id_tree->Fill(); 
      continue; 
    }

    ringid.rc_tx =(*irseg)->txf;
    ringid.rc_ty =(*irseg)->tyf;
    ringid.rc_rad=(*irseg)->time;
    ringid.Rb    =(*irseg)->txe;
    ringid.S     =(*irseg)->tye;
    ringid.sn    =((*irseg)->ye>0) ? (*irseg)->ze/sqrt((*irseg)->ye) : 100;
    id_tree->Fill();
  }

  MultiPhoton();
}

#endif
