//--*-C++-*--
//-----------------------------------------------------------------------//
//
//  Attributes for stand-alone ring reconstruction
//  Author: Denis Dujmic
//  Created: May-00
//
//  Modifications:
//    May 29, 2000 (DD): RingSearchAttrib::finer_search_step added 
//
//-----------------------------------------------------------------------//
#ifndef RINGSEARCHATTRIB_H
#define RINGSEARCHATTRIB_H

#include "arte/arte.hh"

/////////////////////  fit description  ////////////////////////////////

struct RingSearchType {
  enum desc {
    standalone   =   0x1,   // stand alone ring search
    rseg_seed    =   0x2,   // RSEG track based
    rtra_seed    =   0x4,   // RTRA track based
    nobg         =   0x8,   // old fitter (no background)
    lkbg         =   0x10,  // new fitter (background treated)
    seedrefit    =   0x20,  // use seed par in refit rather than ring search par
    bookrseg     =   0x40,  // book rseg
    bookhitb     =   0x80,  // book hitb
    fillrtra     =   0x100  // fill likelihoods and hits assigned to rich into rtra
  };
};


class RingSearchTrack {
public:
  static bool    exist;              // stand alone/track based
  static float   bx, by;             // impact parameters in meters !
  static float   phi, lambda;        // track direction in RICH!
  static float   ip;                 // inverse momentum*charge
  static ArtePointer<RTRA> rtra;     // RTRA seed row
  static ArtePointer<RSEG> rseg;     // RSEG seed row
  static float   max_phi_residual;   // seed residual
  static float   max_lambda_residual;// seed residual
  static float   max_rad_residual;   // seed residual
  static float   aa;                 // dp/p
  static float   bb;                 // dp/p
  static float   sigma2_rr;          // rich resolution
  static float   min_rpnt_z;         // min for rpnt
  static float   max_rpnt_z;         // max for rpnt
};


class RingSearchAttrib {
public:
  static int    type;         // fit type 
  static bool   reuse;        // reuse already found photons
  static bool   bookrseg;     // book rseg
  static bool   bookhitb;     // book rseg-hitb
  static bool   fillrtra;     // fill likelihoods and hits assigned to rich into rtra
  static float  SNcut;        // minimum S/Sqrt(N)
  static float  Scut;         // minimum significance (lkbg only)
  static int    peakCut;      // minimum number of counts in peak-bin in ringsearch
  static float  minSearchRadius;    // minimum search radius
  static float  maxSearchRadius;    // maximum search radius
  static float  searchStep;         // search step
  static int    peakThreshold;      // minimum number of counts in search peak
  static int    occupancyCut;       // event occupancy cut
  static int    seedOccupancyCut;   // event occupancy cut for seeded search
  static bool   renorm;            // renormalize likelihood
  static bool   id;                // make particle id
  static bool   use_simple_R2P2;   // use distance from lines in sigma^2
  static bool   use_seed_phi;      // use tx from seed track
  static bool   use_seed_lambda;   // use ty from seed track
  static int    choose_best_ring;  // make final selection if more than one ring/track is found
  static int    seed_type;         // choose seed type
  static bool   use_underdogs;     // fill RSEG with rings of low quality (but REUSE their photons in next search)
  static bool   finer_search_step; // +/- sigma radius in seeded search
  static int    rlevel;            // reconstruction scenario
  static int    minHsi;            // minimum hits in silicon for rtra ringsearch
  static int    minHou;            // minimum hits in inner tracker for rtra ringsearch
  static int    minHin;            // minimum hits in outer tracker for rtra ringsearch
  static int    minHio;            // minimum hits in inner+outer tracker for rtra ringsearch
  static int    minHca;            // minimum hits in calorimeter for rtra ringsearch
  static int    minHmu;            // minimum hits in muon for rtra ringsearch
  static int    overwriteOccupancyCut ;
  static float  overwriteMinSearchRadius;
  static float  overwriteMaxSearchRadius;
  static float  overwriteSearchStep;
  static float  overwritePeakThreshold;
  static int    fitLevel;
};

#ifdef USE_ARTE
extern "C" {
  /// Set maximum residuals in seed search
  void risesetres_();
  /// Set momentum resolution parameters
  void risesetdpp_();
  /// Set ARTE table booking flags
  void risebook_();
  /// Set particle ID output
  void riselike_();
  /// Set search parameters
  void risesearch_();
  /// Set minimum hits to trigger rtra search
  void risehset_();
  /// Set search parameters
  void risesearch_();
}
#endif
#endif
