#ifndef _riter_geometry_h_
#define _riter_geometry_h_

#include <arte/arte.hh>
#include <riter/RITER.hh>
#include <riter/RITERSupermodule.hh>
#include <riter/RITERPlanarmirror.hh>
#include <riter/RITERSphericalmirror.hh>
#include <riter/RITERBeamshroud.hh>

#define NUMROW 24
#define NUMCOL 12
#define NUMSM  32
#define NUMCH  16

#define NCHAN 86016   
#define NWCHA (NCHAN-1)/32+1 

/// Rough acceptance test for tracks
int IsInside(float tx,float ty);

/// Description of the RICH RITER geometry
class RITERGeometry : 
  public  RITERSphericalmirror, 
  public  RITERPlanarmirror,
  public  RITERSupermodule,  
  public  RITERBeamshroud  
{

private:
  ///  type=0-m16 =1-m4 sigma^2 =sig0^2+sigthc^2+sig_sfa^2 
  float sigma2[2];   
public:
  /** error : ... , spherical abberation
  <li> geometrical : $sigma^2 =sig0^2+sigthc^2+sig_sfa^2$  :
  <li> sig0 =nn*$sig_pad$; nn={4-m1,2-m4, 1-m16}
  */
  float sigthc;
  ///  sigma due to spherical abberation 
  float sig_sfa;
  ///  sigma due to a finite size of the pad
  float sig_pad;   
  /// f_pad=f/d_pad 
  float f_pad;   
  /// N_0
  float an_zero;
  /// refractive index -average
  float ref_ind;  

  /// acceptance limits
  float  th_0,fi_0; 
  /// lens system acceptance
  float acc_lens;
  /// start and and of the rich
  float z_rich_min, z_rich_max; 
  /// square of the maximum and min  Ch.radius 
  float rmax2, rmin2;    
  float x_cut, y_cut, z_cut, z_magnet,p_min;
  /// flag whether corrections are applied
  int is_correction; 

  /// transformation from discrite values to virtual position
  void   to_xyz(int ism,int ir,int ic,int nn,RITERVec &r,float &sig);  
  int src2det(int ism, int ir, int ic , int pmch , int type, 
	      int &sm, int &x0, int &y0  );

  /// print content of the class
  int   Print();
  /// return pmt+optical part of the error for 0-m16 and 1-m4 pmt s 
  float Sigma2(int type){ return sigma2[type]; };
  /// reads KUIP commands
  int   Kuip();
  /// reads corrections if they exist
  int   ReadCorrections();
  /// Correction due to missalignment of the spherical and planar mirrors
  float Correct_thc(int id_sfm,int id_plm,float th_cer,float fi_cer);
  /// returns whether corrections are applied
  int IsCorrections(){return is_correction; };
  /// constructor
  RITERGeometry();
  /// destructor
  ~RITERGeometry(){};
  
};

/// global pointer to RITER geometry;
extern RITERGeometry *RITER_Geometry;

#endif




