#ifndef _riter_iter_h_
#define _riter_iter_h_    

#include <riter/RITERTracks.hh>
#include <riter/RITERHits.hh>

inline void vzero(float  *x,int y) { for (int vz=0;vz<(y);vz++) *x++=0; return;} 

/// description of iterational table entry
class RITERTable {
 private:
 public:
  /// row
  int ir;
  /// track index
  int itra;
  /// hit index
  int ifot;
  /// channel index
  int icha;
  /// utez
  float utez;
  /// cherenkov angle
  float thc;
  /// polar angle
  float fic;
  /// weight of the entry
  float w;
  /// flag 0 = outside , 1 - signal window , -1 = bgr window
  int flag;
  /// sigma of the hit: takes into account photon sigma and track sigma calculated from fi_ch 
  float sigma;
  /// plannar mirror which was hit by the photon radiated from the middle of the track
  int iplm;

  /// spherical mirror  which was hit by the photon radiated from the middle of the track
  int isfm;
  /// constructor of the class
  RITERTable(int ir1,int itra1,int ifot1,int icha1,
	     float utez1,float th_cer,float fi_cer,float w1,int isinside, float sig, int plm, int sfm) {  
    ir   = ir1;
    itra = itra1;
    ifot = ifot1;
    icha = icha1;
    utez = utez1;
    thc  = th_cer;
    fic  = fi_cer;
    w    = w1;
    flag = isinside;
    sigma = sig;
    iplm = plm;
    isfm = sfm;
  };
  // destructor of the class
  ~RITERTable(){};
  // empty constructor of the class
  RITERTable(){};
  
  
};



  /// Filling of the RICH table
  int RITERRingTableFill(
			 Rvector<RITERTracks> *RTracks,
			 Rvector<RITERHits>   *RHits ,  
			 Rvector<RITERTable>  *Rrich );

  /// Performs one iteration
  float RITERIterate(
		     Rvector<RITERTracks> *RTracks,
		     Rvector<RITERHits>   *RHits ,  
		     Rvector<RITERTable>  *Rrich );

  /// Filling of the likelihoods
  int RITERLikelihoods(
		       Rvector<RITERTracks> *RTracks,
		       Rvector<RITERHits>   *RHits ,  
		       Rvector<RITERTable>  *Rrich );

#endif












