#ifndef _riter_main_h_
#define _riter_main_h_

#include <riter/RITER.hh>

#include <riter/RITERPmt.hh>
#include <riter/RITERSupermodule.hh>
#include <riter/RITERPlanarmirror.hh>
#include <riter/RITERSphericalmirror.hh>
#include <riter/RITERGeometry.hh>
#include <riter/RITERAngle.hh>
#include <riter/RITERTracks.hh>
#include <riter/RITERHits.hh>
//#include <riter/RITERFaktor.hh>
#include <riter/RITERIter.hh>

#ifdef RITER_DEBUG
#include <myhisto.hh>
#endif

/// main class of the RITER package
class RITERMain {

protected:
  /// vector of tracks
  Rvector<RITERTracks>  tracks;
  /// vector of photons
  Rvector<RITERHits>    fotons;

  /// returns RITER code of the lund particle
  int   itip_lund(int ipart);
  
  // ARTE interface
  /// extrapolates point (x,y,z) and dir s to the RICH entrance
  RITERVec   toRICH( float x, float y, float z, RITERVec s);
  /// extrapolates point r  and dir s to the RICH entrance
  RITERVec   toRICH( RITERVec &r, RITERVec s);
  // *********************** public members 
public:
  /// main routine of the RITER package
  int    main();
  /// adds a photon to the vector of photons
  int AddPhoton(int sm ,int r,int c,int pos, int ch,int type);
  /// adds a track  to the vector of tracks 
  int AddTrack( RITERVec &dir,float err,RITERVec &point, float p);
  /// adds a track  to the vector of tracks 
  int AddTrack(float x, float y, float z, float tx, float ty, float p, 
	       float sx, float sy, float sz, float stx, float sty, float sp);
  
  /// resets track and photon vectors 
  inline int Reset(){tracks.clear(); fotons.clear(); return RITER_OK;}; 	    
  /// constructor
  RITERMain(){ if (!RITER_Geometry) RITER_Geometry= new RITERGeometry();};
  /// destructor
  ~RITERMain(){ };

};
// ---------------------------------------------------------------------

float pois_log(float mean,float n);

#endif 












