#ifndef _RITER_HEP_THREEVECTOR_H
#define _RITER_HEP_THREEVECTOR_H

typedef int RITERBoolean;

#ifndef false
#define false 0
#endif
#ifndef true
#define true 1
#endif



#include <iostream.h>



/// class RITERVec same as He3Vector except that only float arithmetic is used
class RITERVec {

public:

  enum { X=0, Y=1, Z=2, NUM_COORDINATES=3, SIZE=NUM_COORDINATES };
  // Safe indexing of the coordinates when using with matrices, arrays, etc.
  // (BaBar)

  /// The constructor.
  inline RITERVec(float x = 0.0, float y = 0.0, float z = 0.0);

  /// The copy constructor.
  inline RITERVec(const RITERVec &);

  /// The destructor.
  inline ~RITERVec();

  /// Get components by index (Geant4).
  float operator () (int) const;
  /// Get components by index (Geant4).
  inline float operator [] (int) const;


  /// Set components by index.
  float & operator () (int);
  /// Set components by index.
  inline float & operator [] (int);

  /// The x-component in cartesian coordinate system.
  inline float x() const;
  /// The y-component in cartesian coordinate system.
  inline float y() const;
  /// The z-component in cartesian coordinate system.
  inline float z() const;

  /// Set the x component in cartesian coordinate system.
  inline void setX(float);
  /// Set the y component in cartesian coordinate system.
  inline void setY(float);
  /// Set the z component in cartesian coordinate system.
  inline void setZ(float);

  /// The azimuth angle.
  inline float phi() const;

  /// The polar angle.
  inline float theta() const;


  /// Cosine of the polar angle.
  inline float cosTheta() const;

  /// The magnitude squared (rho^2 in spherical coordinate system).
  inline float mag2() const;


  /// The magnitude (rho in spherical coordinate system).
  inline float mag() const;


  /// Set phi keeping mag and theta constant (BaBar).
  inline void setPhi(float);

  /// Set theta keeping mag and phi constant (BaBar).
  inline void setTheta(float);

  /// Set magnitude keeping theta and phi constant (BaBar).
  inline void setMag(float);

  /// The transverse component squared (R^2 in cylindrical coordinate system).
  inline float perp2() const;

  /// The transverse component (R in cylindrical coordinate system).
  inline float perp() const;


  /// Set the transverse component keeping phi and z constant.
  inline void setPerp(float);


  /// The transverse component w.r.t. given axis squared.
  inline float perp2(const RITERVec &) const;

  /// The transverse component w.r.t. given axis.
  inline float perp(const RITERVec &) const;


  /// Assignment.
  inline RITERVec & operator = (const RITERVec &);

  /// Comparisons (Geant4). 
  inline RITERBoolean operator == (const RITERVec &) const;
  /// Comparisons (Geant4). 
  inline RITERBoolean operator != (const RITERVec &) const;

  /// Addition.
  inline RITERVec & operator += (const RITERVec &);


  /// Subtraction.
  inline RITERVec & operator -= (const RITERVec &);


  /// Unary minus.
  inline RITERVec operator - () const;


  /// Scaling with real numbers.
  inline RITERVec & operator *= (float);


  /// Unit vector parallel to this.
  inline RITERVec unit() const;


  /// Vector orthogonal to this (Geant4).
  inline RITERVec orthogonal() const;


  /// Scalar product.
  inline float dot(const RITERVec &) const;

  /// Cross product.
  inline RITERVec cross(const RITERVec &) const;


  /// The angle w.r.t. another 3-vector.
  inline float angle(const RITERVec &) const;


  /// Returns the pseudo-rapidity, i.e. -ln(tan(theta/2))
  float pseudoRapidity() const;


  /// Rotates the RITERVec around the x-axis.
  void rotateX(float);


  /// Rotates the RITERVec around the y-axis.
  void rotateY(float);


  /// Rotates the RITERVec around the z-axis.
  void rotateZ(float);


  /// Rotates reference frame from Uz to newUz (unit vector) (Geant4).
  void rotateUz(const RITERVec&);

  /// Rotates around the axis specified by another RITERVec.
  void rotate(float, const RITERVec &);



private:

  /// The components.
  float dx, dy, dz;

};

std::ostream & operator << (std::ostream &out , const RITERVec & x);

std::istream & operator >> (std::istream &, RITERVec &);
// Input from a stream.

#include "RITERThreeVector.icc"

#endif /* _RITER_RITER_HEP_THREEVECTOR_H */



