#ifndef _riter_utils_h_
#define _riter_utils_h_
#include <riter/RITER.hh>

#ifndef NO_ARTE
#include <arte/arte.hh>
#endif

/**@name Utility functions
      * RITER Utility functions
      */
//@{
/** Calculate the second point or the distance between two points
 of line . given by:
 * <li> rinit   initial point (input),
 * <li> dir     unit vector of line direction (input),
 * <li> rfinl   final point (input/output),
 * <li> path    distance between initial and final points (input/output),
 depending on parameter
 * <li> izb =0  path given,
 * <li> =1  rfinl.x() given
 * <li> =2  rfinl.y() given
 * <li> =3  rfinl.y() given      
 */
void premica(RITERVec &rinit,RITERVec &dir,RITERVec &rfinl,float  &path,int izb);

/**  transformation of a vector from fixed system S to current system S' .
 <li> such that x' is in plane (x,z) -  horizontal in HERA-B
 <li> SMER_Z() is unit vector of z-axis of S' given in fixed system S
 <li> VIN()  are the coordinates of a vector in S
 <li> VOUT() are the coordinates of a vector in S'     
 */
RITERVec transform_from_fixed( RITERVec  &zdir, RITERVec  &vin );
/** transformation of a vector from current system S' to fixed system S.
<li>  SMER_Z() is unit vector of z-axis of S' given in fixed system S
<li>  VIN()  are the coordinates of a vector in S'
<li>  VOUT() are the coordinates of a vector in S   
*/
RITERVec RotateToFixed( RITERVec &smer_z, RITERVec &vin);

/** crossection of a line r,dir and sphere of radius radij and center rc.
<li>  r1 is concave crossection if return value is OK, 
<li> path length to the intersection   
 */
int SphereIntersection( RITERVec  &r,  RITERVec  &dir, float radij, 
			RITERVec  rc,  RITERVec  &r1 , float &path);

/** cross section of a line R(),DIR() with a tube of radius RADIJ and
 axis along z at RC.
<li>  R2() is convex crossection if return value is OK
 */
int ShroudIntersection( RITERVec  &r,  RITERVec  &dir, float radij, 
			RITERVec  rc, RITERVec  &r2 , float &path);
/**
cross section of a line r,dir with a cylinder of radius radij and
 axis along z.
<li>  r1 is concave crossection if  return value is OK      
<li>  r2 is convex crossection if  return value is OK    
*/
int TubeIntersection(   RITERVec  &r,  RITERVec  &dir, float radij, 
			RITERVec  &r1, RITERVec  &r2 );

/** geometrical reflection on a sphere of radius radij and with center at rc().
<li> r   on input:  point where a ray hits the sphere
<li> r   on output: focus, i.e. a point where reflected ray crosses the optical
		 axis (opt. axis is paralel to impact ray)
<li> dir  on input: ray's direction (unit vector) before reflection
<li> dir   on output: ray's direction (unit vector) after reflection
*/
void SphereReflection(  RITERVec &r,   RITERVec &dir,  float radij,
			RITERVec rc);

#ifndef NO_ARTE
float   Momentum(ArtePointer<RSEG> rseg);
ArtePointer<RCCL> Find_ecal(ArtePointer<RSEG>rseg);
#endif

/// returns rough estimation of the momentum based on assumption of track origin (0,0,0)
float   Momentum(float xf,float zf,float txf,float tyf);
#endif
 //@}











