// *****************************************************************************
//
// source:    L3_Timer.C
//
// type:      source code
//
// created:   06 Dec 1999
//
// author:    Stefan Scharein, Ullrich Schwanke
//            HERA-B Collaboration
//            DESY Zeuthen
//            Platanenallee 6
//            D-15738 Zeuthen
//            Germany
//            email: scharein@ifh.de, schwanke@ifh.de
//
// symbol:    
//
// changes:
//
// *****************************************************************************

#include "L3_Timer.hh"
#include <iostream>

L3_Timer::L3_Timer() : Point_Clock("Point"), Track_Clock("Track"),
  Trigger_Clock("Trigger"),Vertex_Clock("Vertex") {
}

L3_Timer::L3_Timer(const L3_Timer& a) : Point_Clock( a.Point_Clock ),
    Track_Clock( a.Track_Clock ), Trigger_Clock( a.Trigger_Clock ), Vertex_Clock( a.Vertex_Clock )  {
}

L3_Timer& L3_Timer::operator = (const L3_Timer& a) {
  Point_Clock = a.Point_Clock;
  Track_Clock = a.Track_Clock;
  Trigger_Clock = a.Trigger_Clock;
  Vertex_Clock = a.Vertex_Clock;
  return *this;
}

void L3_Timer::reset() {
  Point_Clock.reset();
  Track_Clock.reset();
  Trigger_Clock.reset();
  Vertex_Clock.reset();
}

void L3_Timer::start_PointReco() {
  Point_Clock.start();
}
void L3_Timer::stop_PointReco() {
  Point_Clock.stop();
}
void L3_Timer::start_TrackReco() {
  Track_Clock.start();
}
void L3_Timer::stop_TrackReco() {
  Track_Clock.stop();
}
void L3_Timer::start_Vertex() {
  Vertex_Clock.start();
}
void L3_Timer::stop_Vertex() {
  Vertex_Clock.stop();
}
void L3_Timer::start_Trigger() {
  Trigger_Clock.start();
}
void L3_Timer::stop_Trigger() {
  Trigger_Clock.stop();
}
void L3_Timer::print_PointTime() const {
  Point_Clock.print();
}
void L3_Timer::print_TrackTime() const {
  Track_Clock.print();
}
void L3_Timer::print_VertexTime() const {
  Vertex_Clock.print();
}
void L3_Timer::print_TriggerTime() const {
  Trigger_Clock.print();
}
float L3_Timer::lastPointTime() const {
  return Point_Clock.lastTime();
}
float L3_Timer::lastTrackTime() const {
  return Track_Clock.lastTime();
}
float L3_Timer::lastVertexTime() const {
  return Vertex_Clock.lastTime();
}
float L3_Timer::lastTriggerTime() const {
  return Trigger_Clock.lastTime();
}
void L3_Timer::print() const {
  cout << "----------L3_Timer-------------------------------------------------" << endl; 
  print_PointTime();
  print_TrackTime();
  print_VertexTime();
  print_TriggerTime();
  cout << "-------------------------------------------------------------------" << endl; 
}
