// *****************************************************************************
//
// source:    L3_DTLT.hh
//
// type:      include file
//
// created:   06 Dec 1999
//
// authors:   Stefan Scharein, Ullrich Schwanke
//            HERA-B Collaboration
//            DESY Zeuthen
//            Platanenallee 6
//            D-15738 Zeuthen
//            Germany
//            email: scharein@ifh.de, schwanke@ifh.de
//
// changes:
//
// *****************************************************************************
#ifndef _L3_DTLT_HH_
#define _L3_DTLT_HH_
#include "L3_Defines.hh"
#include "L3_Impact.hh"
#include "L3_Track.hh"
#include "L3_Vertex.hh"

class L3_Event;
class L3_State;

class L3_TrackData {
public:
  L3_TrackData() : flag(0) {}
  L3_TrackData(const L3_Track& t,const L3_Impact& i) : track(t),impact(i),flag(0) {}
  L3_Track track;
  L3_Impact impact;
  int flag;
  L3_TrackData& operator=(const L3_TrackData& a){
    track = a.track;
    impact = a.impact;
    flag = a.flag;
    return *this;
  }  
};

class L3_VertexData {
public:
  L3_VertexData() : flag(0) {}
  L3_Vertex vertex;
  L3_Impact lepImpact;
  L3_Impact trkImpact;
  int flag;
  float downstream;
  L3_Impact dca;
  L3_VertexData& operator=(const L3_VertexData& a){
    vertex = a.vertex;
    lepImpact = a.lepImpact;
    trkImpact = a.trkImpact;
    flag = a.flag;
    downstream = a.downstream;
    dca = a.dca;
    return *this;
  }  
};

class L3_DTLT {
public:
  L3_DTLT();
  L3_DTLT(const L3_DTLT&);
  L3_DTLT& operator = (const L3_DTLT&);
  void fill(const L3_Event&, const L3_State&);
  void reset();
  vector<L3_TrackData> impactParam() const;
  vector<L3_VertexData> vertices() const;
  void add(L3_TrackData& d);
  void add(L3_VertexData& d);
private:
  vector<L3_TrackData> impactP;
  vector<L3_VertexData> vertexP;
};

#endif
