*CMZ :          07/11/96  12.55.17  by  T. Oest
*-- Author :    T. Oest   27/09/96

      SUBROUTINE V0FIT(TRK1, TRK2, PAR, VPAR, WPAR, CHIV0, CHT1,
     +                 PAR2, VPAR2, WPAR2, CHI2V0, CHT2, IT1, IT2, IRET)

************************************************************************
*
* Description: main program for the v0 fit
*              the region for the inhomogeneous field is set in V0FEXT (DZMAG)
*
* Called from:
*
* External references:
*
* Argument in/out description
* TRK1     in     first track
* TRK2     in     second track
* PAR      out    vertex position, track parameter
*                 (z,x,y,tx1,ty1,1/p1,tx2,ty2,1/p2)
* VPAR     out    covariance matrix
* WPAR     out    (VPAR)**-1
* CHIV0    out    chi2 from fit
* CHT1     out    total chi2 (if two solutions)
* IRET     out    >=0 ok
*                  =1 two solutions
*
************************************************************************

      IMPLICIT NONE

#include "arte/Arte.fi"
#include "auto/EV.fi"
#include "auto/CON.fi"

      INTEGER          TRK1, TRK2, IRET, IT1, IT2
      DOUBLE PRECISION PAR(9) , VPAR(9,9) , WPAR(9,9) , CHIV0,  CHT1
      DOUBLE PRECISION PAR2(9), VPAR2(9,9), WPAR2(9,9), CHI2V0, CHT2
      DOUBLE PRECISION DTX1, DTX2, DTY

      INTEGER          I, J

      INTEGER          IERR, ND1, ND2
      DOUBLE PRECISION ZST1, ZST2, SZST1, SZST2, CH1, CH2, XHELP

      INTEGER          IERR2
      DOUBLE PRECISION CHICUT, ZVMIN, ZVMAX

#include "arte/Arte.ff"
#include "auto/EV.ff"
#include "auto/CON.ff"

      DATA   CHICUT /100.D0/, ZVMIN /-100.D0/, ZVMAX /1000.D0/
C-----------------------------------------------------------------
      IRET  = 0
C-----------------------------------------------------------------
C  START VALUES
C-----------------------------------------------------------------
      CALL V0STRT (TRK1, TRK2, ZST1, SZST1, CH1,
     +             ZST2, SZST2, CH2, IERR)

      IF (IERR.LT.0) THEN
       IRET = -1
       GOTO 9000
      ENDIF

c      PRINT*,' START 1',SNGL(ZST1),SNGL(SZST1),SNGL(CH1)
c      PRINT*,' START 2',SNGL(ZST2),SNGL(SZST2),SNGL(CH2)
C-----------------------------------------------------------------
C  apply cuts on the start values
C-----------------------------------------------------------------
      IF (CH1.GT.CHICUT) SZST1 = -1.0
      IF (CH2.GT.CHICUT) SZST2 = -1.0

      IF (ZST1-5.*SZST1.GT.FRTRA_ZF(TRK1) .OR.
     +    ZST1-5.*SZST1.GT.FRTRA_ZF(TRK2)) SZST1 = -1.D0
      IF (ZST2-5.*SZST2.GT.FRTRA_ZF(TRK1) .OR.
     +    ZST2-5.*SZST2.GT.FRTRA_ZF(TRK2)) SZST2 = -1.D0

      IF (ZST1+SZST1.LT.ZVMIN) SZST1 = -1.D0
      IF (ZST2+SZST2.LT.ZVMIN) SZST2 = -1.D0

      IF (ZST1.GT.ZVMAX) SZST1 = -1.D0
      IF (ZST2.GT.ZVMAX) SZST2 = -1.D0

      IF (SZST1.LT.0.0 .AND. SZST2.LT.0.0) THEN
       IRET = -1
       GOTO 9000
      ENDIF

c      PRINT*,' START 1',SNGL(ZST1),SNGL(SZST1),SNGL(CH1)
c      PRINT*,' START 2',SNGL(ZST2),SNGL(SZST2),SNGL(CH2)
c      PRINT*,' FIT',TRK1,TRK2
C-----------------------------------------------------------------
C vertex fits
C-----------------------------------------------------------------
      IERR  = -1
      IERR2 = -1

      CHIV0  = -1.0
      CHI2V0 = -1.0

      IF (SZST1.GT.0.0) 
     + CALL V0FITM (TRK1,TRK2,ZST1,PAR, VPAR, WPAR, CHIV0,ND1,
     +              IT1, IERR)
      IF (SZST2.GT.0.0) THEN
       IF (IERR.LT.0) THEN
         CALL V0FITM (TRK1,TRK2,ZST2,PAR, VPAR, WPAR, CHIV0,ND1,
     +                IT1, IERR)
       ELSE
         CALL V0FITM (TRK1,TRK2,ZST2,PAR2,VPAR2,WPAR2,CHI2V0,ND2,
     +                IT2, IERR2)
       ENDIF
      ENDIF

      IF (IERR.LT.0 .AND. IERR2.LT.0) THEN
       IRET = -2
       GOTO 9000
      ENDIF

      IF (IERR.EQ.2)  IT1 = -IT1
      IF (IERR2.EQ.2) IT2 = -IT2
C-----------------------------------------------------------------
C same solutions
C-----------------------------------------------------------------
      IF (IERR.GE.0 .AND. IERR2.GE.0) THEN
       CH1 = 0.0
       DO I=1,9
       DO J=1,9
        CH1 = CH1 + (PAR(I)-PAR2(I))*WPAR(I,J)*(PAR(J)-PAR2(J))
       ENDDO
       ENDDO
       IF (CH1.LT.0.0001) IERR2 = -1
      ENDIF


C-----------------------------------------------------------------
C in case of two solutions
C check for chi2
C-----------------------------------------------------------------
      IF (IERR.GE.0 .AND. IERR2.GE.0) IRET = 1


      IF (IERR.GE.0) THEN
       CHT1  = CHIV0  + 2.*ND1

C check if vertex behind first measured poit
       IF (FRTRA_ZF(TRK1).LT.PAR(1)) THEN
        CHT1 = CHT1 + (PAR(1)-FRTRA_ZF(TRK1))**2*WPAR(1,1)
       ENDIF
       IF (FRTRA_ZF(TRK2).LT.PAR(1)) THEN
        CHT1 = CHT1 + (PAR(1)-FRTRA_ZF(TRK2))**2*WPAR(1,1)
       ENDIF
      ENDIF


      IF (IERR2.GE.0) THEN
       CHT2  = CHI2V0 + 2.*ND2
       IF (FRTRA_ZF(TRK1).LT.PAR2(1)) THEN
        CHT2 = CHT2 + (PAR2(1)-FRTRA_ZF(TRK1))**2*WPAR2(1,1)
       ENDIF
       IF (FRTRA_ZF(TRK2).LT.PAR2(1)) THEN
        CHT2 = CHT2 + (PAR2(1)-FRTRA_ZF(TRK2))**2*WPAR2(1,1)
       ENDIF
      ENDIF

       IF ((IERR.GE.0 .AND. IERR2.GE.0 .AND. CHT2.LT.CHT1) .OR.
     +     (IERR.LT.0 .AND. IERR2.GE.0)) THEN
        XHELP  = CHT1
        CHT1   = CHT2
        CHT2   = XHELP
        XHELP  = CHI2V0
        CHI2V0 = CHIV0
        CHIV0  = XHELP

        I   = IT1
        IT1 = IT2
        IT2 = I

        I   = ND1
        ND1 = ND2
        ND2 = I

        I     = IERR
        IERR  = IERR2
        IERR2 = I

        DO I=1,9
         XHELP   = PAR(I)
         PAR(I)  = PAR2(I)
         PAR2(I) = XHELP
         DO J=1,9
          XHELP      = VPAR(I,J)
          VPAR(I,J)  = VPAR2(I,J)
          VPAR2(I,J) = XHELP

          XHELP      = WPAR(I,J)
          WPAR(I,J)  = WPAR2(I,J)
          WPAR2(I,J) = XHELP

         ENDDO
        ENDDO
       ENDIF
C-----------------------------------------------------------------
C  the end
C-----------------------------------------------------------------
c      print*,' 1 sol ',sngl(par(1)),sngl(chiv0),nd1,sngl(cht1),ierr
c      if (ierr2.ge.0) 
c     + print*,' 2 sol ',sngl(par2(1)),sngl(chi2v0),nd2,sngl(cht2),ierr2

 9000 END

************************************************************************
* End of V0FIT
************************************************************************





