*CMZ :          07/11/96  12.55.17  by  T. Oest
*-- Author :    T. Oest   27/09/96

      SUBROUTINE V0TRK(M1, M2, PAR, VPAR, PTRK, VTRK)
************************************************************************
*
* Description: add vertex track and return new track parameters
*
* Called from:
*
* External references:
*
* Argument in/out description
* M1       in     mass of first particle
* M2       in     mass of second particle
* PAR(9)   in     vertex fit parameter (z,x,y,tx1,ty1,r1,tx2,ty2,r2)
* VPAR(9,9)in     covaraince matrix for PAR
* PTRK(7)  out    track parameter (z,x,y,tx,ty,rho,m)
* VTRK(7,7)out    covariance matrix for PTRK
*
************************************************************************

      IMPLICIT NONE

#include "arte/Arte.fi"
#include "auto/EV.fi"
#include "auto/CON.fi"

      DOUBLE PRECISION M1, M2, PAR(9) , VPAR(9,9)
      DOUBLE PRECISION PTRK(7), VTRK(7,7)

      DOUBLE PRECISION A, RW1, RW2, W1, W2, B, D, RHO1, RHO2
      DOUBLE PRECISION DERI(7,9)
      INTEGER          I, J, K, L, K1, K2, L1, L2

#include "arte/Arte.ff"
#include "auto/EV.ff"
#include "auto/CON.ff"

      DATA DERI /63*0.0D0/
C-----------------------------------------------------------------
C  calculate track parameter PTRK
C-----------------------------------------------------------------
      PTRK(1) = PAR(1)
      PTRK(2) = PAR(2)
      PTRK(3) = PAR(3)


      RHO1     = ABS(PAR(6))
      RHO2     = ABS(PAR(9))
      W1       = SQRT(1.+PAR(4)**2+PAR(5)**2)
      W2       = SQRT(1.+PAR(7)**2+PAR(8)**2)
      RW1      = RHO1 * W1
      RW2      = RHO2 * W2
      A        = 1./(RW1+RW2)
      D        = 1.+PAR(4)*PAR(7)+PAR(5)*PAR(8)

      PTRK(4) = (RW2 * PAR(4) + RW1 * PAR(7)) * A
      PTRK(5) = (RW2 * PAR(5) + RW1 * PAR(8)) * A
      PTRK(6) = 1./SQRT(1./RHO1**2 + 1./RHO2**2 + 2.*D/(RW1*RW2))

      PTRK(7) = SQRT(M1**2+M2**2
     +           +2.*SQRT(M1**2+1./RHO1**2)*SQRT(M2**2+1./RHO2**2)
     +           -2.*D/(RW1*RW2))
C-----------------------------------------------------------------
C  derivative
C-----------------------------------------------------------------
      DERI(1,1) = 1.0D0
      DERI(2,2) = 1.0D0
      DERI(3,3) = 1.0D0

C d TX / d TX1,   d TX / d TX2
      DERI(4,4) = A*( RW2 + RHO1*(PAR(7)-PTRK(4))*PAR(4)/W1)
      DERI(4,7) = A*( RW1 + RHO2*(PAR(4)-PTRK(4))*PAR(7)/W2)

C d TX / d TY1,   d TX / d TY2
      DERI(4,5) = A*RHO1*(PAR(7)-PTRK(4))*PAR(5)/W1
      DERI(4,8) = A*RHO2*(PAR(4)-PTRK(4))*PAR(8)/W2

C d TY / d TX1,   d TY / d TX2
      DERI(5,4) = A*RHO1*(PAR(8)-PTRK(5))*PAR(4)/W1
      DERI(5,7) = A*RHO2*(PAR(5)-PTRK(5))*PAR(7)/W2

C d TY / d TY1,   d TY / d TY2
      DERI(5,5) = A*( RW2 + RHO1*(PAR(8)-PTRK(5))*PAR(5)/W1)
      DERI(5,8) = A*( RW1 + RHO2*(PAR(5)-PTRK(5))*PAR(8)/W2)

C d TX / d TR1,   d TX / d TR2
      DERI(4,6) = A*W1*(PAR(7)-PTRK(4))
      DERI(4,9) = A*W2*(PAR(4)-PTRK(4))

C d TY / d TR1,   d TY / d TR2
      DERI(5,6) = A*W1*(PAR(8)-PTRK(5))
      DERI(5,9) = A*W2*(PAR(5)-PTRK(5))

C D rho / d PAR
      B         = -PTRK(5)**3/(RW1*RW2)

C D rho / d TX
      DERI(6,4) = B * (PAR(7)-PAR(4)*D/W1**2)
      DERI(6,7) = B * (PAR(4)-PAR(7)*D/W2**2)

C D rho / d TY
      DERI(6,5) = B * (PAR(8)-PAR(5)*D/W1**2)
      DERI(6,8) = B * (PAR(5)-PAR(8)*D/W2**2)

C D rho / d RHO1,2
      DERI(6,6) = PTRK(6)/RHO1 *( (PTRK(6)/RHO1)**2 +
     +            D * PTRK(6)**2/(RW1*RW2))
      DERI(6,9) = PTRK(6)/RHO2 *( (PTRK(6)/RHO2)**2 +
     +            D * PTRK(6)**2/(RW1*RW2))

C D M / d TX
      DERI(7,4) = (D*PAR(4)/W1**2 - PAR(7))/(PTRK(7)*RW1*RW2)
      DERI(7,7) = (D*PAR(7)/W2**2 - PAR(4))/(PTRK(7)*RW1*RW2)

C D M / d TY
      DERI(7,5) = (D*PAR(5)/W1**2 - PAR(8))/(PTRK(7)*RW1*RW2)
      DERI(7,8) = (D*PAR(8)/W2**2 - PAR(5))/(PTRK(7)*RW1*RW2)

C D M / d RHO
      DERI(7,6) = (D/(RW1*RW2)
     +             -SQRT((M2**2+1./RHO2**2)/(1.+RHO1**2*M1**2))/RHO1)
     +             /(RHO1*PTRK(7))
      DERI(7,9) = (D/(RW1*RW2)
     +             -SQRT((M1**2+1./RHO1**2)/(1.+RHO2**2*M2**2))/RHO2)
     +             /(RHO2*PTRK(7))


      DO I=1,7
      DO J=I,7
       VTRK(I,J) = 0.0D0

       IF (I.LE.3) THEN
        K1=I
        K2=I
       ELSE
        K1=4
        K2=9
       ENDIF

       IF (J.LE.3) THEN
        L1=J
        L2=J
       ELSE
        L1=4
        L2=9
       ENDIF

       DO K=K1,K2
       DO L=L1,L2
        VTRK(I,J) = VTRK(I,J) + DERI(I,K)*VPAR(K,L)*DERI(J,L)
       ENDDO
       ENDDO

       VTRK(J,I) = VTRK(I,J)
      ENDDO
      ENDDO

C-----------------------------------------------------------------
C  the end
C-----------------------------------------------------------------
      END





